@@ -46,7 +46,7 @@
 #include <gsf/gsf-outfile.h>
 #include <gsf/gsf-outfile-msole.h>
 #include <gsf/gsf-structured-blob.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <string.h>
 
 GNM_PLUGIN_MODULE_HEADER;
@@ -59,7 +59,7 @@ GNM_PLUGIN_MODULE_HEADER;
  * >1 increasing levels of detail.
  */
 gint ms_excel_read_debug = 0;
-gint ms_excel_pivot_debug = 0;
+gint ms_excel_pivot_debug = 10;
 gint ms_excel_escher_debug = 0;
 gint ms_excel_formula_debug = 0;
 gint ms_excel_chart_debug = 0;
