@@ -5,7 +5,7 @@
  * Authors:
  *   Michael Meeks <michael@ximian.com>
  *   Jukka-Pekka Iivonen <iivonen@iki.fi>
- *   Morten Welinder <terra@diku.dk>
+ *   Morten Welinder <terra@gnome.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,9 @@
 #include <mathfunc.h>
 #include <collect.h>
 #include <gnm-i18n.h>
+#include <number-match.h>
+#include <workbook.h>
+#include <sheet.h>
 
 #include <goffice/app/go-plugin.h>
 #include <gnm-plugin.h>
@@ -41,63 +44,117 @@
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <errno.h>
 
 GNM_PLUGIN_MODULE_HEADER;
 
+
+typedef enum {
+	V2B_STRINGS_GENERAL = 1,        /* Allow "1/1/2000" as number.  */
+	V2B_STRINGS_0XH = 2,            /* Allow "4444h" and "0xABCD".  */
+	V2B_STRINGS_MAXLEN = 4,         /* Impose 10 character input length.  */
+	V2B_STRINGS_BLANK_ZERO = 8,     /* Treat "" as "0".  */
+	V2B_KILLME
+} Val2BaseFlags;
+
+
+
 /**
  * FIXME: In the long term this needs optimising.
  **/
 static GnmValue *
 val_to_base (FunctionEvalInfo *ei,
-	     GnmValue const * const *argv, int num_argv,
+	     GnmValue const *value,
+	     GnmValue const *aplaces,
 	     int src_base, int dest_base,
-	     gboolean relaxed)
+	     gnm_float min_value, gnm_float max_value,
+	     Val2BaseFlags flags)
 {
-	GnmValue const *value;
-	int digit, max, places;
-	char *err;
-	char const *str;
+	int digit, min, max, places;
 	gnm_float v, b10;
-	gboolean ok, had_hex_prefix = FALSE;
 	GString *buffer;
+	GnmValue *vstring = NULL;
 
 	g_return_val_if_fail (src_base > 1 && src_base <= 36,
 			      value_new_error_VALUE (ei->pos));
 	g_return_val_if_fail (dest_base > 1 && dest_base <= 36,
 			      value_new_error_VALUE (ei->pos));
 
-	value = argv[0];
-	if (VALUE_IS_EMPTY (value))
+	/* func.c ought to take care of this.  */
+	if (VALUE_IS_BOOLEAN (value))
+		return value_new_error_VALUE (ei->pos);
+	if (aplaces && VALUE_IS_BOOLEAN (aplaces))
+		return value_new_error_VALUE (ei->pos);
+
+	switch (value->type) {
+	default:
 		return value_new_error_NUM (ei->pos);
-	else if (VALUE_IS_EMPTY_OR_ERROR (value))
-		return value_dup (value);
 
-	places = (num_argv >= 2 && argv[1]) ? value_get_as_int (argv[1]) : 0;
+	case VALUE_STRING:
+		if (flags & V2B_STRINGS_GENERAL) {
+			vstring = format_match_number
+				(value_peek_string (value), NULL,
+				 workbook_date_conv (ei->pos->sheet->workbook));
+			if (!vstring || !VALUE_IS_FLOAT (vstring)) {
+				if (vstring)
+					value_release (vstring);
+				return value_new_error_VALUE (ei->pos);
+			}
+		} else {
+			char const *str = value_peek_string (value);
+			size_t len;
+			gboolean hsuffix = FALSE;
+			char *err;
 
-	str = value_peek_string (value);
-	if (relaxed) {
-		while (*str == ' ' || *str == '\t')
-			str++;
-		if (src_base == 16 &&
-		    str[0] == '0' &&
-		    (str[1] == 'x' || str[1] == 'X')) {
-			str += 2;
-			had_hex_prefix = TRUE;
+			if ((flags & V2B_STRINGS_BLANK_ZERO) && *str == 0)
+				str = "0";
+
+			/* This prevents leading spaces, signs, etc, and "".  */
+			if (!g_ascii_isalnum (*str))
+				return value_new_error_NUM (ei->pos);
+
+			len = strlen (str);
+			/* We check length in bytes.  Since we are going to
+			   require nothing but digits, that is fine.  */
+			if ((flags & V2B_STRINGS_MAXLEN) && len > 10)
+				return value_new_error_NUM (ei->pos);
+
+			if (flags & V2B_STRINGS_0XH) {
+				if (str[0] == '0' && (str[1] == 'x' || str[1] == 'X'))
+					str += 2;
+				else if (str[len - 1] == 'h' || str[len - 1] == 'H')
+					hsuffix = TRUE;
+			}
+
+			v = strtol (str, &err, src_base);
+			if (err == str || err[hsuffix] != 0)
+				return value_new_error_NUM (ei->pos);
+
+			break;
 		}
-	}
-	v = strtol (str, &err, src_base);
+		/* Fall through.  */
 
-	ok = (err != str && *err == 0);
-	if (!ok && relaxed && err != str) {
-		if (src_base == 16 &&
-		    !had_hex_prefix &&
-		    (err[0] == 'h' || err[0] == 'H') &&
-		    err[1] == 0)
-			ok = TRUE;
-	}
+	case VALUE_FLOAT: {
+		gnm_float val = gnm_fake_trunc (value_get_as_float (vstring ? vstring : value));
+		char buf[GNM_MANT_DIG + 10];
+		char *err;
 
-	if (!ok)
-		return value_new_error_NUM (ei->pos);
+		if (vstring)
+			value_release (vstring);
+
+		if (val < min_value || val > max_value)
+			return value_new_error_NUM (ei->pos);
+
+		g_ascii_formatd (buf, sizeof (buf) - 1,
+				 "%.0" GNM_FORMAT_f,
+				 val);
+
+		v = strtol (buf, &err, src_base);
+		if (*err != 0)
+			return value_new_error_NUM (ei->pos);
+		break;
+	}
+	}
 
 	b10 = gnm_pow (src_base, 10);
 	if (v >= b10 / 2) /* N's complement */
@@ -107,18 +164,26 @@ val_to_base (FunctionEvalInfo *ei,
 		return value_new_int (v);
 
 	if (v < 0) {
+		min = 1;
 		max = 10;
 		v += gnm_pow (dest_base, max);
 	} else {
 		if (v == 0)
-			max = 1;
+			min = max = 1;
 		else
-			max = (int)(gnm_log (v + 0.5) /
-				    gnm_log (dest_base)) + 1;
+			min = max = (int)(gnm_log (v + 0.5) /
+					  gnm_log (dest_base)) + 1;
 	}
 
-	if (places > max)
-		max = places;
+	if (aplaces) {
+		gnm_float fplaces = value_get_as_float (aplaces);
+		if (fplaces < min || fplaces > 10)
+			return value_new_error_NUM (ei->pos);
+		places = (int)fplaces;
+		if (v >= 0 && places > max)
+			max = places;
+	} else
+		places = 1;
 
 	buffer = g_string_sized_new (max);
 	g_string_set_size (buffer, max);
@@ -160,15 +225,15 @@ static GnmFuncHelp const help_base[] = {
 static GnmValue *
 gnumeric_base (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	GnmValue const *argv2[2];
-	int base = value_get_as_int (argv[1]);
+	static const gnm_float max = 1 / GNM_EPSILON;
+	gnm_float base = value_get_as_float (argv[1]);
 
-	if (base < 2 || base > 36)
+	if (base < 2 || base >= 37)
 		return value_new_error_NUM (ei->pos);
 
-	argv2[0] = argv[0];
-	argv2[1] = argv[2];
-	return val_to_base (ei, argv2, 2, 10, base, FALSE);
+	return val_to_base (ei, argv[0], argv[2], 10, (int)base,
+			    -max, +max,
+			    V2B_STRINGS_GENERAL | V2B_STRINGS_0XH);
 }
 
 /***************************************************************************/
@@ -194,7 +259,10 @@ static GnmFuncHelp const help_bin2dec[] = {
 static GnmValue *
 gnumeric_bin2dec (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	return val_to_base (ei, argv, 1, 2, 10, FALSE);
+	return val_to_base (ei, argv[0], NULL,
+			    2, 10,
+			    0, GNM_const(1111111111.0),
+			    V2B_STRINGS_MAXLEN | V2B_STRINGS_BLANK_ZERO);
 }
 
 /***************************************************************************/
@@ -223,7 +291,10 @@ static GnmFuncHelp const help_bin2oct[] = {
 static GnmValue *
 gnumeric_bin2oct (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	return val_to_base (ei, argv, 2, 2, 8, FALSE);
+	return val_to_base (ei, argv[0], argv[1],
+			    2, 8,
+			    0, GNM_const(1111111111.0),
+			    V2B_STRINGS_MAXLEN | V2B_STRINGS_BLANK_ZERO);
 }
 
 /***************************************************************************/
@@ -252,7 +323,10 @@ static GnmFuncHelp const help_bin2hex[] = {
 static GnmValue *
 gnumeric_bin2hex (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	return val_to_base (ei, argv, 2, 2, 16, FALSE);
+	return val_to_base (ei, argv[0], argv[1],
+			    2, 16,
+			    0, GNM_const(1111111111.0),
+			    V2B_STRINGS_MAXLEN | V2B_STRINGS_BLANK_ZERO);
 }
 
 /***************************************************************************/
@@ -281,7 +355,10 @@ static GnmFuncHelp const help_dec2bin[] = {
 static GnmValue *
 gnumeric_dec2bin (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	return val_to_base (ei, argv, 2, 10, 2, FALSE);
+	return val_to_base (ei, argv[0], argv[1],
+			    10, 2,
+			    -512, 511,
+			    V2B_STRINGS_GENERAL);
 }
 
 /***************************************************************************/
@@ -310,7 +387,10 @@ static GnmFuncHelp const help_dec2oct[] = {
 static GnmValue *
 gnumeric_dec2oct (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	return val_to_base (ei, argv, 2, 10, 8, FALSE);
+	return val_to_base (ei, argv[0], argv[1],
+			    10, 8,
+			    -536870912, 536870911,
+			    V2B_STRINGS_GENERAL);
 }
 
 /***************************************************************************/
@@ -339,7 +419,10 @@ static GnmFuncHelp const help_dec2hex[] = {
 static GnmValue *
 gnumeric_dec2hex (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	return val_to_base (ei, argv, 2, 10, 16, FALSE);
+	return val_to_base (ei, argv[0], argv[1],
+			    10, 16,
+			    GNM_const(-549755813888.0), GNM_const(549755813887.0),
+			    V2B_STRINGS_GENERAL);
 }
 
 /***************************************************************************/
@@ -366,9 +449,9 @@ static GnmFuncHelp const help_decimal[] = {
 static GnmValue *
 gnumeric_decimal (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	int base = value_get_as_int (argv[1]);
+	gnm_float base = value_get_as_float (argv[1]);
 
-	if (base < 2 || base > 36)
+	if (base < 2 || base >= 37)
 		return value_new_error_NUM (ei->pos);
 
 	return value_new_error_NUM (ei->pos);
@@ -397,7 +480,10 @@ static GnmFuncHelp const help_oct2dec[] = {
 static GnmValue *
 gnumeric_oct2dec (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	return val_to_base (ei, argv, 1, 8, 10, FALSE);
+	return val_to_base (ei, argv[0], NULL,
+			    8, 10,
+			    0, GNM_const(7777777777.0),
+			    V2B_STRINGS_MAXLEN | V2B_STRINGS_BLANK_ZERO);
 }
 
 /***************************************************************************/
@@ -426,7 +512,10 @@ static GnmFuncHelp const help_oct2bin[] = {
 static GnmValue *
 gnumeric_oct2bin (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	return val_to_base (ei, argv, 2, 8, 2, FALSE);
+	return val_to_base (ei, argv[0], argv[1],
+			    8, 2,
+			    0, GNM_const(7777777777.0),
+			    V2B_STRINGS_MAXLEN | V2B_STRINGS_BLANK_ZERO);
 }
 
 /***************************************************************************/
@@ -455,7 +544,10 @@ static GnmFuncHelp const help_oct2hex[] = {
 static GnmValue *
 gnumeric_oct2hex (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	return val_to_base (ei, argv, 2, 8, 16, FALSE);
+	return val_to_base (ei, argv[0], argv[1],
+			    8, 16,
+			    0, GNM_const(7777777777.0),
+			    V2B_STRINGS_MAXLEN | V2B_STRINGS_BLANK_ZERO);
 }
 
 /***************************************************************************/
@@ -484,7 +576,10 @@ static GnmFuncHelp const help_hex2bin[] = {
 static GnmValue *
 gnumeric_hex2bin (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	return val_to_base (ei, argv, 2, 16, 2, FALSE);
+	return val_to_base (ei, argv[0], argv[1],
+			    16, 2,
+			    0, GNM_const(9999999999.0),
+			    V2B_STRINGS_MAXLEN | V2B_STRINGS_BLANK_ZERO);
 }
 
 /***************************************************************************/
@@ -513,7 +608,10 @@ static GnmFuncHelp const help_hex2oct[] = {
 static GnmValue *
 gnumeric_hex2oct (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	return val_to_base (ei, argv, 2, 16, 8, FALSE);
+	return val_to_base (ei, argv[0], argv[1],
+			    16, 8,
+			    0, GNM_const(9999999999.0),
+			    V2B_STRINGS_MAXLEN | V2B_STRINGS_BLANK_ZERO);
 }
 
 /***************************************************************************/
@@ -539,7 +637,10 @@ static GnmFuncHelp const help_hex2dec[] = {
 static GnmValue *
 gnumeric_hex2dec (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	return val_to_base (ei, argv, 1, 16, 10, FALSE);
+	return val_to_base (ei, argv[0], NULL,
+			    16, 10,
+			    0, GNM_const(9999999999.0),
+			    V2B_STRINGS_MAXLEN | V2B_STRINGS_BLANK_ZERO);
 }
 
 /***************************************************************************/
@@ -572,14 +673,16 @@ static GnmFuncHelp const help_besseli[] = {
 static GnmValue *
 gnumeric_besseli (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	gnm_float x, order;
-
-	x = value_get_as_float (argv[0]);	/* value to evaluate I_n at. */
-	order = value_get_as_float (argv[1]);	/* the order */
+	gnm_float x = value_get_as_float (argv[0]);	/* value to evaluate I_n at. */
+	gnm_float order = value_get_as_float (argv[1]);	/* the order */
 
 	if (order < 0)
 		return value_new_error_NUM (ei->pos);
 
+	/* It seems that XL uses abs value of x */
+	if (x < 0.)
+		x = -x;
+
 	return value_new_float (bessel_i (x, order, 1.0));
 }
 
@@ -612,10 +715,8 @@ static GnmFuncHelp const help_besselk[] = {
 static GnmValue *
 gnumeric_besselk (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	gnm_float x, order;
-
-	x = value_get_as_float (argv[0]);	/* value to evaluate K_n at. */
-	order = value_get_as_float (argv[1]);	/* the order */
+	gnm_float x = value_get_as_float (argv[0]);	/* value to evaluate K_n at. */
+	gnm_float order = value_get_as_float (argv[1]);	/* the order */
 
 	if (order < 0)
 		return value_new_error_NUM (ei->pos);
@@ -651,15 +752,14 @@ static GnmFuncHelp const help_besselj[] = {
 static GnmValue *
 gnumeric_besselj (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	int x, y;
+	gnm_float x = value_get_as_float (argv[0]);
+	gnm_float y = value_get_as_float (argv[1]);
 
-	x = value_get_as_int (argv[0]);
-	y = value_get_as_int (argv[1]);
-
-	if (y < 0)
+	if (y < 0 || y > INT_MAX)
 		return value_new_error_NUM (ei->pos);
 
-	return value_new_float (jn (y, value_get_as_float (argv[0])));
+	/* FIXME: Why not gnm_jn?  */
+	return value_new_float (jn ((int)y, x));
 }
 
 /***************************************************************************/
@@ -691,17 +791,13 @@ static GnmFuncHelp const help_bessely[] = {
 static GnmValue *
 gnumeric_bessely (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	int y;
-	if (argv[0]->type != VALUE_INTEGER &&
-	    argv[1]->type != VALUE_INTEGER &&
-	    argv[0]->type != VALUE_FLOAT &&
-	    argv[1]->type != VALUE_FLOAT)
-		return value_new_error_VALUE (ei->pos);
+	gnm_float x = value_get_as_float (argv[0]);
+	gnm_float y = value_get_as_float (argv[1]);
 
-	if ((y = value_get_as_int (argv[1])) < 0)
+	if (y < 0 || y > INT_MAX)
 		return value_new_error_NUM (ei->pos);
 
-	return value_new_float (gnm_yn (y, value_get_as_float (argv[0])));
+	return value_new_float (gnm_yn ((int)y, x));
 }
 
 /***************************************************************************/
@@ -803,7 +899,7 @@ static GnmFuncHelp const help_convert[] = {
 	   "\t'y'  \tyocto \t\t1E-24\n"
 	   "\n"
 	   "* If @from_unit and @to_unit are different types, CONVERT returns "
-	   "#NUM! error.\n"
+	   "#N/A error.\n"
 	   "* This function is Excel compatible.\n"
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -856,11 +952,11 @@ get_constant_of_unit(const eng_convert_unit_t units[],
 
 /* See also http://physics.nist.gov/cuu/Units/prefixes.html */
 
-static GnmValue *
-convert (const eng_convert_unit_t units[],
-	 const eng_convert_unit_t prefixes[],
+static gboolean
+convert (eng_convert_unit_t const units[],
+	 eng_convert_unit_t const prefixes[],
 	 char const *from_unit, char const *to_unit,
-	 gnm_float n, GnmValue **v, const GnmEvalPos *ep)
+	 gnm_float n, GnmValue **v, GnmEvalPos const *ep)
 {
         gnm_float from_c, from_prefix, to_c, to_prefix;
 
@@ -869,17 +965,16 @@ convert (const eng_convert_unit_t units[],
 
 	        if (!get_constant_of_unit (units, prefixes,
 					   to_unit, &to_c, &to_prefix))
-			return value_new_error_NUM (ep);
-
-	        if (from_c == 0 || to_prefix == 0)
-	                return value_new_error_NUM (ep);
-
-		*v = value_new_float (((n * from_prefix) / from_c) *
-				 to_c / to_prefix);
-		return *v;
+			*v = value_new_error_NUM (ep);
+	        else if (from_c == 0 || to_prefix == 0)
+	                *v = value_new_error_NUM (ep);
+		else
+			*v = value_new_float (((n * from_prefix) / from_c) *
+					      to_c / to_prefix);
+		return TRUE;
 	}
 
-	return NULL;
+	return FALSE;
 }
 
 static GnmValue *
@@ -1123,7 +1218,7 @@ gnumeric_convert (FunctionEvalInfo *ei, GnmValue const * const *argv)
 		    ei->pos))
 	        return v;
 
-	return value_new_error_NUM (ei->pos);
+	return value_new_error_NA (ei->pos);
 }
 
 /***************************************************************************/
@@ -1169,6 +1264,7 @@ gnumeric_erf (FunctionEvalInfo *ei, GnmValue const * const *argv)
 	ans = gnm_erf (lower);
 
 	if (argv[1]) {
+		/* FIXME: This is suboptimal.  */
 		upper = value_get_as_float (argv[1]);
 		ans = gnm_erf (upper) - ans;
 	}
@@ -1202,9 +1298,7 @@ static GnmFuncHelp const help_erfc[] = {
 static GnmValue *
 gnumeric_erfc (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	gnm_float x;
-
-	x = value_get_as_float (argv[0]);
+	gnm_float x = value_get_as_float (argv[0]);
 
 	return value_new_float (gnm_erfc (x));
 }
@@ -1235,36 +1329,10 @@ static GnmFuncHelp const help_delta[] = {
 static GnmValue *
 gnumeric_delta (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	GnmValue *err = NULL;
-	gboolean ans = FALSE;
-	GnmValue const *vx, *vy;
+	gnm_float x = value_get_as_float (argv[0]);
+	gnm_float y = argv[1] ? value_get_as_float (argv[1]) : 0;
 
-	vx = argv[0];
-	if (argv[1])
-		vy = argv[1];
-	else
-		vy = value_new_int (0);
-
-	/* Promote to the largest value */
-	switch ((vx->type > vy->type) ? vx->type : vy->type) {
-	case VALUE_BOOLEAN:
-		/* Only happens when both are bool */
-		ans = vx->v_bool.val == vy->v_bool.val;
-		break;
-	case VALUE_EMPTY:
-	case VALUE_INTEGER:
-		ans = value_get_as_int (vx) == value_get_as_int (vy);
-		break;
-	case VALUE_FLOAT:
-		ans = value_get_as_float (vx) == value_get_as_float (vy);
-		break;
-	default:
-		err = value_new_error (ei->pos, _("Impossible"));
-	}
-	if (!argv[1])
-		value_release ((GnmValue *)vy);
-
-	return (err != NULL) ? err : value_new_int (ans ? 1 : 0);
+	return value_new_int (x == y);
 }
 
 /***************************************************************************/
@@ -1274,7 +1342,7 @@ static GnmFuncHelp const help_gestep[] = {
 	F_("@FUNCTION=GESTEP\n"
 	   "@SYNTAX=GESTEP(x[,y])\n"
 	   "@DESCRIPTION="
-	   "GESTEP function test for if @x is >= @y, returning 1 if it "
+	   "GESTEP function tests if @x is >= @y, returning 1 if it "
 	   "is so, and 0 otherwise. @y is optional, and defaults to 0.\n"
 	   "\n"
 	   "* If either argument is non-numeric returns a #VALUE! error.\n"
@@ -1292,36 +1360,10 @@ static GnmFuncHelp const help_gestep[] = {
 static GnmValue *
 gnumeric_gestep (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	GnmValue *err = NULL;
-	gboolean ans = FALSE;
-	GnmValue const *vx, *vy;
+	gnm_float x = value_get_as_float (argv[0]);
+	gnm_float y = argv[1] ? value_get_as_float (argv[1]) : 0;
 
-	vx = argv[0];
-	if (argv[1])
-		vy = argv[1];
-	else
-		vy = value_new_int (0);
-
-	/* Promote to the largest value */
-	switch ((vx->type > vy->type) ? vx->type : vy->type) {
-	case VALUE_BOOLEAN:
-		/* Only happens when both are bool */
-		ans = vx->v_bool.val >= vy->v_bool.val;
-		break;
-	case VALUE_EMPTY:
-	case VALUE_INTEGER:
-		ans = value_get_as_int (vx) >= value_get_as_int (vy);
-		break;
-	case VALUE_FLOAT:
-		ans = value_get_as_float (vx) >= value_get_as_float (vy);
-		break;
-	default:
-		err = value_new_error (ei->pos, _("Impossible"));
-	}
-
-	if (!argv[1])
-		value_release ((GnmValue *)vy);
-	return (err != NULL) ? err : value_new_int (ans ? 1 : 0);
+	return value_new_int (x >= y);
 }
 
 /***************************************************************************/
@@ -1370,9 +1412,9 @@ range_invsuminv (gnm_float const *xs, int n, gnm_float *res)
 }
 
 static GnmValue *
-gnumeric_invsuminv (FunctionEvalInfo *ei, GnmExprList const *nodes)
+gnumeric_invsuminv (FunctionEvalInfo *ei, int argc, GnmExprConstPtr const *argv)
 {
-	return float_range_function (nodes, ei,
+	return float_range_function (argc, argv, ei,
 				     range_invsuminv,
 				     COLLECT_IGNORE_STRINGS |
 				     COLLECT_IGNORE_BOOLS |
@@ -1382,7 +1424,7 @@ gnumeric_invsuminv (FunctionEvalInfo *ei, GnmExprList const *nodes)
 
 /***************************************************************************/
 
-const GnmFuncDescriptor engineering_functions[] = {
+GnmFuncDescriptor const engineering_functions[] = {
         { "base",     "Sf|f",   "text,base,length", help_base,
 	  gnumeric_base, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
