@@ -6,7 +6,7 @@
  * Authors:
  *   Miguel de Icaza (miguel@gnu.org)
  *   Jukka-Pekka Iivonen (iivonen@iki.fi)
- *   Morten Welinder (terra@diku.dk)
+ *   Morten Welinder (terra@gnome.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -139,9 +139,10 @@ typedef struct {
 } randdiscrete_t;
 
 static GnmValue *
-cb_randdiscrete (Sheet *sheet, int col, int row, GnmCell *cell, void *user_data)
+cb_randdiscrete (GnmCellIter const *iter, gpointer user)
 {
-	randdiscrete_t *p = (randdiscrete_t *) user_data;
+	randdiscrete_t *p = (randdiscrete_t *) user;
+	GnmCell *cell = iter->cell;
 
 	if (p->res != NULL)
 		return NULL;
@@ -318,12 +319,12 @@ static GnmValue *
 gnumeric_randbinom (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float p      = value_get_as_float (argv[0]);
-	int        trials = value_get_as_int (argv[1]);
+	gnm_float trials = value_get_as_float (argv[1]);
 
 	if (p < 0 || p > 1 || trials < 0)
 		return value_new_error_NUM (ei->pos);
 
-        return value_new_float (random_binomial (p, trials));
+        return value_new_float (random_binomial (p, gnm_floor (trials)));
 }
 
 /***************************************************************************/
@@ -337,7 +338,8 @@ static GnmFuncHelp const help_randbetween[] = {
 	   "RANDBETWEEN function returns a random integer number "
 	   "between and including @bottom and @top.\n"
            "\n"
-	   "* If @bottom or @top is non-integer, they are truncated.\n"
+	   "* If @bottom is non-integer, it is rounded up.\n"
+	   "* If @top is non-integer, it is rounded down.\n"
 	   "* If @bottom > @top, RANDBETWEEN returns #NUM! error.\n"
 	   "* This function is Excel compatible.\n"
 	   "\n"
@@ -352,16 +354,17 @@ static GnmFuncHelp const help_randbetween[] = {
 static GnmValue *
 gnumeric_randbetween (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-        int bottom, top;
-	gnm_float r;
+	gnm_float bottom = value_get_as_float (argv[0]);
+	gnm_float top = value_get_as_float (argv[1]);
 
-	bottom = value_get_as_int (argv[0]);
-	top    = value_get_as_int (argv[1]);
 	if (bottom > top)
 		return value_new_error_NUM (ei->pos);
 
-	r = bottom + gnm_floor ((top + 1.0 - bottom) * random_01 ());
-	return value_new_int ((int)r);
+	/* Bottom is ceiled, top floored.  Neither is truncated.  */
+	bottom = gnm_ceil (bottom);
+	top = gnm_floor (top);
+
+	return value_new_float (bottom + gnm_floor ((top + 1.0 - bottom) * random_01 ()));
 }
 
 /***************************************************************************/
@@ -390,12 +393,12 @@ static GnmValue *
 gnumeric_randnegbinom (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float p = value_get_as_float (argv[0]);
-	int failures = value_get_as_int (argv[1]);
+	gnm_float failures = value_get_as_float (argv[1]);
 
-	if (p < 0 || p > 1 || failures < 0)
+	if (p < 0 || p > 1 || failures < 1)
 		return value_new_error_NUM (ei->pos);
 
-        return value_new_float (random_negbinom (p, failures));
+        return value_new_float (random_negbinom (p, gnm_floor (failures)));
 }
 
 /***************************************************************************/
@@ -806,7 +809,7 @@ gnumeric_randgeom (FunctionEvalInfo *ei, GnmValue const * const *argv)
 	if (p < 0 || p > 1)
 		return value_new_error_NUM (ei->pos);
 
-        return value_new_int (random_geometric (p));
+        return value_new_float (random_geometric (p));
 }
 
 /***************************************************************************/
@@ -835,11 +838,13 @@ static GnmFuncHelp const help_randhyperg[] = {
 static GnmValue *
 gnumeric_randhyperg (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
-	unsigned int n1 = value_get_as_int (argv[0]);
-	unsigned int n2 = value_get_as_int (argv[1]);
-	unsigned int t = value_get_as_int (argv[2]);
+	gnm_float n1 = value_get_as_float (argv[0]);
+	gnm_float n2 = value_get_as_float (argv[1]);
+	gnm_float t = value_get_as_float (argv[2]);
 
-        return value_new_int (random_hypergeometric (n1, n2, t));
+        return value_new_float (random_hypergeometric (gnm_floor (n1),
+						       gnm_floor (n2),
+						       gnm_floor (t)));
 }
 
 /***************************************************************************/
@@ -870,7 +875,7 @@ gnumeric_randlog (FunctionEvalInfo *ei, GnmValue const * const *argv)
 	if (p < 0 || p > 1)
 		return value_new_error_NUM (ei->pos);
 
-        return value_new_int (random_logarithmic (p));
+        return value_new_float (random_logarithmic (p));
 }
 
 /***************************************************************************/
@@ -953,15 +958,14 @@ gnumeric_randgumbel (FunctionEvalInfo *ei, GnmValue const * const *argv)
 {
 	gnm_float a = value_get_as_float (argv[0]);
 	gnm_float b = value_get_as_float (argv[1]);
-	int type     = (argv[2] == NULL) ? 1 : value_get_as_int (argv[2]);
-
-	if (type != 1 && type != 2)
-		return value_new_error_NUM (ei->pos);
+	gnm_float type = argv[2] ? value_get_as_float (argv[2]) : 1;
 
 	if (type == 1)
 		return value_new_float (random_gumbel1 (a, b));
-	else
+	else if (type == 2)
 		return value_new_float (random_gumbel2 (a, b));
+	else
+		return value_new_error_NUM (ei->pos);
 }
 
 /***************************************************************************/
@@ -1166,7 +1170,7 @@ callback_function_simtable (GnmEvalPos const *ep, GnmValue const *value, void *c
 }
 
 static GnmValue *
-gnumeric_simtable (FunctionEvalInfo *ei, GnmExprList const *nodes)
+gnumeric_simtable (FunctionEvalInfo *ei, int argc, GnmExprConstPtr const *argv)
 {
 	simtable_t p;
 
@@ -1174,8 +1178,8 @@ gnumeric_simtable (FunctionEvalInfo *ei, GnmExprList const *nodes)
 	p.value = NULL;
 
 	function_iterate_argument_values
-		(ei->pos, callback_function_simtable, &p, nodes,
-		 FALSE, CELL_ITER_IGNORE_BLANK);
+		(ei->pos, callback_function_simtable, &p,
+		 argc, argv, FALSE, CELL_ITER_IGNORE_BLANK);
 
 	/* See if there was any value worth using. */
 	if (p.value == NULL)
@@ -1186,7 +1190,7 @@ gnumeric_simtable (FunctionEvalInfo *ei, GnmExprList const *nodes)
 
 /***************************************************************************/
 
-const GnmFuncDescriptor random_functions[] = {
+GnmFuncDescriptor const random_functions[] = {
 	{ "rand",    "", "",           help_rand,
 	  gnumeric_rand, NULL, NULL, NULL, NULL,
 	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
