@@ -28,7 +28,7 @@
  */
 
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gnumeric.h>
 #include <string.h>
 #include "html.h"
@@ -46,14 +46,15 @@
 #include <ranges.h>
 #include <goffice/app/io-context.h>
 #include <goffice/app/error-info.h>
+#include <goffice/utils/go-glib-extras.h>
 
 #include <gsf/gsf-input.h>
 #include <libxml/HTMLparser.h>
 #include <libxml/HTMLtree.h>
 
-#define CC2XML(s) ((const xmlChar *)(s))
+#define CC2XML(s) ((xmlChar const *)(s))
 #define C2XML(s) ((xmlChar *)(s))
-#define CXML2C(s) ((const char *)(s))
+#define CXML2C(s) ((char const *)(s))
 #define XML2C(s) ((char *)(s))
 
 typedef struct {
@@ -73,7 +74,7 @@ html_get_sheet (char const *name, Workbook *wb)
 			workbook_sheet_attach (wb, sheet);
 		}
 	} else
-		sheet = workbook_sheet_add (wb, -1, FALSE);
+		sheet = workbook_sheet_add (wb, -1);
 	return sheet;
 }
 
@@ -302,7 +303,7 @@ html_read_table (htmlNodePtr cur, htmlDocPtr doc, WorkbookView *wb_view,
 	g_return_if_fail (cur != NULL);
 	g_return_if_fail (wb_view != NULL);
 
-	wb = wb_view_workbook (wb_view);
+	wb = wb_view_get_workbook (wb_view);
 	for (ptr = cur->children; ptr != NULL ; ptr = ptr->next) {
 		if (ptr->type != XML_ELEMENT_NODE)
 			continue;
@@ -331,7 +332,7 @@ html_read_table (htmlNodePtr cur, htmlDocPtr doc, WorkbookView *wb_view,
 }
 
 /* Element types which imply that we are inside a table */
-static const char *table_start_elt_types[] = {
+static char const *table_start_elt_types[] = {
 	"caption",
 	"col",
 	"colgroup",
@@ -343,23 +344,23 @@ static const char *table_start_elt_types[] = {
 };
 
 /* Element types which imply that we are inside a row */
-static const char *row_start_elt_types[] = {
+static char const *row_start_elt_types[] = {
 	"td",
 	"th",
 	NULL
 };
 
 /* Element types which occur inside tables and rows, but also outside */
-static const char *cont_elt_types[] = {
+static char const *cont_elt_types[] = {
 	"del",
 	"ins",
 	NULL
 };
 
 static gboolean 
-is_elt_type (htmlNodePtr ptr, const char** types)
+is_elt_type (htmlNodePtr ptr, char const ** types)
 {
-	const char **p;
+	char const **p;
 	gboolean ret = FALSE;
 
 	for (p = types; *p; p++)
@@ -476,6 +477,9 @@ html_file_open (GOFileOpener const *fo, IOContext *io_context,
 
 	g_return_if_fail (input != NULL);
 
+	if (gsf_input_seek (input, 0, G_SEEK_SET))
+		return;
+
 	size = gsf_input_size (input);
 	if (size >= 4) {
 		size -= 4;
@@ -514,7 +518,7 @@ html_file_open (GOFileOpener const *fo, IOContext *io_context,
 				bomlen = 0;
 			}
 			ctxt = htmlCreatePushParserCtxt (
-				NULL, NULL, (const char *)(buf + bomlen),
+				NULL, NULL, (char const *)(buf + bomlen),
 				4 - bomlen, gsf_input_name (input), enc);
 			
 			for (; size > 0 ; size -= len) {
@@ -523,10 +527,10 @@ html_file_open (GOFileOpener const *fo, IOContext *io_context,
 				if (buf == NULL)
 					break;
 				htmlParseChunk (
-					ctxt, (const char *)buf, len, 0);
+					ctxt, (char const *)buf, len, 0);
 			}
 
-			htmlParseChunk (ctxt, (const char *)buf, 0, 1);
+			htmlParseChunk (ctxt, (char const *)buf, 0, 1);
 			doc = ctxt->myDoc;
 			htmlFreeParserCtxt (ctxt);
 		}
@@ -543,3 +547,49 @@ html_file_open (GOFileOpener const *fo, IOContext *io_context,
 		gnumeric_io_error_info_set (io_context,
 			error_info_new_str (_("Unable to parse the html.")));
 }
+
+/* Quick and dirty html probe. */
+gboolean 
+html_file_probe (GOFileOpener const *fo, GsfInput *input, FileProbeLevel pl)
+{
+	gsf_off_t size = 200;
+	guint8 const* buf = gsf_input_read (input, size, NULL);
+	gchar *ulstr = NULL;
+	gboolean res = FALSE;
+	int try;
+
+	/* Avoid seeking in large streams - try to read, fall back if 
+	 * stream is too short.  (Actually, currently _size does not
+	 * involve any syscalls -- MW).  */
+	if (!buf) {
+		size = gsf_input_size (input);
+		buf = gsf_input_read (input, size, NULL);
+		if (!buf)
+			return res;
+	}
+
+	/*
+	 * It is conceivable that encoding guessing could fail
+	 * if our truncated buffer had partial characters.  We
+	 * really need go_guess_encoding_truncated, but for now
+	 * let's just try cutting a byte away at a time.
+	 */
+	for (try = 0; try < MIN (size, 6); try++) {
+		char *ustr;
+		if (go_guess_encoding (buf, size - try, NULL, &ustr)) {
+			ulstr = g_utf8_strdown (ustr, -1);
+			g_free (ustr);
+			break;
+		}
+	}
+	if (!ulstr)
+		return res;
+
+	res = (strstr (ulstr, "<table") != NULL ||
+	       strstr (ulstr, "<html") != NULL ||
+	       strstr (ulstr, "<!doctype html") != NULL);
+
+	g_free (ulstr);
+
+	return res;
+}
