@@ -17,7 +17,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -25,7 +25,7 @@
 
 /* TODO: Limit the number of include files a bit */
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gnumeric.h>
 #include "psiconv-plugin.h"
 #include "application.h"
@@ -174,13 +174,13 @@ set_format(GnmStyle *style, const psiconv_sheet_numberformat psi_numberformat)
 	}  /* TODO: Add True/False */
 
 	if (fmt_string[0])
-		mstyle_set_format_text (style, fmt_string);
+		gnm_style_set_format_text (style, fmt_string);
 }
 
 static GnmColor *
 get_color(const psiconv_color color)
 {
-	return style_color_new(color->red*256,color->green*256,color->blue*256);
+	return style_color_new_i8 (color->red, color->green, color->blue);
 }
 
 static void
@@ -189,28 +189,28 @@ set_layout(GnmStyle * style,const psiconv_sheet_cell_layout psi_layout)
 	GnmColor *color;
 
 	set_format(style,psi_layout->numberformat);
-	mstyle_set_font_size(style,psi_layout->character->font_size);
-	mstyle_set_font_italic(style,psi_layout->character->italic?TRUE:FALSE);
-	mstyle_set_font_bold(style,psi_layout->character->bold?TRUE:FALSE);
-	mstyle_set_font_uline(style,
+	gnm_style_set_font_size(style,psi_layout->character->font_size);
+	gnm_style_set_font_italic(style,psi_layout->character->italic?TRUE:FALSE);
+	gnm_style_set_font_bold(style,psi_layout->character->bold?TRUE:FALSE);
+	gnm_style_set_font_uline(style,
 	                      psi_layout->character->underline?TRUE:FALSE);
-	mstyle_set_font_strike(style,
+	gnm_style_set_font_strike(style,
 	                       psi_layout->character->strikethrough?TRUE:FALSE);
-	mstyle_set_font_name(style,
+	gnm_style_set_font_name(style,
 			     (const char *) psi_layout->character->font->name);
 	color = get_color(psi_layout->character->color);
 	if (color)
-		mstyle_set_color(style,MSTYLE_COLOR_FORE,color);
+		gnm_style_set_font_color (style, color);
 	/* TODO: Character level layouts: super_sub */
 	/* TODO: Paragraph level layouts: all */
 	/* TODO: Background color: add transparant if white */
 #if 0
 	color = get_color(psi_layout->paragraph->back_color);
 	if (color) {
-		mstyle_set_color(style,MSTYLE_COLOR_BACK,color);
-		mstyle_set_color(style,MSTYLE_COLOR_PATTERN,color);
+		gnm_style_set_back_color(style, color);
+		gnm_style_set_pattern_color(style, color);
 		/* TODO: Replace 24 with some symbol */
-		mstyle_set_pattern(style,1);
+		gnm_style_set_pattern(style,1);
 	}
 #endif
 }
@@ -221,8 +221,7 @@ set_style(Sheet *sheet, int row, int col,
           const psiconv_sheet_cell_layout psi_layout,
           const GnmStyle *default_style)
 {
-	GnmStyle *style;
-	style = mstyle_copy(default_style);
+	GnmStyle *style = gnm_style_dup(default_style);
 	if (!style)
 		return;
 	set_layout(style,psi_layout);
@@ -363,11 +362,11 @@ parse_subexpr(const psiconv_formula psi_formula)
 		if (nrargs >= 2) {
 			if (!(psi_form2 = psiconv_list_get
 			                  (psi_formula->data.fun_operands,1))) {
-				gnm_expr_unref(expr1);
+				gnm_expr_free(expr1);
 				return NULL;
 			}
 			if (!(expr2 = parse_subexpr(psi_form2))) {
-				gnm_expr_unref(expr1);
+				gnm_expr_free(expr1);
 				return NULL;
 			}
 		}
@@ -397,8 +396,8 @@ parse_subexpr(const psiconv_formula psi_formula)
 		case psiconv_formula_op_neg:
 			return gnm_expr_new_unary (GNM_EXPR_OP_UNARY_NEG,expr1);
 		default:
-			gnm_expr_unref(expr1);
-			gnm_expr_unref(expr2);
+			gnm_expr_free(expr1);
+			gnm_expr_free(expr2);
 			return NULL;
 		}
 	} else if (kind == 3) {
@@ -473,7 +472,7 @@ add_cell (Sheet *sheet, const psiconv_sheet_cell psi_cell,
 		g_warning ("Cell with no value or expression ?");
 	}
 	if (expr)
-		gnm_expr_unref (expr);
+		gnm_expr_free (expr);
 
 	/* TODO: Perhaps this must be moved above set_format */
 	set_style(sheet,psi_cell->row,psi_cell->column,psi_cell->layout,
@@ -514,7 +513,7 @@ add_worksheet(Workbook *wb, psiconv_sheet_worksheet psi_worksheet,int nr,
 		return;
 
 	/* Default layout */
-	default_style = mstyle_new_default();
+	default_style = gnm_style_new_default();
 	if (!default_style) {
 		g_object_unref (sheet);
 		return;
@@ -528,7 +527,7 @@ add_worksheet(Workbook *wb, psiconv_sheet_worksheet psi_worksheet,int nr,
 	/* TODO: What about the NULL? */
 	sheet_flag_recompute_spans(sheet);
 	workbook_sheet_attach (wb, sheet);
-	mstyle_unref (default_style);
+	gnm_style_unref (default_style);
 }
 
 static void
@@ -659,4 +658,3 @@ out:
 		psiconv_buffer_free(buf);
 	return res;
 }
-
