@@ -7,7 +7,7 @@
 
 #include <gnumeric-config.h>
 #include <Python.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gnumeric.h>
 #include <glib.h>
 #include "pygobject.h"
