@@ -1,3 +1,4 @@
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * sc.c - file import of SC/xspread files
  * Copyright 1999 Jeff Garzik <jgarzik@mandrakesoft.com>
@@ -6,7 +7,7 @@
  */
 
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gnumeric.h>
 #include <string.h>
 #include <errno.h>
@@ -15,6 +16,8 @@
 #include <gnm-plugin.h>
 #include <goffice/app/io-context.h>
 #include <goffice/app/error-info.h>
+#include <goffice/utils/go-glib-extras.h>
+#include "sheet-style.h"
 #include "workbook-view.h"
 #include "workbook.h"
 #include "parse-util.h"
@@ -24,6 +27,7 @@
 #include "style.h"
 #include "sheet.h"
 #include "expr.h"
+#include "func.h"
 
 #include <gsf/gsf-input.h>
 #include <gsf/gsf-input-textline.h>
@@ -38,6 +42,14 @@ gboolean sc_file_probe (GOFileOpener const *fo, GsfInput *input,
 void sc_file_open (GOFileOpener const *fo, IOContext *io_context,
                    WorkbookView *wb_view, GsfInput *input);
 
+typedef struct {
+	GsfInputTextline   *textline;
+	Sheet              *sheet;
+	GIConv             converter;
+	GnmExprConventions *exprconv;
+} ScParseState;
+
+
 typedef enum {
 	LABEL,
 	LEFTSTRING,
@@ -47,13 +59,11 @@ typedef enum {
 
 /* we can't use cellpos_parse b/c it doesn't support 0 bases (A0, B0, ...) */
 static gboolean
-sc_cellname_to_coords (char const *cellname, int *col, int *row)
+sc_cellname_to_coords (char const *cellname, GnmCellPos *pos)
 {
 	int mult;
 
 	g_return_val_if_fail (cellname, FALSE);
-	g_return_val_if_fail (col, FALSE);
-	g_return_val_if_fail (row, FALSE);
 
 	if (!cellname || !*cellname || !g_ascii_isalpha (*cellname))
 		goto err_out;
@@ -68,12 +78,12 @@ sc_cellname_to_coords (char const *cellname, int *col, int *row)
 		int ofs = g_ascii_toupper (*cellname) - 'A';
 		if (ofs < 0 || ofs > 25)
 			goto err_out;
-		*col = ((mult + 1) * 26) + ofs;
+		pos->col = ((mult + 1) * 26) + ofs;
 		cellname++;
 	}
 
 	else {
-		*col = mult;
+		pos->col = mult;
 	}
 
 	/* XXX need to replace this block with strtol+error checking */
@@ -81,21 +91,21 @@ sc_cellname_to_coords (char const *cellname, int *col, int *row)
 		if (!g_ascii_isdigit (*cellname))
 			goto err_out;
 
-		*row = atoi (cellname);
+		pos->row = atoi (cellname);
 	}
 
-	g_return_val_if_fail (*col > -1, FALSE);
-	g_return_val_if_fail (*row > -1, FALSE);
+	g_return_val_if_fail (pos->col > -1, FALSE);
+	g_return_val_if_fail (pos->row > -1, FALSE);
 	return TRUE;
 
 err_out:
-	*col = *row = -1;
+	pos->col = pos->row = -1;
 	return FALSE;
 }
 
 
 static void
-sc_parse_coord (char const **strdata, int *col, int *row)
+sc_parse_coord (char const **strdata, GnmCellPos *pos)
 {
 	char const *s = *strdata, *eq;
 	int len = strlen (s);
@@ -103,8 +113,6 @@ sc_parse_coord (char const **strdata, int *col, int *row)
 	size_t tmplen;
 
 	g_return_if_fail (strdata);
-	g_return_if_fail (col);
-	g_return_if_fail (row);
 
 	eq = strstr (s, " = ");
 	if (!eq)
@@ -117,11 +125,11 @@ sc_parse_coord (char const **strdata, int *col, int *row)
 	memcpy (tmpstr, s, tmplen);
 	tmpstr [tmplen] = 0;
 
-	if (!sc_cellname_to_coords (tmpstr, col, row))
+	if (!sc_cellname_to_coords (tmpstr, pos))
 		return;
 
-	g_assert (*col >= 0);
-	g_assert (*row >= 0);
+	g_return_if_fail (pos->col >= 0);
+	g_return_if_fail (pos->row >= 0);
 
 	if ((eq - s + 1 + 3) > len)
 		return;
@@ -130,22 +138,28 @@ sc_parse_coord (char const **strdata, int *col, int *row)
 }
 
 
+static void
+set_h_align (Sheet *sheet, GnmCellPos const *pos, GnmHAlign ha)
+{
+	GnmRange r;
+	GnmStyle *style = gnm_style_new ();
+	gnm_style_set_align_h (style, ha);
+	r.start = r.end = *pos;
+	sheet_style_apply_range	(sheet, &r, style);
+}
+
 static gboolean
-sc_parse_label (Sheet *sheet, char const *cmd, char const *str, int col, int row)
+sc_parse_label (ScParseState *state, char const *cmd, char const *str,
+		GnmCellPos const *pos)
 {
 	GnmCell *cell;
 	char *s = NULL, *tmpout;
 	char const *tmpstr;
 	gboolean result = FALSE;
-	sc_string_cmd_t cmdtype;
 
-	g_return_val_if_fail (sheet, FALSE);
-	g_return_val_if_fail (cmd, FALSE);
 	g_return_val_if_fail (str, FALSE);
-	g_return_val_if_fail (col >= 0, FALSE);
-	g_return_val_if_fail (row >= 0, FALSE);
 
-	if (!str || *str != '"' || col == -1 || row == -1)
+	if (*str != '"')
 		goto err_out;
 
 	s = tmpout = g_strdup (str);
@@ -165,31 +179,20 @@ sc_parse_label (Sheet *sheet, char const *cmd, char const *str, int col, int row
 	tmpout--;
 	*tmpout = 0;
 
-	cell = sheet_cell_fetch (sheet, col, row);
+	cell = sheet_cell_fetch (state->sheet, pos->col, pos->row);
 	if (!cell)
 		goto err_out;
 
 	cell_set_text (cell, s);
 
 	if (strcmp (cmd, "leftstring") == 0)
-		cmdtype = LEFTSTRING;
+		set_h_align (state->sheet, pos, HALIGN_LEFT);
 	else if (strcmp (cmd, "rightstring") == 0)
-		cmdtype = RIGHTSTRING;
+		set_h_align (state->sheet, pos, HALIGN_RIGHT);
+#if 0
 	else
 		cmdtype = LABEL;
-
-	if (cmdtype == LEFTSTRING || cmdtype == RIGHTSTRING) {
-		GnmStyle *mstyle;
-
-		mstyle = cell_get_mstyle (cell);
-		if (!mstyle)
-			goto err_out;
-
-		if (cmdtype == LEFTSTRING)
-			mstyle_set_align_h (mstyle, HALIGN_LEFT);
-		else
-			mstyle_set_align_h (mstyle, HALIGN_RIGHT);
-	}
+#endif
 
 	result = TRUE;
 	/* fall through */
@@ -244,61 +247,116 @@ sc_parse_cell_name_list (Sheet *sheet, char const *cell_name_str,
 #endif
 
 
-G_GNUC_UNUSED static char const *
-sc_rangeref_parse (GnmRangeRef *res, char const *start, GnmParsePos const *pp,
-		   GnmExprConventions const *convention)
+static char const *
+sc_row_parse (char const *str, int *res, unsigned char *relative)
 {
-	/* This is a hack.  We still cannot handle sc's row 0.  */
-	char const *end = rangeref_parse (res, start, pp, convention);
-	if (end != start) {
-		res->a.row++;
-		res->b.row++;
-	}
-	return end;
+	char const *end, *ptr = str;
+	long int row;
+
+	if (!(*relative = (*ptr != '$')))
+		ptr++;
+
+	if (*ptr < '0' || *ptr > '9')
+		return NULL;
+
+	/*
+	 * Do not allow letters after the row number.  If we did, then
+	 * the name "K3P" would lex as the reference K3 followed by the
+	 * name "P".
+	 */
+	row = strtol (ptr, (char **)&end, 10);
+	if (ptr != end &&
+	    !g_unichar_isalnum (g_utf8_get_char (end)) && *end != '_' &&
+	    0 <= row && row  < SHEET_MAX_ROWS) {
+		*res = row;
+		return end;
+	} else
+		return NULL;
+}
+
+
+static char const *
+sc_rangeref_parse (GnmRangeRef *res, char const *start, GnmParsePos const *pp,
+			G_GNUC_UNUSED GnmExprConventions const *convs)
+{
+	char const *ptr = start, *tmp1, *tmp2;
+
+	g_return_val_if_fail (start != NULL, start);
+	g_return_val_if_fail (pp != NULL, start);
+
+	res->a.sheet = NULL;
+	tmp1 = col_parse (ptr, &res->a.col, &res->a.col_relative);
+	if (!tmp1)
+		return start;
+	tmp2 = sc_row_parse (tmp1, &res->a.row, &res->a.row_relative);
+	if (!tmp2)
+		return start;
+	if (res->a.col_relative)
+		res->a.col -= pp->eval.col;
+	if (res->a.row_relative)
+		res->a.row -= pp->eval.row;
+
+	/* prepare as if it's a singleton, in case we want to fall back */
+	res->b = res->a;
+	if (*tmp2 != ':')
+		return tmp2;
+
+	start = tmp2;
+	tmp1 = col_parse (start+1, &res->b.col, &res->b.col_relative);
+	if (!tmp1)
+		return start;
+	tmp2 = sc_row_parse (tmp1, &res->b.row, &res->b.row_relative);
+	if (!tmp2)
+		return start;
+	if (res->b.col_relative)
+		res->b.col -= pp->eval.col;
+	if (res->b.row_relative)
+		res->b.row -= pp->eval.row;
+	return tmp2;
 }
 
 
 static gboolean
-sc_parse_let (Sheet *sheet, char const *cmd, char const *str, int col, int row)
+sc_parse_let (ScParseState *state, char const *cmd, char const *str,
+	      GnmCellPos const *pos)
 {
-	GnmExpr const *tree;
+	GnmExprTop const *texpr;
 	GnmCell *cell;
-	GnmParsePos pos;
+	GnmParsePos pp;
 	GnmValue const *v;
+	char *str2, *p1, *p2;
 
-	g_return_val_if_fail (sheet, FALSE);
 	g_return_val_if_fail (cmd, FALSE);
 	g_return_val_if_fail (str, FALSE);
-	g_return_val_if_fail (col >= 0, FALSE);
-	g_return_val_if_fail (row >= 0, FALSE);
 
-	cell = sheet_cell_fetch (sheet, col, row);
+	cell = sheet_cell_fetch (state->sheet, pos->col, pos->row);
 	if (!cell)
 		return FALSE;
 
-	/* FIXME FIXME FIXME sc/xspread rows start at A0 not A1.  we must
-	 * go through and fixup each row number in each cell reference */
-	tree = gnm_expr_parse_str (str,
-				   parse_pos_init_cell (&pos, cell),
-				   GNM_EXPR_PARSE_DEFAULT,
-				   gnm_expr_conventions_default, /* FIXME! */
-				   NULL);
-
-	if (!tree) {
+	str2 = g_malloc0 (strlen (str) + 1);
+	for (p1 = (char *) str, p2 = str2; *p1; p1++)
+		if (*p1 != '@')
+			*p2++ = *p1;
+	texpr = gnm_expr_parse_str (str2,
+			parse_pos_init_cell (&pp, cell),
+			GNM_EXPR_PARSE_DEFAULT,
+			state->exprconv, NULL);
+	g_free (str2);
+	if (!texpr) {
 		g_warning ("cannot parse cmd='%s', str='%s', col=%d, row=%d.",
-			   cmd, str, col, row);
-		goto out;
+			   cmd, str, pos->col, pos->row);
+		return TRUE;
 	}
 
-	v = gnm_expr_get_constant (tree);
+	v = gnm_expr_top_get_constant (texpr);
 	if (v && VALUE_IS_NUMBER (v)) {
 		cell_set_value (cell, value_dup (v));
 	} else {
-		cell_set_expr (cell, tree);
+		cell_set_expr (cell, texpr);
+		cell_queue_recalc (cell);
 	}
 
-out:
-	if (tree) gnm_expr_unref (tree); 
+	if (texpr) gnm_expr_top_unref (texpr); 
 	return TRUE;
 }
 
@@ -306,29 +364,29 @@ out:
 typedef struct {
 	char const *name;
 	int namelen;
-	gboolean (*handler) (Sheet *sheet, char const *name,
-			     char const *str, int col, int row);
-	unsigned have_coord : 1;
+	gboolean (*handler) (ScParseState *state, char const *name,
+			     char const *str, GnmCellPos const *pos);
+	gboolean have_coord;
 } sc_cmd_t;
 
-
 static sc_cmd_t const sc_cmd_list[] = {
-	{ "leftstring", 10, sc_parse_label, 1 },
-	{ "rightstring", 11, sc_parse_label, 1 },
-	{ "label", 5, sc_parse_label, 1 },
-	{ "let", 3, sc_parse_let, 1 },
+	{ "leftstring", 10,	sc_parse_label,	TRUE },
+	{ "rightstring", 11,	sc_parse_label,	TRUE },
+	{ "label", 5,		sc_parse_label,	TRUE },
+	{ "let", 3,		sc_parse_let,	TRUE },
 	{ NULL, 0, NULL, 0 },
 };
 
 
 static gboolean
-sc_parse_line (Sheet *sheet, char *buf)
+sc_parse_line (ScParseState *state, char *buf)
 {
 	char const *space;
 	int i, cmdlen;
 	sc_cmd_t const *cmd;
 
-	g_return_val_if_fail (sheet, FALSE);
+	g_return_val_if_fail (state, FALSE);
+	g_return_val_if_fail (state->sheet, FALSE);
 	g_return_val_if_fail (buf, FALSE);
 
 	for (space = buf; g_ascii_isalnum (*space) || *space == '_'; space++)
@@ -343,13 +401,13 @@ sc_parse_line (Sheet *sheet, char *buf)
 		cmd = &sc_cmd_list [i];
 		if (cmd->namelen == cmdlen &&
 		    strncmp (cmd->name, buf, cmdlen) == 0) {
-			int col = -1, row = -1;
+			GnmCellPos pos = { -1, -1 };
 			char const *strdata = space;
 
 			if (cmd->have_coord)
-				sc_parse_coord (&strdata, &col, &row);
+				sc_parse_coord (&strdata, &pos);
 
-			cmd->handler (sheet, cmd->name, strdata, col, row);
+			cmd->handler (state, cmd->name, strdata, &pos);
 			return TRUE;
 		}
 	}
@@ -364,16 +422,17 @@ sc_parse_line (Sheet *sheet, char *buf)
 
 
 static ErrorInfo *
-sc_parse_sheet (GsfInputTextline *input, Sheet *sheet, GIConv ic)
+sc_parse_sheet (ScParseState *state)
 {
 	unsigned char *data;
-	while ((data = gsf_input_textline_ascii_gets (input)) != NULL) {
+	while ((data = gsf_input_textline_ascii_gets (state->textline)) != NULL) {
 		char *utf8data;
 
 		g_strchomp (data);
-		utf8data = g_convert_with_iconv (data, -1, ic, NULL, NULL, NULL);
+		utf8data = g_convert_with_iconv (data, -1, state->converter,
+						 NULL, NULL, NULL);
 
-		if (g_ascii_isalpha (*data) && !sc_parse_line (sheet, utf8data)) {
+		if (g_ascii_isalpha (*data) && !sc_parse_line (state, utf8data)) {
 			g_free (utf8data);
 			return error_info_new_str (_("Error parsing line"));
 		}
@@ -384,6 +443,72 @@ sc_parse_sheet (GsfInputTextline *input, Sheet *sheet, GIConv ic)
 	return NULL;
 }
 
+static struct {
+	char const *scname;
+	char const *gnumericname;
+} const simple_renames[] = {
+	{ "AVG",    "AVERAGE" },
+	{ "DTR",    "RADIANS" },
+	{ "FABS",   "ABS" },
+	{ "COLS",   "COLUMNS" },
+	{ "AVG",    "AVERAGE" },
+	{ "POW",    "POWER" },
+	{ "PROD",   "PRODUCT" },
+	{ "RND",    "ROUND" },
+	{ "RTD",    "DEGREES" },
+	{ "STDDEV", "STDEV" },
+	{ "STON",   "INT" },
+	{ "SUBSTR", "MID" },
+	{ NULL, NULL }
+};
+
+
+static GnmExpr const *
+function_renamer (char const *name,
+		  GnmExprList *args,
+		  GnmExprConventions *convs)
+{
+	Workbook *wb = NULL;
+	int i;
+	GnmFunc *f;
+
+	for (i = 0; simple_renames[i].scname; i++)
+		if (strcasecmp (name, simple_renames[i].scname) == 0) {
+			name = simple_renames[i].gnumericname;
+			break;
+		}
+
+	f = gnm_func_lookup (name, wb);
+	if (f)
+		return gnm_expr_new_funcall (f, args);
+
+	return gnm_func_placeholder_factory (name, args, convs);
+}
+
+
+static GnmExprConventions *
+sc_conventions (void)
+{
+	GnmExprConventions *convs = gnm_expr_conventions_new ();
+	int i;
+
+	convs->decimal_sep_dot = TRUE;
+	convs->ref_parser = sc_rangeref_parse;
+	convs->range_sep_colon = TRUE;
+	convs->sheet_sep_exclamation	= TRUE;
+	convs->unknown_function_handler = gnm_func_placeholder_factory;
+	convs->function_rewriter_hash =
+		g_hash_table_new (go_ascii_strcase_hash, 
+				  go_ascii_strcase_equal);
+	for (i = 0; simple_renames[i].scname; i++)
+		g_hash_table_insert (convs->function_rewriter_hash,
+				     (gchar *) simple_renames[i].scname,
+				     function_renamer);
+
+	return convs;
+}
+
+
 void
 sc_file_open (GOFileOpener const *fo, IOContext *io_context,
               WorkbookView *wb_view, GsfInput *input)
@@ -391,29 +516,30 @@ sc_file_open (GOFileOpener const *fo, IOContext *io_context,
 	Workbook  *wb;
 	char      *name;
 	ErrorInfo *error;
-	Sheet	  *sheet;
-	GsfInputTextline *textline;
-	GIConv    ic;
+	ScParseState state;
 
-	wb    = wb_view_workbook (wb_view);
+	wb    = wb_view_get_workbook (wb_view);
 	name  = workbook_sheet_get_free_name (wb, "SC", FALSE, TRUE);
-	sheet = sheet_new (wb, name);
+	state.sheet = sheet_new (wb, name);
 	g_free (name);
-	workbook_sheet_attach (wb, sheet);
+	workbook_sheet_attach (wb, state.sheet);
 
 	/* This should probably come from import dialog.  */
-	ic = g_iconv_open ("UTF-8", "ISO-8859-1");
-
-	textline = (GsfInputTextline *) gsf_input_textline_new (input);
-	error = sc_parse_sheet (textline, sheet, ic);
+	state.converter = g_iconv_open ("UTF-8", "ISO-8859-1");
+	
+	state.exprconv = sc_conventions ();
+	state.textline = (GsfInputTextline *) gsf_input_textline_new (input);
+	error = sc_parse_sheet (&state);
 	if (error != NULL) {
-		workbook_sheet_delete (sheet);
+		workbook_sheet_delete (state.sheet);
 		gnumeric_io_error_info_set (io_context, error);
 	}
-	g_object_unref (G_OBJECT (textline));
-	g_iconv_close (ic);
+	g_object_unref (G_OBJECT (state.textline));
+	g_iconv_close (state.converter);
+	gnm_expr_conventions_free (state.exprconv);
 }
 
+
 static guint8 const signature[] =
 "# This data file was generated by the Spreadsheet Calculator.";
 
