@@ -15,7 +15,9 @@ GObject     *gnm_app_get_app (void);
 void         gnm_app_workbook_list_add     (Workbook *wb);
 void         gnm_app_workbook_list_remove  (Workbook *wb);
 GList *      gnm_app_workbook_list 	   (void);
-Workbook    *gnm_app_workbook_get_by_name  (char const *name);
+Workbook    *gnm_app_workbook_get_by_uri   (char const *uri);
+Workbook    *gnm_app_workbook_get_by_name  (char const *name,
+					    char const *ref_uri);
 Workbook    *gnm_app_workbook_get_by_index (int i);
 gboolean     gnm_app_workbook_foreach	   (GnmWbIterFunc func, gpointer data);
 
@@ -30,7 +32,6 @@ gboolean     gnm_app_live_scrolling	  (void);
 int	     gnm_app_auto_expr_recalc_lag (void);
 
 /* stuff that should move */
-GdkPixbuf   *gnm_app_get_pixbuf		  (char const *name);
 void         gnm_app_release_pref_dialog  (void);
 gpointer     gnm_app_get_pref_dialog	  (void);
 void         gnm_app_set_pref_dialog	  (gpointer dialog);
@@ -58,7 +59,6 @@ GnmRange const	*gnm_app_clipboard_area_get	  (void);
  * with libgoffice
  **/
  
-typedef struct _GnmAction GnmAction;
 typedef void (*GnmActionHandler) (GnmAction const *action, WorkbookControl *wbc,
 				  gpointer user_data);
 struct _GnmAction {
