@@ -17,9 +17,7 @@ typedef enum {
 
 	COLLECT_IGNORE_BLANKS	= 0x1000,
 	COLLECT_IGNORE_SUBTOTAL	= 0x2000,
-	COLLECT_INFO		= 0x8000,
-
-	COLLECT_NO_INFO_MASK	= 0x7777
+	COLLECT_INFO		= 0x8000
 } CollectFlags;
 
 
@@ -27,11 +25,13 @@ typedef int (*float_range_function_t) (gnm_float const *, int, gnm_float *);
 typedef int (*float_range_function2_t) (gnm_float const *, gnm_float const *, int, gnm_float *);
 typedef int (*string_range_function_t) (GSList *, char**);
 
-gnm_float *collect_floats_value (GnmValue const *val, const GnmEvalPos *ep,
+gnm_float *collect_floats_value (GnmValue const *val,
+				 GnmEvalPos const *ep,
 				 CollectFlags flags,
 				 int *n, GnmValue **error);
 
-GnmValue *float_range_function (GnmExprList const *exprlist, FunctionEvalInfo *ei,
+GnmValue *float_range_function (int argc, GnmExprConstPtr const *argv,
+				FunctionEvalInfo *ei,
 				float_range_function_t func,
 				CollectFlags flags,
 				GnmStdError func_error);
@@ -41,7 +41,8 @@ GnmValue *float_range_function2 (GnmValue const *val0, GnmValue const *val1,
 				 float_range_function2_t func,
 				 CollectFlags flags,
 				 GnmStdError func_error);
-GnmValue *string_range_function (GnmExprList const *exprlist, FunctionEvalInfo *ei,
+GnmValue *string_range_function (int argc, GnmExprConstPtr const *argv,
+				 FunctionEvalInfo *ei,
 				 string_range_function_t func,
 				 CollectFlags flags,
 				 GnmStdError func_error);
