@@ -1,19 +1,13 @@
-#ifndef GNUMERIC_COLROW_H
-#define GNUMERIC_COLROW_H
+#ifndef GNM_COLROW_H
+#define GNM_COLROW_H
 
 #include "gnumeric.h"
 
 struct _ColRowInfo {
-	int	pos;		/* the column or row number */
-
 	/* Size including margins, and right grid line */
 	float	 size_pts;
 	int      size_pixels;
 
-	/* These are not scaled, and are the same in points and pixels */
-	unsigned  margin_a	: 3;  	/* top/left margin */
-	unsigned  margin_b	: 3; 	/* bottom/right margin */
-
 	unsigned  is_default	: 1;
 	unsigned  outline_level : 4;
 	unsigned  is_collapsed  : 1;	/* Does this terminate an outline ? */
@@ -32,11 +26,13 @@ struct _ColRowCollection {
 	ColRowInfo  default_style;
 	GPtrArray * info;
 	int	    max_outline_level;
-
-	float	    size_pts;
-	int	    size_pixels;
 };
 
+/* We never did get around to support 'thick' borders so these are effectively
+ * unitless (margins do not scale) constants . */
+#define	GNM_COL_MARGIN	2
+#define	GNM_ROW_MARGIN	0
+
 /* The size, mask, and shift must be kept in sync */
 #define COLROW_SEGMENT_SIZE	0x80
 #define COLROW_SUB_INDEX(i)	((i) & 0x7f)
@@ -52,7 +48,8 @@ struct _ColRowSegment {
 	int	size_pixels;
 };
 
-#define COL_INTERNAL_WIDTH(col) ((col)->size_pixels - ((col)->margin_b + (col)->margin_a + 1))
+#define COL_INTERNAL_WIDTH(col)	\
+	((col)->size_pixels - (GNM_COL_MARGIN + GNM_COL_MARGIN + 1))
 
 void	colrow_compute_pixels_from_pts (ColRowInfo *cri, Sheet const *sheet,
 					gboolean horizontal);
@@ -64,7 +61,12 @@ gboolean colrow_is_empty   (ColRowInfo const *cri);
 gboolean colrow_equal	   (ColRowInfo const *a, ColRowInfo const *b);
 void     colrow_copy	   (ColRowInfo *dst, ColRowInfo const *src);
 
-typedef gboolean (*ColRowHandler)(ColRowInfo *info, gpointer user_data);
+typedef struct {
+	int	pos;
+	ColRowInfo const *cri;
+} GnmColRowIter;
+
+typedef gboolean (*ColRowHandler)(GnmColRowIter const *iter, gpointer user_data);
 gboolean colrow_foreach	   (ColRowCollection const *infos,
 			    int first, int last,
 			    ColRowHandler callback,
@@ -118,4 +120,14 @@ int              colrow_find_adjacent_visible   (Sheet *sheet, gboolean is_col,
 
 void             rows_height_update		(Sheet *sheet, GnmRange const *range,
 						 gboolean shrink);
-#endif /* GNUMERIC_COLROW_H */
+
+void             colrow_autofit                 (Sheet *sheet,
+						 GnmRange const *r,
+						 gboolean is_cols,
+						 gboolean ignore_strings,
+						 gboolean min_current,
+						 gboolean min_default,
+						 ColRowIndexList **indices,
+						 ColRowStateList **sizes);
+
+#endif /* GNM_COLROW_H */
