@@ -12,7 +12,7 @@ void command_setup_combos	(WorkbookControl *wbc);
 void command_list_release	(GSList *cmds);
 
 gboolean cmd_set_text		(WorkbookControl *wbc, Sheet *sheet,
-				 GnmCellPos const *pos, const char *new_text,
+				 GnmCellPos const *pos, char const *new_text,
 				 PangoAttrList *markup);
 
 gboolean cmd_area_set_text	(WorkbookControl *wbc, SheetView *sv,
@@ -48,7 +48,7 @@ gboolean cmd_autofill		(WorkbookControl *wbc, Sheet *sheet,
 
 gboolean cmd_copyrel		(WorkbookControl *wbc,
 				 int dx, int dy,
-				 const char *name);
+				 char const *name);
 
 /* currently these operate on the current sheet, and it calling control's
  * selection.  In the future we should pass in a virtualized selection.
@@ -89,14 +89,15 @@ gboolean cmd_objects_move	(WorkbookControl *wbc,
 gboolean cmd_object_format	(WorkbookControl *wbc, SheetObject *so,
 				 gpointer orig_style);
 
-gboolean cmd_reorganize_sheets2 (WorkbookControl *wbc,
-				 WorkbookSheetState *old_state);
+gboolean cmd_reorganize_sheets  (WorkbookControl *wbc,
+				 WorkbookSheetState *old_state,
+				 Sheet *undo_sheet);
 
 gboolean cmd_rename_sheet	(WorkbookControl *wbc, Sheet *sheet,
 				 char const *new_name);
 
 gboolean cmd_set_comment	(WorkbookControl *wbc, Sheet *sheet,
-				 GnmCellPos const *pos, const char *new_text);
+				 GnmCellPos const *pos, char const *new_text);
 
 gboolean cmd_analysis_tool	(WorkbookControl *wbc, Sheet *sheet,
 				 data_analysis_output_t *dao, gpointer specs,
@@ -105,10 +106,13 @@ gboolean cmd_analysis_tool	(WorkbookControl *wbc, Sheet *sheet,
 gboolean cmd_merge_data		(WorkbookControl *wbc, Sheet *sheet,
 				 GnmValue *merge_zone, GSList *merge_fields, GSList *merge_data);
 
-gboolean cmd_change_summary	(WorkbookControl *wbc, GSList *sin_changes);
+gboolean cmd_change_meta_data	(WorkbookControl *wbc, GSList *changes, GSList *removed);
 gboolean cmd_print_setup	(WorkbookControl *wbc, Sheet *sheet, PrintInformation const *pi);
 gboolean cmd_define_name	(WorkbookControl *wbc, char const *name,
-				 GnmParsePos const *pp, GnmExpr const *expr);
+				 GnmParsePos const *pp,
+				 GnmExprTop const *texpr,
+				 char const *descriptor);
+gboolean cmd_remove_name        (WorkbookControl *wbc, GnmNamedExpr *nexpr);
 
 typedef enum  {
 	cmd_object_pull_to_front,
@@ -140,8 +144,6 @@ gboolean cmd_solver (WorkbookControl *wbc,
 gboolean cmd_goal_seek (WorkbookControl *wbc,
 			GnmCell *cell, GnmValue *ov, GnmValue *nv);
 
-gboolean cmd_clone_sheet (WorkbookControl *wbc, Sheet *sheet);
-
 gboolean cmd_tabulate (WorkbookControl *wbc, gpointer data);
 
 gboolean cmd_so_graph_config (WorkbookControl *wbc, SheetObject *sog,  
