@@ -5,8 +5,7 @@
 #include <goffice/utils/goffice-utils.h>
 #include <stdio.h>
 
-struct _GnmDependent
-{
+struct _GnmDependent {
 	guint	  flags;
 	Sheet	 *sheet;
 	GnmExprTop const *texpr;
@@ -16,8 +15,8 @@ struct _GnmDependent
 };
 
 typedef struct {
-	void (*eval) (GnmDependent *dep);
-	void (*set_expr) (GnmDependent *dep, GnmExprTop const *new_texpr);
+	void (*eval)	   (GnmDependent *dep);
+	void (*set_expr)   (GnmDependent *dep, GnmExprTop const *new_texpr);
 	void (*debug_name) (GnmDependent const *dep, GString *target);
 } GnmDependentClass;
 
@@ -88,7 +87,6 @@ void	 dependent_set_expr	   (GnmDependent *dep, GnmExprTop const *new_texpr);
 void	 dependent_set_sheet	   (GnmDependent *dep, Sheet *sheet);
 void	 dependent_link		   (GnmDependent *dep);
 void	 dependent_unlink	   (GnmDependent *dep);
-gboolean dependent_eval		   (GnmDependent *dep);
 void	 dependent_queue_recalc	   (GnmDependent *dep);
 void	 dependent_add_dynamic_dep (GnmDependent *dep, GnmRangeRef const *rr);
 
