@@ -24,7 +24,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  **/
 #include <gnumeric-config.h>
-#include <gnumeric-i18n.h>
+#include <glib/gi18n-lib.h>
 #include <gnumeric.h>
 #include "dialogs.h"
 #include "help.h"
@@ -39,7 +39,10 @@
 #include <workbook-edit.h>
 
 #include <glade/glade.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtktogglebutton.h>
 #include <widgets/gnumeric-expr-entry.h>
+#include <widgets/gnm-dao.h>
 #include "filter.h"
 
 
@@ -58,11 +61,8 @@ static void
 advanced_filter_update_sensitivity_cb (G_GNUC_UNUSED GtkWidget *dummy,
 				       AdvancedFilterState *state)
 {
-        Value *output_range   = NULL;
-        Value *input_range    = NULL;
-        Value *criteria_range = NULL;
-
-	int i;
+        GnmValue *input_range    = NULL;
+        GnmValue *criteria_range = NULL;
 
         input_range = gnm_expr_entry_parse_as_value (
 		GNM_EXPR_ENTRY (state->input_entry), state->sheet);
@@ -84,18 +84,11 @@ advanced_filter_update_sensitivity_cb (G_GNUC_UNUSED GtkWidget *dummy,
 	} else
 		value_release (criteria_range);
 
-	i = gnumeric_glade_group_value (state->gui, output_group);
-	if (i == 2) {
-		output_range = gnm_expr_entry_parse_as_value
-			(GNM_EXPR_ENTRY (state->output_entry),
-			 state->sheet);
-		if (output_range == NULL) {
-			gtk_label_set_text (GTK_LABEL (state->warning),
-					    _("The output range is invalid."));
-			gtk_widget_set_sensitive (state->ok_button, FALSE);
-			return;
-		} else
-			value_release (output_range);
+	if (!gnm_dao_is_ready (GNM_DAO (state->gdao))) {
+		gtk_label_set_text (GTK_LABEL (state->warning),
+				    _("The output range is invalid."));
+		gtk_widget_set_sensitive (state->ok_button, FALSE);
+		return;
 	}
 
 	gtk_label_set_text (GTK_LABEL (state->warning), "");
@@ -117,8 +110,8 @@ advanced_filter_ok_clicked_cb (G_GNUC_UNUSED GtkWidget *button,
 			       AdvancedFilterState *state)
 {
 	data_analysis_output_t  dao;
-	Value                   *input;
-	Value                   *criteria;
+	GnmValue                   *input;
+	GnmValue                   *criteria;
 	char                    *text;
 	GtkWidget               *w;
 	int                     err = 0;
@@ -151,7 +144,7 @@ advanced_filter_ok_clicked_cb (G_GNUC_UNUSED GtkWidget *button,
 				_("The given criteria are invalid."));
 		break;
 	case NO_RECORDS_FOUND:
-		gnumeric_notice_nonmodal ((GtkWindow *) state->dialog,
+		go_gtk_notice_nonmodal_dialog ((GtkWindow *) state->dialog,
 					  &(state->warning_dialog),
 					  GTK_MESSAGE_INFO,
 					  _("No matching records were found."));
@@ -194,7 +187,6 @@ dialog_advanced_filter (WorkbookControlGUI *wbcg)
 	if (dialog_tool_init (state, wbcg, wb_control_cur_sheet (wbc),
 			      GNUMERIC_HELP_LINK_ADVANCED_FILTER,
 			      "advanced-filter.glade", "Filter",
-			      _("_List Range:"), _("Criteria _Range:"),
 			      _("Could not create the Advanced Filter dialog."),
 			      ADVANCED_FILTER_KEY,
 			      G_CALLBACK (advanced_filter_ok_clicked_cb), NULL,
@@ -202,6 +194,8 @@ dialog_advanced_filter (WorkbookControlGUI *wbcg)
 			      0))
 		return;
 
+	gnm_dao_set_inplace (GNM_DAO (state->gdao), _("Filter _in-place"));
+	gnm_dao_set_put (GNM_DAO (state->gdao), FALSE, FALSE);
 	advanced_filter_update_sensitivity_cb (NULL, state);
 	tool_load_selection ((GenericToolState *)state, TRUE);
 
