@@ -24,7 +24,7 @@
  */
 
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gnumeric.h>
 #include "dialogs.h"
 #include "help.h"
@@ -306,7 +306,7 @@ cb_dialog_function_select_fun_selection_changed (GtkTreeSelection *the_selection
 					   cursor[1] == '*' &&
 					   cursor[2] == ' ') {
 					int j = g_utf8_pointer_to_offset (f_desc, cursor);
-					const char *p;
+					char const *p;
 
 					tag = gtk_text_buffer_create_tag
 						(state->description, NULL,
@@ -512,7 +512,7 @@ dialog_function_select (WorkbookControlGUI *wbcg, char const *key)
 
 	state = g_new (FunctionSelectState, 1);
 	state->wbcg  = wbcg;
-	state->wb    = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
+	state->wb    = wb_control_get_workbook (WORKBOOK_CONTROL (wbcg));
         state->gui   = gui;
         state->dialog = glade_xml_get_widget (state->gui, "selection_dialog");
 	state->formula_guru_key = key;
