@@ -4,7 +4,7 @@
  *
  * Authors:
  *   Jukka-Pekka Iivonen <iivonen@iki.fi>
- *   Morten Welinder (terra@diku.dk)
+ *   Morten Welinder (terra@gnome.org)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gnumeric.h>
 #include "dialogs.h"
 #include "help.h"
@@ -87,18 +87,25 @@ typedef struct {
 
 
 typedef struct {
+	GoalSeekState	*state;
 	GnmCell *xcell, *ycell;
-	gnm_float ytarget;
+	gnm_float	ytarget;
+	gboolean	update_ui;
 } GoalEvalData;
 
 static GoalSeekStatus
 goal_seek_eval (gnm_float x, gnm_float *y, void *vevaldata)
 {
-	GoalEvalData *evaldata = vevaldata;
+	GoalEvalData const *evaldata = vevaldata;
+	GnmValue *v = value_new_float (x);
 
-	cell_set_value (evaldata->xcell, value_new_float (x));
-	cell_queue_recalc (evaldata->xcell);
-	workbook_recalc (evaldata->xcell->base.sheet->workbook);
+	if (evaldata->update_ui) {
+		sheet_cell_set_value (evaldata->xcell, v);
+	} else {
+		cell_set_value (evaldata->xcell, v);
+		cell_queue_recalc (evaldata->xcell);
+	}
+	workbook_recalc (evaldata->state->wb);
 
 	if (evaldata->ycell->value) {
 	        *y = value_get_as_float (evaldata->ycell->value) - evaldata->ytarget;
@@ -126,6 +133,7 @@ gnumeric_goal_seek (GoalSeekState *state)
 	evaldata.xcell = state->change_cell;
 	evaldata.ycell = state->set_cell;
 	evaldata.ytarget = state->target_value;
+	evaldata.update_ui = FALSE;
 
 	hadold = !VALUE_IS_EMPTY_OR_ERROR (state->change_cell->value);
 	oldx = hadold ? value_get_as_float (state->change_cell->value) : 0;
@@ -236,6 +244,7 @@ gnumeric_goal_seek (GoalSeekState *state)
 	}
 
  DONE:
+	evaldata.update_ui = TRUE;
 	if (status == GOAL_SEEK_OK) {
 		gnm_float yroot;
 		(void) goal_seek_eval (seekdata.root, &yroot, &evaldata);
@@ -244,9 +253,6 @@ gnumeric_goal_seek (GoalSeekState *state)
 		(void) goal_seek_eval (oldx, &ydummy, &evaldata);
 	}
 
-	sheet_cell_calc_span (state->change_cell, SPANCALC_RENDER);
-	sheet_flag_status_update_cell (state->change_cell);
-
 	return status;
 }
 
@@ -294,13 +300,6 @@ dialog_destroy (GtkObject *w, GoalSeekState  *state)
 	return FALSE;
 }
 
-/**
- * cb_dialog_cancel_clicked:
- * @button:
- * @state:
- *
- * Close (destroy) the dialog
- **/
 static void
 cb_dialog_cancel_clicked (G_GNUC_UNUSED GtkWidget *button,
 			  GoalSeekState *state)
@@ -309,26 +308,17 @@ cb_dialog_cancel_clicked (G_GNUC_UNUSED GtkWidget *button,
 
 	if ((state->old_cell != NULL) && (state->old_value != NULL)) {
 		sheet_cell_set_value (state->old_cell, state->old_value);
-		workbook_recalc (state->old_cell->base.sheet->workbook);
+		workbook_recalc (state->wb);
 		state->old_value = NULL;
 	}
 	gtk_widget_destroy (state->dialog);
-	return;
 }
 
-/**
- * cb_dialog_close_clicked:
- * @button:
- * @state:
- *
- * Close (destroy) the dialog
- **/
 static void
 cb_dialog_close_clicked (G_GNUC_UNUSED GtkWidget *button,
 			 GoalSeekState *state)
 {
 	gtk_widget_destroy (state->dialog);
-	return;
 }
 
 /**
@@ -404,7 +394,7 @@ cb_dialog_apply_clicked (G_GNUC_UNUSED GtkWidget *button,
 	}
 
 
-	format = gnm_style_get_format (cell_get_mstyle (state->set_cell));
+	format = gnm_style_get_format (cell_get_style (state->set_cell));
 	if (entry_to_float_with_format (GTK_ENTRY(state->to_value_entry),
 					&state->target_value, TRUE, format)){
 		go_gtk_notice_nonmodal_dialog (GTK_WINDOW(state->dialog),
@@ -416,7 +406,7 @@ cb_dialog_apply_clicked (G_GNUC_UNUSED GtkWidget *button,
 		return;
 	}
 
-	format = gnm_style_get_format (cell_get_mstyle (state->change_cell));
+	format = gnm_style_get_format (cell_get_style (state->change_cell));
 	if (entry_to_float_with_format (GTK_ENTRY(state->at_least_entry),
 					 &state->xmin, TRUE, format)) {
 		state->xmin = -max_range_val;
@@ -431,7 +421,7 @@ cb_dialog_apply_clicked (G_GNUC_UNUSED GtkWidget *button,
 
 	if ((state->old_cell != NULL) && (state->old_value != NULL)) {
 		sheet_cell_set_value (state->old_cell, state->old_value);
-		workbook_recalc (state->old_cell->base.sheet->workbook);
+		workbook_recalc (state->wb);
 		state->old_value = NULL;
 	}
 	state->old_cell = state->change_cell;
@@ -444,7 +434,7 @@ cb_dialog_apply_clicked (G_GNUC_UNUSED GtkWidget *button,
 	case GOAL_SEEK_OK: {
 		const char *actual_str;
 		const char *solution_str;
-		GOFormat *format = style_format_general ();
+		GOFormat *format = go_format_general ();
 		GnmValue *error_value = value_new_float (state->target_value -
 						      value_get_as_float (state->set_cell->value));
   		char *target_str = format_value (format, error_value, NULL, 0,
@@ -644,7 +634,7 @@ dialog_goal_seek (WorkbookControlGUI *wbcg, Sheet *sheet)
 
 	state = g_new (GoalSeekState, 1);
 	state->wbcg  = wbcg;
-	state->wb    = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
+	state->wb    = wb_control_get_workbook (WORKBOOK_CONTROL (wbcg));
 	state->sheet = sheet;
 	state->gui   = gui;
 	state->warning_dialog = NULL;
