@@ -20,7 +20,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  **/
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gnumeric.h>
 #include "dialogs.h"
 #include "help.h"
@@ -109,8 +109,8 @@ prepare_ranges (simulation_t *sim)
 	    sim->outputs->type != VALUE_CELLRANGE)
 		return TRUE;
 
-	sim->ref_inputs  = rangeref_dup (value_get_rangeref (sim->inputs));
-	sim->ref_outputs = rangeref_dup (value_get_rangeref (sim->outputs));
+	sim->ref_inputs  = gnm_rangeref_dup (value_get_rangeref (sim->inputs));
+	sim->ref_outputs = gnm_rangeref_dup (value_get_rangeref (sim->outputs));
 
 	sim->n_input_vars =
 		(abs (sim->ref_inputs->a.col - sim->ref_inputs->b.col) + 1) *
@@ -154,7 +154,7 @@ prepare_ranges (simulation_t *sim)
 static void
 update_log (SimulationState *state, simulation_t *sim)
 {
-	const gchar   *txt [6] = {
+	char const *txt [6] = {
 		_("Simulations"), _("Iterations"), _("# Input variables"),
 		_("# Output variables"), _("Runtime"), _("Run on")
 	};
