@@ -23,7 +23,7 @@
  */
 
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gnumeric.h>
 #include "dialogs.h"
 #include "help.h"
@@ -105,9 +105,9 @@ typedef struct {
 	SolverModelType     type;
 } algorithm_def_t;
 
-static const algorithm_def_t algorithm_defs [] = {
+static algorithm_def_t const algorithm_defs [] = {
 	{ N_("Simplex (LP Solve)"), LPSolve, SolverLPModel },
-	{ N_("Revised Simplex (GLPK 4.5)"), GLPKSimplex, SolverLPModel },
+	{ N_("Revised Simplex (GLPK 4.9)"), GLPKSimplex, SolverLPModel },
 	{ N_("< Not available >"), QPDummy, SolverQPModel },
 	{ NULL, 0, 0 }
 };
@@ -118,14 +118,14 @@ typedef struct {
 	Sheet    *sheet;
 } constraint_conversion_t;
 
-static const char *const problem_type_group[] = {
+static char const * const problem_type_group[] = {
 	"min_button",
 	"max_button",
 	"equal_to_button",
 	NULL
 };
 
-static const char *const model_type_group[] = {
+static char const * const model_type_group[] = {
 	"lp_model_button",
 	"qp_model_button",
 	"nlp_model_button",
@@ -181,18 +181,14 @@ static gboolean
 dialog_set_sec_button_sensitivity (G_GNUC_UNUSED GtkWidget *dummy,
 				   SolverState *state)
 {
-	gboolean ready;
-	gboolean select_ready;
+	SolverConstraintType t = gtk_combo_box_get_active (state->type_combo);
+	gboolean select_ready = (state->constr != NULL);
 
-	select_ready = (state->constr != NULL);
-	ready = gnm_expr_entry_is_cell_ref (state->lhs.entry, state->sheet,
-					    TRUE) &&
-		((gtk_combo_box_get_active (state->type_combo)
-		  == SolverINT)
-		 || (gtk_combo_box_get_active (state->type_combo)
-		     == SolverBOOL)
-		 || (is_hom_row_or_col_ref (state->lhs.entry, state->rhs.entry,
-					    state->sheet)));
+	gboolean ready =
+		gnm_expr_entry_is_cell_ref (state->lhs.entry, state->sheet, TRUE) &&
+		((t == SolverINT) ||
+		 (t == SolverBOOL) ||
+		 (is_hom_row_or_col_ref (state->lhs.entry, state->rhs.entry, state->sheet)));
 
 	gtk_widget_set_sensitive (state->add_button, ready);
 	gtk_widget_set_sensitive (state->change_button, select_ready && ready);
@@ -520,12 +516,14 @@ cb_dialog_close_clicked (G_GNUC_UNUSED GtkWidget *button,
 }
 
 static GnmValue *
-grab_cells (Sheet *sheet, int col, int row, GnmCell *cell, void *user_data)
+cb_grab_cells (GnmCellIter const *iter, gpointer user)
 {
-	GList **the_list = user_data;
+	GList **the_list = user;
+	GnmCell *cell;
 
-	if (cell == NULL)
-		cell = sheet_cell_fetch (sheet, col, row);
+	if (NULL == (cell = iter->cell))
+		cell = sheet_cell_create (iter->pp.sheet,
+			iter->pp.eval.col, iter->pp.eval.row);
 	*the_list = g_list_append (*the_list, cell);
 	return NULL;
 }
@@ -539,7 +537,7 @@ check_int_constraints (GnmValue *input_range, SolverState *state)
 	store = gtk_tree_view_get_model (state->constraint_list);
 	if (gtk_tree_model_get_iter_first (store, &iter))
 		do {
-			const constraint_t *a_constraint;
+			constraint_t const *a_constraint;
 			gchar *text;
 
 			gtk_tree_model_get (store, &iter, 0, &text, 1, &a_constraint, -1);
@@ -576,7 +574,7 @@ convert_constraint_format (constraint_conversion_t *conv)
 {
 	GtkTreeModel *store;
 	GtkTreeIter iter;
-	const constraint_t *a_constraint;
+	constraint_t const *a_constraint;
 	SolverConstraint *engine_constraint;
 
 	store = gtk_tree_view_get_model (conv->c_listing);
@@ -633,7 +631,7 @@ revert_constraint_format (constraint_conversion_t * conv)
 	GSList *engine_constraint_list = conv->c_list;
 
 	while (engine_constraint_list != NULL) {
-		const SolverConstraint *engine_constraint =
+		SolverConstraint const *engine_constraint =
 			engine_constraint_list->data;
 		GnmRange r;
 		constraint_t *a_constraint = g_new (constraint_t, 1);
@@ -688,7 +686,7 @@ save_original_values (GSList *input_cells)
 /* Returns FALSE if the reports deleted the current sheet
  * and forced the dialog to die */
 static gboolean
-solver_reporting (SolverState *state, SolverResults *res, const gchar *errmsg)
+solver_reporting (SolverState *state, SolverResults *res, gchar const *errmsg)
 {
 	SolverOptions *opt = &res->param->options;
 	gchar         *err = NULL;
@@ -770,7 +768,7 @@ solver_reporting (SolverState *state, SolverResults *res, const gchar *errmsg)
 			((GtkWindow *) state->dialog,
 			 &(state->warning_dialog),
 			 GTK_MESSAGE_ERROR,
-			 solver_max_time_err);
+			 SOLVER_MAX_TIME_ERR);
 		err = solver_reports (WORKBOOK_CONTROL(state->wbcg),
 				      state->sheet, res,
 				      FALSE, FALSE, FALSE,
@@ -796,11 +794,11 @@ solver_reporting (SolverState *state, SolverResults *res, const gchar *errmsg)
 }
 
 static void
-solver_add_scenario (SolverState *state, SolverResults *res, const gchar *name)
+solver_add_scenario (SolverState *state, SolverResults *res, gchar const *name)
 {
 	SolverParameters *param = res->param;
 	GnmValue         *input_range;
-	const gchar      *comment = _("Optimal solution created by solver.\n");
+	gchar const      *comment = _("Optimal solution created by solver.\n");
 	scenario_t       *scenario;
 
 	input_range = gnm_expr_entry_parse_as_value (state->change_cell_entry,
@@ -834,10 +832,10 @@ cb_dialog_solve_clicked (G_GNUC_UNUSED GtkWidget *button,
 	gint                    i;
 	gboolean                answer, sensitivity, limits, performance;
 	gboolean                program, dual_program;
-	const gchar             *errmsg = _("Unknown error.");
+	gchar const             *errmsg = _("Unknown error.");
 	SolverParameters        *param;
 	GtkTreeIter iter;
-	const gchar *name;
+	gchar const *name;
 
 	param = state->sheet->solver_parameters;
 
@@ -879,7 +877,7 @@ cb_dialog_solve_clicked (G_GNUC_UNUSED GtkWidget *button,
 
 	result = workbook_foreach_cell_in_range (
 		eval_pos_init_sheet (&pos, state->sheet),
-		input_range, CELL_ITER_ALL, grab_cells, &input_cells);
+		input_range, CELL_ITER_ALL, cb_grab_cells, &input_cells);
 
 	param->input_cells = input_cells;
 
@@ -1341,7 +1339,7 @@ dialog_solver (WorkbookControlGUI *wbcg, Sheet *sheet)
 
 	state                 = g_new (SolverState, 1);
 	state->wbcg           = wbcg;
-	state->wb             = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
+	state->wb             = wb_control_get_workbook (WORKBOOK_CONTROL (wbcg));
 	state->sheet          = sheet;
 	state->ov             = NULL;
 	state->ov_stack       = NULL;
