@@ -21,7 +21,7 @@
  */
 
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gnumeric.h>
 #include "dialog-stf.h"
 #include <gnm-format.h>
@@ -179,8 +179,10 @@ cb_col_check_clicked (GtkToggleButton *togglebutton, gpointer _i)
 			pagedata->format.col_import_count++;
 			format_page_update_column_selection (pagedata);
 		} else {
-			char *msg = g_strdup_printf 
-				(_("A maximum of %d columns can be imported."), 
+			char *msg = g_strdup_printf( 
+				ngettext("A maximum of %d column can be imported.",
+					 "A maximum of %d columns can be imported.",
+					 SHEET_MAX_COLS), 
 				 SHEET_MAX_COLS);
 			gtk_toggle_button_set_active (togglebutton, FALSE);
 			go_gtk_notice_dialog (GTK_WINDOW (pagedata->dialog),
@@ -702,6 +704,10 @@ stf_dialog_format_page_init (GladeXML *gui, StfDialogData *pagedata)
 
 	pagedata->format.locale_selector =
 		GO_LOCALE_SEL (go_locale_sel_new ());
+	if (pagedata->locale && !go_locale_sel_set_locale (pagedata->format.locale_selector, pagedata->locale)) {
+		g_free (pagedata->locale);
+		pagedata->locale = go_locale_sel_get_locale (pagedata->format.locale_selector);
+	}
 	gtk_table_attach (
 		GTK_TABLE (glade_xml_get_widget (gui, "locale_table")),
 		GTK_WIDGET (pagedata->format.locale_selector),
