@@ -5,7 +5,7 @@
  *   Dialog for making tables of function dependcies.
  *
  * Author:
- *   COPYRIGHT (C) Morten Welinder (terra@diku.dk)
+ *   COPYRIGHT (C) Morten Welinder (terra@gnome.org)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of version 2 of the GNU General Public License as published
@@ -23,7 +23,7 @@
  */
 
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gnumeric.h>
 #include "dialogs.h"
 #include "help.h"
@@ -151,7 +151,7 @@ get_table_float_entry (GtkTable *t, int y, int x, GnmCell *cell, gnm_float *numb
 		if (child->left_attach == x && child->top_attach == y &&
 		    GTK_IS_ENTRY (child->widget)) {
 			*wp = GTK_ENTRY (child->widget);
-			format = gnm_style_get_format (cell_get_mstyle (cell));
+			format = gnm_style_get_format (cell_get_style (cell));
 			return (with_default ?
 				entry_to_float_with_format_default (GTK_ENTRY (child->widget), number,
 							   TRUE, format, default_float) :
