@@ -30,7 +30,7 @@
 #include <goffice/graph/gog-style.h>
 #include <goffice/utils/go-color.h>
 #include <gsf/gsf-impl-utils.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <string.h>
 
 #define GNM_SO_LINE(o)		(G_TYPE_CHECK_INSTANCE_CAST((o), GNM_SO_LINE_TYPE, GnmSOLine))
@@ -206,14 +206,14 @@ gnm_so_line_print (SheetObject const *so, GnomePrintContext *ctx,
 	if (style->color == 0 || style->width < 0 || style->pattern == 0)
 		return;
 
-	switch (so->anchor.direction) {
-	case SO_DIR_UP_RIGHT:
-	case SO_DIR_DOWN_RIGHT:
+	switch (so->anchor.base.direction) {
+	case GOD_ANCHOR_DIR_UP_RIGHT:
+	case GOD_ANCHOR_DIR_DOWN_RIGHT:
 		x1 = 0.;
 		x2 = width;
 		break;
-	case SO_DIR_UP_LEFT:
-	case SO_DIR_DOWN_LEFT:
+	case GOD_ANCHOR_DIR_UP_LEFT:
+	case GOD_ANCHOR_DIR_DOWN_LEFT:
 		x1 = width;
 		x2 = 0.;
 		break;
@@ -222,14 +222,14 @@ gnm_so_line_print (SheetObject const *so, GnomePrintContext *ctx,
 		return;
 	}
 
-	switch (so->anchor.direction) {
-	case SO_DIR_UP_LEFT:
-	case SO_DIR_UP_RIGHT:
+	switch (so->anchor.base.direction) {
+	case GOD_ANCHOR_DIR_UP_LEFT:
+	case GOD_ANCHOR_DIR_UP_RIGHT:
 		y1 = -height;
 		y2 = 0.;
 		break;
-	case SO_DIR_DOWN_LEFT:
-	case SO_DIR_DOWN_RIGHT:
+	case GOD_ANCHOR_DIR_DOWN_LEFT:
+	case GOD_ANCHOR_DIR_DOWN_RIGHT:
 		y1 = 0.;
 		y2 = -height;
 		break;
@@ -239,9 +239,9 @@ gnm_so_line_print (SheetObject const *so, GnomePrintContext *ctx,
 	}
 
 	gnome_print_setrgbcolor (ctx,
-		style->color / (double) 0xffff,
-		style->color / (double) 0xffff,
-		style->color / (double) 0xffff);
+		DOUBLE_RGBA_R (style->color),
+		DOUBLE_RGBA_G (style->color),
+		DOUBLE_RGBA_B (style->color));
 
 	if (sol->end_arrow.c > 0.) {
 		double phi;
@@ -276,6 +276,90 @@ gnm_so_line_print (SheetObject const *so, GnomePrintContext *ctx,
 
 #endif /* WITH_GTK */
 
+static void
+gnm_so_line_draw_cairo (SheetObject const *so, gpointer data,
+	double width, double height)
+{
+#ifdef GOFFICE_WITH_CAIRO
+	GnmSOLine *sol = GNM_SO_LINE (so);
+	cairo_t *cairo = (cairo_t*) data;
+	GogStyleLine const *style = &sol->style->line;
+	double x1, y1, x2, y2;
+
+	if (style->color == 0 || style->width < 0 || style->pattern == 0)
+		return;
+
+	switch (so->anchor.base.direction) {
+	case GOD_ANCHOR_DIR_UP_RIGHT:
+	case GOD_ANCHOR_DIR_DOWN_RIGHT:
+		x1 = 0.;
+		x2 = width;
+		break;
+	case GOD_ANCHOR_DIR_UP_LEFT:
+	case GOD_ANCHOR_DIR_DOWN_LEFT:
+		x1 = width;
+		x2 = 0.;
+		break;
+	default:
+		g_warning ("Cannot guess direction!");
+		return;
+	}
+
+	switch (so->anchor.base.direction) {
+	case GOD_ANCHOR_DIR_UP_LEFT:
+	case GOD_ANCHOR_DIR_UP_RIGHT:
+		y1 = height;
+		y2 = 0.;
+		break;
+	case GOD_ANCHOR_DIR_DOWN_LEFT:
+	case GOD_ANCHOR_DIR_DOWN_RIGHT:
+		y1 = 0.;
+		y2 = height;
+		break;
+	default:
+		g_warning ("Cannot guess direction!");
+		return;
+	}
+
+	cairo_set_line_width (cairo, (style->width)? style->width: 1.);
+	cairo_set_source_rgba (cairo,
+		UINT_RGBA_R(style->color),
+		UINT_RGBA_B(style->color),
+		UINT_RGBA_G(style->color),
+		UINT_RGBA_A(style->color));
+
+	if (sol->end_arrow.c > 0.) {
+		double phi;
+
+		phi = atan2 (y2 - y1, x2 - x1) - M_PI_2;
+
+		cairo_save (cairo);
+		cairo_translate (cairo, x2, y2);
+		cairo_rotate (cairo, phi);
+		cairo_set_line_width (cairo, 1.0);
+		cairo_new_path (cairo);
+		cairo_move_to (cairo, 0.0, 0.0);
+		cairo_line_to (cairo, -sol->end_arrow.c, -sol->end_arrow.b);
+		cairo_line_to (cairo, 0.0, -sol->end_arrow.a);
+		cairo_line_to (cairo, sol->end_arrow.c, -sol->end_arrow.b);
+		cairo_close_path (cairo);
+		cairo_fill (cairo);
+		cairo_restore (cairo);
+
+		/* Make the line shorter so that the arrow won't be
+		 * on top of a (perhaps quite fat) line.  */
+		x2 += sol->end_arrow.a * sin (phi);
+		y2 -= sol->end_arrow.a * cos (phi);
+	}
+
+	cairo_set_line_width (cairo, style->width);
+	cairo_new_path (cairo);
+	cairo_move_to (cairo, x1, y1);
+	cairo_line_to (cairo, x2, y2);
+	cairo_stroke (cairo);
+#endif
+}
+
 static gboolean
 gnm_so_line_read_xml_dom (SheetObject *so, char const *typename,
 			  XmlParseContext const *ctxt,
@@ -412,6 +496,7 @@ gnm_so_line_class_init (GObjectClass *gobject_class)
 	so_class->xml_export_name	= "SheetObjectGraphic";
 
 #ifdef WITH_GTK
+	so_class->draw_cairo	= gnm_so_line_draw_cairo;
 	so_class->user_config		= gnm_so_line_user_config;
 	so_class->new_view		= gnm_so_line_new_view;
 	so_class->print			= gnm_so_line_print;
@@ -439,7 +524,7 @@ gnm_so_line_init (GObject *obj)
 	go_arrow_init (&sol->start_arrow, 0., 0., 0.);
 	go_arrow_init (&sol->end_arrow,   0., 0., 0.);
 
-	SHEET_OBJECT (obj)->anchor.direction = SO_DIR_NONE_MASK;
+	SHEET_OBJECT (obj)->anchor.base.direction = GOD_ANCHOR_DIR_NONE_MASK;
 }
 
 GSF_CLASS (GnmSOLine, gnm_so_line,
