@@ -2,11 +2,10 @@
 /*
  * gnumeric-gconf.c:
  *
- *
  * Author:
  * 	Andreas J. Guelzow <aguelzow@taliesin.ca>
  *
- * (C) Copyright 2002-2004 Andreas J. Guelzow <aguelzow@taliesin.ca>
+ * (C) Copyright 2002-2005 Andreas J. Guelzow <aguelzow@taliesin.ca>
  *
  * Introduced the concept of "node" and implemented the win32 backend
  * by Ivan, Wong Yat Cheung <email@ivanwong.info>, 2005
@@ -821,7 +820,7 @@ go_conf_get_str_list (GOConfNode *node, gchar const *key)
 	gint i;
 
 	if ((ptr = go_conf_get_string (node, key)) != NULL) {
-		str_list = g_strsplit ((const gchar *) ptr, "\n", 0);
+		str_list = g_strsplit ((gchar const *) ptr, "\n", 0);
 		for (i = 0; str_list[i]; ++i)
 			list = g_slist_prepend (list, g_strcompress (str_list[i]));
 		list = g_slist_reverse (list);
@@ -1190,7 +1189,7 @@ go_conf_set_double (GOConfNode *node, gchar const *key, gnm_float val)
 	gchar str[G_ASCII_DTOSTR_BUF_SIZE];
 
 	g_ascii_dtostr (str, sizeof (str), val);
-	g_key_file_set_value (key_file, DOUBLE_GROUP, key, str);
+	g_key_file_set_value (key_file, DOUBLE_GROUP, real_key, str);
 	g_free (real_key);
 }
 
@@ -1223,7 +1222,7 @@ go_conf_set_str_list (GOConfNode *node, gchar const *key, GSList *list)
 	}
 
 	g_key_file_set_string_list (key_file, STRLIST_GROUP, real_key, 
-				    (const gchar **const) strs, ns);
+				    (gchar const **const) strs, ns);
 	g_free (real_key);
 
 	for (i = 0; i < ns; i++)
@@ -1578,6 +1577,10 @@ gnm_conf_init_essential (void)
 		(g_str_hash, g_str_equal,
 		 (GDestroyNotify)g_free,
 		 NULL);
+	prefs.toolbar_positions = g_hash_table_new_full
+		(g_str_hash, g_str_equal,
+		 (GDestroyNotify)g_free,
+		 NULL);
 	go_conf_free_node (node);
 
 	/* Unfortunately we need the printing stuff in essentials since the */
@@ -1694,6 +1697,8 @@ gnm_conf_init_extras (void)
 	node = go_conf_get_node (root, GNM_CONF_SORT_DIR);
 	prefs.sort_default_by_case = go_conf_load_bool (
 		node, GNM_CONF_SORT_DEFAULT_BY_CASE, FALSE);
+	prefs.sort_default_has_header = go_conf_load_bool (
+		node, GNM_CONF_SORT_DEFAULT_HAS_HEADER, FALSE);
 	prefs.sort_default_retain_formats = go_conf_load_bool (
 		node, GNM_CONF_SORT_DEFAULT_RETAIN_FORM, TRUE);
 	prefs.sort_default_ascending = go_conf_load_bool (
@@ -1739,6 +1744,7 @@ gnm_conf_shutdown (void)
 		prefs.printer_decoration_font = NULL;
 	}
 	g_hash_table_destroy (prefs.toolbars);
+	g_hash_table_destroy (prefs.toolbar_positions);
 	go_conf_free_node (root);
 	go_conf_shutdown ();
 }
@@ -2056,7 +2062,7 @@ gnm_gconf_set_hf_font (GnmStyle const *mstyle)
 		prefs.printer_decoration_font :
 		gnm_style_new_default ();
 	
-	prefs.printer_decoration_font = gnm_style_merge (old_style, mstyle);
+	prefs.printer_decoration_font = gnm_style_new_merged (old_style, mstyle);
 	gnm_style_unref (old_style);
 	
 	node = go_conf_get_node (root, PRINTSETUP_GCONF_DIR);
@@ -2146,6 +2152,14 @@ gnm_gconf_set_sort_by_case (gboolean val)
 		root, GNM_CONF_SORT_DIR "/" GNM_CONF_SORT_DEFAULT_BY_CASE, val != FALSE);
 }
 
+void     
+gnm_gconf_set_sort_has_header (gboolean val)
+{
+	prefs.sort_default_has_header = val;
+	go_conf_set_bool (
+		root, GNM_CONF_SORT_DIR "/" GNM_CONF_SORT_DEFAULT_HAS_HEADER, val != FALSE);
+}
+
 void     
 gnm_gconf_set_sort_ascending (gboolean val)
 {
@@ -2211,7 +2225,7 @@ gnm_gconf_set_gui_resolution_v (gnm_float val)
 }
 
 gboolean
-gnm_gconf_get_toolbar_visible (const char *name)
+gnm_gconf_get_toolbar_visible (char const *name)
 {
 	gpointer pval;
 	char *key = g_strconcat (GNM_CONF_GUI_DIR "/" GNM_CONF_GUI_TOOLBARS "/",
@@ -2236,7 +2250,7 @@ gnm_gconf_get_toolbar_visible (const char *name)
 }
 
 void
-gnm_gconf_set_toolbar_visible (const char *name, gboolean vis)
+gnm_gconf_set_toolbar_visible (char const *name, gboolean vis)
 {
 	char *key = g_strconcat (GNM_CONF_GUI_DIR "/" GNM_CONF_GUI_TOOLBARS "/",
 				 name,
@@ -2249,6 +2263,53 @@ gnm_gconf_set_toolbar_visible (const char *name, gboolean vis)
 	g_free (key);
 }
 
+/*
+ * Actually returns a GtkPositionType.
+ */
+int
+gnm_gconf_get_toolbar_position (char const *name)
+{
+	gpointer pval;
+	char *key = g_strconcat (GNM_CONF_GUI_DIR "/" GNM_CONF_GUI_TOOLBARS "/",
+				 name, "-position",
+				 NULL);
+	gboolean found;
+	int pos;
+	static const int TOP = 2;
+
+	found = g_hash_table_lookup_extended (prefs.toolbar_positions,
+					      key,
+					      NULL, &pval);
+	if (found) {
+		pos = GPOINTER_TO_INT (pval);
+	} else {
+		pos = go_conf_load_int (root, key, 0, 3, TOP);
+		g_hash_table_insert (prefs.toolbar_positions,
+				     g_strdup (name),
+				     GINT_TO_POINTER (pos));
+	}
+
+	g_free (key);
+	return pos;
+}
+
+void
+gnm_gconf_set_toolbar_position (char const *name, int pos)
+{
+	char *key;
+
+	g_return_if_fail (pos >= 0 && pos <= 3);
+
+	key = g_strconcat (GNM_CONF_GUI_DIR "/" GNM_CONF_GUI_TOOLBARS "/",
+				 name, "-position",
+				 NULL);
+	g_hash_table_replace (prefs.toolbar_positions,
+			      g_strdup (name),
+			      GINT_TO_POINTER (pos));
+	go_conf_set_int (root, key, pos);
+	g_free (key);
+}
+
 void     
 gnm_gconf_set_unfocused_rs (gboolean val)
 {
