@@ -36,13 +36,15 @@
 #include <gtk/gtkimage.h>
 #include <gtk/gtkdnd.h>
 #include <gdk/gdkdisplay.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <math.h>
 #define GNUMERIC_ITEM "GnmPane"
 #include "item-debug.h"
 
 #undef DEBUG_DND
 
+static void cb_pane_popup_menu (GnmPane *pane);
+
 /**
  * For now, application/x-gnumeric is disabled. It handles neither
  * images nor graphs correctly.
@@ -147,33 +149,6 @@ gnm_pane_display_obj_size_tip (GnmPane *pane, SheetObject const *so)
 	g_free (msg);
 }
 
-static void
-cb_pane_popup_menu (GnmPane *pane)
-{
-	/* the popup-menu signal is a binding. the grid almost always has focus
-	 * we need to cheat to find out if the user realllllly wants a col/row
-	 * header menu */
-	gboolean is_col = FALSE;
-	gboolean is_row = FALSE;
-	GdkWindow *gdk_win = gdk_display_get_window_at_pointer (
-		gtk_widget_get_display (GTK_WIDGET (pane->gcanvas)),
-		NULL, NULL);
-
-	if (gdk_win != NULL) {
-		gpointer gtk_win_void = NULL;
-		GtkWindow *gtk_win = NULL;
-		gdk_window_get_user_data (gdk_win, &gtk_win_void);
-		gtk_win = gtk_win_void;
-		if (gtk_win != NULL) {
-			if (gtk_win == (GtkWindow *)pane->col.canvas)
-				is_col = TRUE;
-			else if (gtk_win == (GtkWindow *)pane->row.canvas)
-				is_row = TRUE;
-		}
-	}
-	scg_context_menu (pane->gcanvas->simple.scg, NULL, is_col, is_row);
-}
-
 static void
 cb_ctrl_pts_free (GtkObject **ctrl_pts)
 {
@@ -239,11 +214,13 @@ cb_pane_drag_motion (GtkWidget *widget, GdkDragContext *context,
 			"wbcg", scg_get_wbcg (scg));
 		gnm_canvas_window_to_coord (gcanvas, x, y, &wx, &wy);
 
-		gdk_window_get_pointer (gtk_widget_get_parent_window (widget),
+		gdk_window_get_pointer (gtk_widget_get_parent_window (source_widget),
 			NULL, NULL, &mask);
 		gnm_pane_objects_drag (GNM_CANVAS (source_widget)->pane, NULL,
 			wx, wy, 8, FALSE, (mask & GDK_SHIFT_MASK) != 0);
-		gdk_drag_status (context, GDK_ACTION_MOVE, time);
+		gdk_drag_status (context, 
+				 (mask & GDK_CONTROL_MASK) != 0 ? GDK_ACTION_COPY : GDK_ACTION_MOVE, 
+				 time);
 	}
 	return TRUE;
 }
@@ -852,6 +829,7 @@ cb_ptr_array_free (GPtrArray *actions)
 	g_ptr_array_free (actions, TRUE);
 }
 
+/* event and so can be NULL */
 static void
 display_object_menu (GnmPane *pane, SheetObject *so, GdkEvent *event)
 {
@@ -860,9 +838,11 @@ display_object_menu (GnmPane *pane, SheetObject *so, GdkEvent *event)
 	GtkWidget *menu;
 	unsigned i = 0;
 
-	if (NULL == g_hash_table_lookup (scg->selected_objects, so))
+	if (NULL != so &&
+	    NULL == g_hash_table_lookup (scg->selected_objects, so))
 		scg_object_select (scg, so);
-	SHEET_OBJECT_CLASS (G_OBJECT_GET_CLASS(so))->populate_menu (so, actions);
+
+	sheet_object_populate_menu (so, actions);
 
 	if (actions->len == 0) {
 		g_ptr_array_free (actions, TRUE);
@@ -878,6 +858,53 @@ display_object_menu (GnmPane *pane, SheetObject *so, GdkEvent *event)
 	gnumeric_popup_menu (GTK_MENU (menu), &event->button);
 }
 
+static void
+cb_collect_selected_objs (SheetObject *so, double *coords, GSList **accum)
+{
+	*accum = g_slist_prepend (*accum, so);
+}
+
+static void
+cb_pane_popup_menu (GnmPane *pane)
+{
+	SheetControlGUI *scg = pane->gcanvas->simple.scg;
+
+	/* ignore new_object, it is not visible, and should not create a
+	 * context menu */
+	if (NULL != scg->selected_objects) {
+		GSList *accum = NULL;
+		g_hash_table_foreach (scg->selected_objects,
+			(GHFunc) cb_collect_selected_objs, &accum);
+		if (NULL != accum && NULL == accum->next)
+			display_object_menu (pane, accum->data, NULL);
+		g_slist_free (accum);
+	} else {
+		/* the popup-menu signal is a binding. the grid almost always
+		 * has focus we need to cheat to find out if the user
+		 * realllllly wants a col/row header menu */
+		gboolean is_col = FALSE;
+		gboolean is_row = FALSE;
+		GdkWindow *gdk_win = gdk_display_get_window_at_pointer (
+			gtk_widget_get_display (GTK_WIDGET (pane->gcanvas)),
+			NULL, NULL);
+
+		if (gdk_win != NULL) {
+			gpointer gtk_win_void = NULL;
+			GtkWindow *gtk_win = NULL;
+			gdk_window_get_user_data (gdk_win, &gtk_win_void);
+			gtk_win = gtk_win_void;
+			if (gtk_win != NULL) {
+				if (gtk_win == (GtkWindow *)pane->col.canvas)
+					is_col = TRUE;
+				else if (gtk_win == (GtkWindow *)pane->row.canvas)
+					is_row = TRUE;
+			}
+		}
+
+		scg_context_menu (scg, NULL, is_col, is_row);
+	}
+}
+
 static void
 control_point_set_cursor (SheetControlGUI const *scg, FooCanvasItem *ctrl_pt)
 {
@@ -1000,33 +1027,12 @@ void
 gnm_pane_object_start_resize (GnmPane *pane, GdkEventButton *event,
 			      SheetObject *so, int drag_type, gboolean is_creation)
 {
-	/* ctrl-click on acetate dups the object */
-	gboolean const make_dup = drag_type == 8 &&
-		(event->state & GDK_CONTROL_MASK);
 	FooCanvasItem **ctrl_pts;
 
 	g_return_if_fail (IS_SHEET_OBJECT (so));
 	g_return_if_fail (0 <= drag_type);
 	g_return_if_fail (drag_type < 9);
 
-	if (make_dup) {
-		SheetControlGUI *scg = pane->gcanvas->simple.scg;
-		GSList *ptr, *objs = go_hash_keys (scg->selected_objects);
-
-		for (ptr = objs ; ptr != NULL ; ptr = ptr->next) {
-			SheetObject *dup_obj = sheet_object_dup (ptr->data);
-			if (dup_obj != NULL) {
-				sheet_object_set_sheet (dup_obj, sc_sheet (SHEET_CONTROL (scg)));
-				scg_object_select (scg, dup_obj);
-				g_object_unref (dup_obj);
-				scg_object_unselect (scg, ptr->data);
-				if (so == ptr->data)
-					so = dup_obj;
-			}
-		}
-		g_slist_free (objs);
-	}
-
 	ctrl_pts = g_hash_table_lookup (pane->drag.ctrl_pts, so);
 
 	g_return_if_fail (NULL != ctrl_pts);
@@ -1036,7 +1042,7 @@ gnm_pane_object_start_resize (GnmPane *pane, GdkEventButton *event,
 		GDK_BUTTON_PRESS_MASK |
 		GDK_BUTTON_RELEASE_MASK,
 		NULL, event->time);
-	pane->drag.created_objects = is_creation || make_dup;
+	pane->drag.created_objects = is_creation;
 	pane->drag.button = event->button;
 	pane->drag.last_x = pane->drag.origin_x = event->x;
 	pane->drag.last_y = pane->drag.origin_y = event->y;
@@ -1139,12 +1145,13 @@ cb_control_point_event (FooCanvasItem *ctrl_pt, GdkEvent *event, GnmPane *pane)
 			gnm_pane_drag_begin (pane, so, event);
 		else if (gnm_canvas_handle_motion (GNM_CANVAS (ctrl_pt->canvas),
 						   ctrl_pt->canvas, &event->motion,
-						   GNM_CANVAS_SLIDE_X | GNM_CANVAS_SLIDE_Y | GNM_CANVAS_SLIDE_EXTERIOR_ONLY,
+						   GNM_CANVAS_SLIDE_X | GNM_CANVAS_SLIDE_Y |
+						   GNM_CANVAS_SLIDE_EXTERIOR_ONLY,
 						   cb_slide_handler, ctrl_pt))
 			gnm_pane_object_move (pane, G_OBJECT (ctrl_pt),
-				event->motion.x, event->motion.y,
-				(event->button.state & GDK_CONTROL_MASK) != 0,
-				(event->button.state & GDK_SHIFT_MASK) != 0);
+					      event->motion.x, event->motion.y,
+					      (event->button.state & GDK_CONTROL_MASK) != 0,
+					      (event->button.state & GDK_SHIFT_MASK) != 0);
 		break;
 
 	default:
