@@ -9,10 +9,9 @@
  *
  * Port to Maemo:
  * 	Eduardo Lima  (eduardo.lima@indt.org.br)
- * 	Renato Araujo (renato.filho@indt.org.br) 
  */
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include "gnumeric.h"
 #include "gui-file.h"
 
@@ -28,6 +27,7 @@
 
 #include <goffice/gtk/go-charmap-sel.h>
 #include <goffice/app/io-context.h>
+#include <goffice/app/go-doc.h>
 #include <goffice/utils/go-file.h>
 
 #include <gtk/gtkcombobox.h>
@@ -45,9 +45,7 @@
 #include <string.h>
 
 #ifdef USE_HILDON
-	#include <hildon-lgpl/hildon-widgets/hildon-app.h>
-	#include <hildon-lgpl/hildon-widgets/hildon-appview.h>
-	#include <hildon-fm/hildon-widgets/hildon-file-chooser-dialog.h>
+#include <hildon-fm/hildon-widgets/hildon-file-chooser-dialog.h>
 #endif
 
 typedef struct {
@@ -103,33 +101,9 @@ void
 gui_wb_view_show (WorkbookControlGUI *wbcg, WorkbookView *wbv)
 {
 	WorkbookControlGUI *new_wbcg = NULL;
-	Workbook *tmp_wb = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
+	Workbook *tmp_wb = wb_control_get_workbook (WORKBOOK_CONTROL (wbcg));
 
-#ifdef USE_HILDON
-	if (workbook_is_dirty (tmp_wb)) {
-		switch (wbcg_show_save_dialog (wbcg, tmp_wb, FALSE)) {
-
-			case GTK_RESPONSE_YES:
-			case GNM_RESPONSE_SAVE_ALL:
-				gui_file_save (wbcg, wb_control_view (WORKBOOK_CONTROL (wbcg)));
-				break;
-
-			case GTK_RESPONSE_NO:
-			case GNM_RESPONSE_DISCARD_ALL:
-				/* Do nothing */
-				break;
-			
-			default:  /* CANCEL */
-				return;
-		}
-	}
-
-	g_object_ref (wbcg);
-	g_object_unref (tmp_wb);
-	wb_control_set_view (WORKBOOK_CONTROL (wbcg), wbv, NULL);
-	wb_control_init_state (WORKBOOK_CONTROL (wbcg));
-#else
-	if (workbook_is_pristine (tmp_wb)) {
+	if (go_doc_is_pristine (GO_DOC (tmp_wb))) {
 		g_object_ref (wbcg);
 		g_object_unref (tmp_wb);
 		wb_control_set_view (WORKBOOK_CONTROL (wbcg), wbv, NULL);
@@ -143,7 +117,6 @@ gui_wb_view_show (WorkbookControlGUI *wbcg, WorkbookView *wbv)
 
 		wbcg_copy_toolbar_visibility (new_wbcg, wbcg);
 	}
-#endif
 
 	sheet_update (wb_view_cur_sheet	(wbv));
 }
@@ -220,9 +193,9 @@ gui_file_open (WorkbookControlGUI *wbcg, char const *default_format)
 	gint opener_default;
 	char const *title;
 	char *uri = NULL;
-	const char *encoding = NULL;
+	char const *encoding = NULL;
 	GOFileOpener *fo = NULL;
-	Workbook *workbook = wb_control_workbook (WORKBOOK_CONTROL (wbcg));
+	Workbook *workbook = wb_control_get_workbook (WORKBOOK_CONTROL (wbcg));
 
 	openers = g_list_sort (g_list_copy (get_file_openers ()),
 			       file_opener_description_cmp);
@@ -273,7 +246,7 @@ gui_file_open (WorkbookControlGUI *wbcg, char const *default_format)
 	}
 
 	/* Start in the same directory as the current workbook.  */
-	gtk_file_chooser_select_uri (fsel, workbook_get_uri (workbook));
+	gtk_file_chooser_select_uri (fsel, go_doc_get_uri (GO_DOC (workbook)));
 	gtk_file_chooser_unselect_all (fsel);
 
 	/* Filters */
@@ -325,8 +298,16 @@ gui_file_open (WorkbookControlGUI *wbcg, char const *default_format)
 				  0, 1, 1, 2, GTK_SHRINK | GTK_FILL, GTK_SHRINK, 5, 2);
 		gtk_label_set_mnemonic_widget (GTK_LABEL (data.charmap_label),
 					       go_charmap_sel);
-
+#ifdef USE_HILDON
+		/*
+		 * Don't need to show the vbox. This is here just to avoid the warning :
+		 * GLIB WARNING ** default - hildon-file-chooser-dialog.c:1226: invalid
+		 * property id 4103 for "extra-widget" of type `GParamObject' in `HildonFileChooserDialog'
+		 */
+		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (fsel)->vbox), box, FALSE, TRUE, 6);
+#else
 		gtk_file_chooser_set_extra_widget (fsel, box);
+#endif
 	}
 
 	/* Show file selector */
@@ -366,7 +347,7 @@ check_multiple_sheet_support_if_needed (GOFileSaver *fs,
 			       "in separate files or select different file format.\n"
 			       "Do you want to save only current sheet?");
 
-		sheets = workbook_sheets (wb_view_workbook (wb_view));
+		sheets = workbook_sheets (wb_view_get_workbook (wb_view));
 		if (g_list_length (sheets) > 1) {
 			ret_val = go_gtk_query_yes_no (parent, TRUE, msg);
 		}
@@ -427,8 +408,8 @@ gui_file_save_as (WorkbookControlGUI *wbcg, WorkbookView *wb_view)
 		gtk_file_filter_set_name (filter, _("Spreadsheets"));
 		for (l = savers->next; l; l = l->next) {
 			GOFileSaver *fs = l->data;
-			const char *ext = go_file_saver_get_extension (fs);
-			const char *mime = go_file_saver_get_mime_type (fs);
+			char const *ext = go_file_saver_get_extension (fs);
+			char const *mime = go_file_saver_get_mime_type (fs);
 
 			if (mime)
 				gtk_file_filter_add_mime_type (filter, mime);
@@ -456,20 +437,25 @@ gui_file_save_as (WorkbookControlGUI *wbcg, WorkbookView *wb_view)
 		gtk_box_pack_start (GTK_BOX (box), GTK_WIDGET (format_combo), FALSE, TRUE, 6);
 		gtk_label_set_mnemonic_widget (GTK_LABEL (label), GTK_WIDGET (format_combo));
 
+#ifdef USE_HILDON
+		gtk_widget_show_all (box);
+		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (fsel)->vbox), box, FALSE, TRUE, 6);
+#else
 		gtk_file_chooser_set_extra_widget (fsel, box);
+#endif
 	}
 
 	/* Set default file saver */
 	fs = wbcg->current_saver;
 	if (fs == NULL)
-		fs = workbook_get_file_saver (wb_view_workbook (wb_view));
+		fs = workbook_get_file_saver (wb_view_get_workbook (wb_view));
 	if (fs == NULL || g_list_find (savers, fs) == NULL)
 		fs = go_file_saver_get_default ();
 
 	gtk_combo_box_set_active (format_combo, g_list_index (savers, fs));
 
 	/* Set default file name */
-	wb_uri = workbook_get_uri (wb_view_workbook (wb_view));
+	wb_uri = go_doc_get_uri (GO_DOC (wb_view_get_workbook (wb_view)));
 	if (wb_uri != NULL) {
 		char *basename = go_basename_from_uri (wb_uri);
 		char *dot = basename ? strrchr (basename, '.') : NULL;
@@ -550,7 +536,7 @@ gui_file_save (WorkbookControlGUI *wbcg, WorkbookView *wb_view)
 	                        GTK_WIDGET (wbcg->notebook)->allocation.width,
 	                        GTK_WIDGET (wbcg->notebook)->allocation.height);
 
-	wb = wb_view_workbook (wb_view);
+	wb = wb_view_get_workbook (wb_view);
 	if (wb->file_format_level < FILE_FL_AUTO)
 		return gui_file_save_as (wbcg, wb_view);
 	else
