@@ -7,7 +7,7 @@
  */
 
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include "gnumeric.h"
 #include "libgnumeric.h"
 #include "gui-util.h"
@@ -26,21 +26,9 @@
 
 #include <goffice/app/error-info.h>
 #include <goffice/gtk/go-combo-color.h>
+#include <goffice/gtk/goffice-gtk.h>
 #include <glade/glade.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkfilechooser.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtkeventbox.h>
-#include <gtk/gtkradiobutton.h>
-#include <gtk/gtkalignment.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtkframe.h>
-#include <gtk/gtkwidget.h>
-#include <gtk/gtkimagemenuitem.h>
-#include <gtk/gtkbbox.h>
-#include <gtk/gtkhbox.h>
+#include <gtk/gtk.h>
 #include <atk/atkrelation.h>
 #include <atk/atkrelationset.h>
 #include <gdk/gdkkeysyms.h>
@@ -204,7 +192,7 @@ cb_keyed_dialog_keypress (GtkWidget *dialog, GdkEventKey *event,
  * the object data is managed here.
  **/
 void
-gnumeric_keyed_dialog (WorkbookControlGUI *wbcg, GtkWindow *dialog, const char *key)
+gnumeric_keyed_dialog (WorkbookControlGUI *wbcg, GtkWindow *dialog, char const *key)
 {
 	KeyedDialogContext *ctxt;
 
@@ -214,6 +202,8 @@ gnumeric_keyed_dialog (WorkbookControlGUI *wbcg, GtkWindow *dialog, const char *
 
 	wbcg_set_transient (wbcg, dialog);
 
+	go_dialog_guess_alternative_button_order (GTK_DIALOG (dialog));
+
 	ctxt = g_new (KeyedDialogContext, 1);
 	ctxt->wbcg   = wbcg;
 	ctxt->dialog = GTK_WIDGET (dialog);
@@ -238,7 +228,7 @@ gnumeric_keyed_dialog (WorkbookControlGUI *wbcg, GtkWindow *dialog, const char *
  * Returns TRUE if dialog found, FALSE if not.
  **/
 gpointer
-gnumeric_dialog_raise_if_exists (WorkbookControlGUI *wbcg, const char *key)
+gnumeric_dialog_raise_if_exists (WorkbookControlGUI *wbcg, char const *key)
 {
 	KeyedDialogContext *ctxt;
 
@@ -319,7 +309,7 @@ gtk_radio_group_get_selected (GSList *radio_group)
 
 
 int
-gnumeric_glade_group_value (GladeXML *gui, const char *const group[])
+gnumeric_glade_group_value (GladeXML *gui, char const * const group[])
 {
 	int i;
 	for (i = 0; group[i]; i++) {
@@ -339,14 +329,26 @@ kill_popup_menu (GtkWidget *widget, GtkMenu *menu)
 	g_object_unref (G_OBJECT (menu));
 }
 
+/**
+ * gnumeric_popup_menu :
+ * @menu : #GtkMenu
+ * @event : #GdkEventButton optionally NULL
+ *
+ * Bring up a popup and if @event is non-NULL ensure that the popup is on the
+ * right screen.
+ **/
 void
 gnumeric_popup_menu (GtkMenu *menu, GdkEventButton *event)
 {
 	g_return_if_fail (menu != NULL);
 	g_return_if_fail (GTK_IS_MENU (menu));
 
+#if GLIB_CHECK_VERSION(2,10,0) && GTK_CHECK_VERSION(2,8,14)
+	g_object_ref_sink (menu);
+#else
 	g_object_ref (menu);
 	gtk_object_sink (GTK_OBJECT (menu));
+#endif
 
 	if (event)
 		gtk_menu_set_screen (menu,
@@ -639,11 +641,6 @@ entry_to_float_with_format (GtkEntry *entry, gnm_float *the_float, gboolean upda
 	if (!value)
 		return TRUE;
 
-	if (!VALUE_IS_NUMBER (value)) {
-		value_release (value);
-		return TRUE;
-	}
-
 	*the_float = value_get_as_float (value);
 	if (update) {
 		char *tmp = format_value (format, value, NULL, 16, NULL);
@@ -661,24 +658,25 @@ entry_to_float_with_format (GtkEntry *entry, gnm_float *the_float, gboolean upda
  * @the_int:
  * @update:
  *
- * retrieve an int from an entry field parsing all reasonable formats
+ * Retrieve an int from an entry field parsing all reasonable formats
  *
  **/
 gboolean
 entry_to_int (GtkEntry *entry, gint *the_int, gboolean update)
 {
 	GnmValue *value = format_match_number (gtk_entry_get_text (entry), NULL, NULL);
+	gnm_float f;
 
 	*the_int = 0;
 	if (!value)
 		return TRUE;
 
-	if (value->type != VALUE_INTEGER) {
+	f = value_get_as_float (value);
+	if (f < INT_MIN || f > INT_MAX || f != (*the_int = (int)f)) {
 		value_release (value);
 		return TRUE;
 	}
 
-	*the_int = value_get_as_int (value);
 	if (update) {
 		char *tmp = format_value (NULL, value, NULL, 16, NULL);
 		gtk_entry_set_text (entry, tmp);
@@ -760,7 +758,7 @@ gnumeric_load_pixbuf (char const *filename)
  * The fractional tiles are spead evenly left-right and top-bottom.
  */
 GdkPixbuf *
-gnm_pixbuf_tile (const GdkPixbuf *src, int w, int h)
+gnm_pixbuf_tile (GdkPixbuf const *src, int w, int h)
 {
 	int src_w = gdk_pixbuf_get_width (src);
 	int src_h = gdk_pixbuf_get_height (src);
@@ -959,13 +957,12 @@ gnumeric_message_dialog_new (GtkWindow * parent,
 		gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
 		gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
 		gtk_misc_set_alignment (GTK_MISC (label), 0.0 , 0.0);
-		gtk_misc_set_alignment (GTK_MISC (label), 0.0 , 0.0);
 		gtk_box_set_spacing (GTK_BOX (hbox), 12);
 		gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
 		gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 12);
 		gtk_container_set_border_width (GTK_CONTAINER (dialog), 6);
 		gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-		gtk_window_set_resizable (GTK_WINDOW(dialog), FALSE);
+		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 		gtk_widget_show_all (GTK_WIDGET (GTK_DIALOG (dialog)->vbox));
 	}
 
