@@ -45,8 +45,8 @@ typedef enum {
 		MSTYLE_WRAP_TEXT,		/* marks a header */
 		MSTYLE_SHRINK_TO_FIT,		/* marks a header */
 
-	        MSTYLE_CONTENT_LOCKED,
-	        MSTYLE_CONTENT_HIDDEN,
+	        MSTYLE_CONTENTS_LOCKED,
+	        MSTYLE_CONTENTS_HIDDEN,
 
 	/* Things not in MS Excel's Style */
 	        MSTYLE_VALIDATION,
@@ -59,12 +59,12 @@ typedef enum {
 
 GnmStyle   *gnm_style_new           (void);
 GnmStyle   *gnm_style_new_default   (void);
+GnmStyle   *gnm_style_new_merged    (GnmStyle const *orig, GnmStyle const *overlay);
 GnmStyle   *gnm_style_dup	    (GnmStyle const *src);
-GnmStyle   *gnm_style_merge	    (GnmStyle const *orig, GnmStyle const *overlay);
 void        gnm_style_merge_element (GnmStyle *dst, GnmStyle const *src,
 				     GnmStyleElement elem);
-void        gnm_style_ref           (GnmStyle *style);
-void        gnm_style_unref         (GnmStyle *style);
+void        gnm_style_ref           (GnmStyle const *style);
+void        gnm_style_unref         (GnmStyle const *style);
 
 GnmStyle   *gnm_style_link_sheet    (GnmStyle *style, Sheet *sheet);
 void        gnm_style_link          (GnmStyle *style);
@@ -79,9 +79,10 @@ gboolean    gnm_style_equal_header  (GnmStyle const *a, GnmStyle const *b,
 guint       gnm_style_hash          (gconstpointer style);
 guint       gnm_style_hash_XL	    (gconstpointer style);
 
-gboolean     gnm_style_is_element_set	   (GnmStyle const *style, GnmStyleElement elem);
 unsigned int gnm_style_find_conflicts      (GnmStyle *accum, GnmStyle const *overlay,
 					    unsigned int conflicts);
+gboolean     gnm_style_is_complete    	   (GnmStyle const *style);
+gboolean     gnm_style_is_element_set	   (GnmStyle const *style, GnmStyleElement elem);
 void         gnm_style_unset_element	   (GnmStyle *style, GnmStyleElement elem);
 void         gnm_style_set_font_color	   (GnmStyle *style, GnmColor *col);
 void         gnm_style_set_back_color	   (GnmStyle *style, GnmColor *col);
@@ -94,7 +95,7 @@ void         gnm_style_set_border	   (GnmStyle *style, GnmStyleElement elem,
 GnmBorder   *gnm_style_get_border	   (GnmStyle const *style, GnmStyleElement elem);
 void         gnm_style_set_pattern	   (GnmStyle *style, int pattern);
 int          gnm_style_get_pattern	   (GnmStyle const *style);
-void         gnm_style_set_font_name	   (GnmStyle *style, const char *name);
+void         gnm_style_set_font_name	   (GnmStyle *style, char const *name);
 char const  *gnm_style_get_font_name	   (GnmStyle const *style);
 void         gnm_style_set_font_bold	   (GnmStyle *style, gboolean bold);
 gboolean     gnm_style_get_font_bold	   (GnmStyle const *style);
@@ -135,10 +136,10 @@ gboolean	 gnm_style_get_effective_wrap_text   (GnmStyle const *style);
 void		 gnm_style_set_shrink_to_fit (GnmStyle *style, gboolean f);
 gboolean	 gnm_style_get_shrink_to_fit (GnmStyle const *style);
 
-void		 gnm_style_set_content_locked (GnmStyle *style, gboolean f);
-gboolean	 gnm_style_get_content_locked (GnmStyle const *style);
-void		 gnm_style_set_content_hidden (GnmStyle *style, gboolean f);
-gboolean	 gnm_style_get_content_hidden (GnmStyle const *style);
+void		 gnm_style_set_contents_locked (GnmStyle *style, gboolean f);
+gboolean	 gnm_style_get_contents_locked (GnmStyle const *style);
+void		 gnm_style_set_contents_hidden (GnmStyle *style, gboolean f);
+gboolean	 gnm_style_get_contents_hidden (GnmStyle const *style);
 
 void		 gnm_style_set_validation	(GnmStyle *style, GnmValidation *v);
 GnmValidation	*gnm_style_get_validation	(GnmStyle const *style);
