@@ -1,5 +1,5 @@
-#ifndef GNUMERIC_PARSE_UTIL_H
-# define GNUMERIC_PARSE_UTIL_H
+#ifndef GNM_PARSE_UTIL_H
+# define GNM_PARSE_UTIL_H
 
 #include "gnumeric.h"
 
@@ -80,12 +80,14 @@ typedef char const *(*GnmRangeRefParse) (GnmRangeRef *res, char const *in,
 					 GnmParsePos const *pp,
 					 GnmExprConventions const *convs);
 					 /* GError **err); */
+typedef char const *(*GnmNameParse) (char const *in,
+				     GnmExprConventions const *convs);
 
 /*
  * On success, this functions should return a non-NULL value and
  * absorb the args, including the list.
  */
-typedef GnmExpr const *(*GnmParseFunctionHandler) (const char *name,
+typedef GnmExpr const *(*GnmParseFunctionHandler) (char const *name,
 						   GnmExprList *args,
 						   GnmExprConventions const *convs);
 
@@ -105,6 +107,9 @@ typedef void (*GnmParseRangeRefHandler) (GString *target,
 					 GnmRangeRef const *cell_ref,
 					 GnmParsePos const *pp);
 
+typedef GString * (*GnmSheetNameQuoteHandler) (GnmExprConventions const *convs,
+					       char const *name);
+
 struct _GnmExprConventions {
 #if 0
 	/* Not yet.  */
@@ -114,6 +119,9 @@ struct _GnmExprConventions {
 #endif
 	gboolean r1c1_addresses;
 
+	/* What character denotes range intersection?  */
+	gunichar intersection_char;
+
 	/* What characters are range separators?  */
 	gboolean range_sep_colon;  /* A1:B2 */
 	gboolean range_sep_dotdot; /* A1..B2 */
@@ -122,9 +130,6 @@ struct _GnmExprConventions {
 	gboolean sheet_sep_exclamation;  /* Sheet!... */
 	gboolean sheet_sep_colon; /* Sheet:... */
 
-	/* Is ERROR.TYPE allowed?  */
-	gboolean dots_in_names;
-
 	/* Formerly USE_APPLIX_CONVENTIONS.  */
 	gboolean ignore_whitespace;
 
@@ -149,6 +154,10 @@ struct _GnmExprConventions {
 	/* Called a lot for anything that might be a reference.  */
 	GnmRangeRefParse ref_parser;
 
+	/* Called a lot for anything that might be a function name or defined
+	   name.  */
+	GnmNameParse name_parser;
+
 	/*
 	 * Optional name->GnmParseFunctionHandler hash.  When a name is
 	 * present in this hash, unknown_function_handler will not be
@@ -170,6 +179,9 @@ struct _GnmExprConventions {
 	/* Called to make strings of range refs.  */
 	GnmParseRangeRefHandler range_ref_handler;
 
+	/* Called to optionally quote a sheet name.  */
+	GnmSheetNameQuoteHandler sheet_name_quote;
+
 	/* Used to separate sheet from name when both are needed.  */
 	char const *output_sheet_name_sep;
 
@@ -185,7 +197,8 @@ struct _GnmExprConventions {
 	gboolean output_translated;
 };
 
-GnmExprConventions *gnm_expr_conventions_new (void);
+GnmExprConventions *gnm_expr_conventions_new	  (void);
+GnmExprConventions *gnm_expr_conventions_new_full (unsigned size);
 void gnm_expr_conventions_free (GnmExprConventions *c);
 
 extern GnmExprConventions const *gnm_expr_conventions_default;
@@ -193,20 +206,24 @@ extern GnmExprConventions const *gnm_expr_conventions_r1c1;
 void parse_util_init (void);
 void parse_util_shutdown (void);
 
-GnmExpr const *gnm_expr_parse_str (char const *expr, GnmParsePos const *pp,
-				   GnmExprParseFlags flags,
-				   GnmExprConventions const *convs,
-				   GnmParseError *error);
+GnmExprTop const *gnm_expr_parse_str (char const *expr, GnmParsePos const *pp,
+				      GnmExprParseFlags flags,
+				      GnmExprConventions const *convs,
+				      GnmParseError *error);
 
-GnmExpr const *gnm_expr_parse_str_simple (char const *expr, GnmParsePos const *pp);
+GnmExprTop const *gnm_expr_parse_str_simple (char const *expr,
+					     GnmParsePos const *pp);
 
 /* Is this string potentially the start of an expression */
 char const *gnm_expr_char_start_p (char const *c);
 
 void	    parse_text_value_or_expr (GnmParsePos const *pos,
 				      char const *text,
-				      GnmValue **val, GnmExpr const **expr,
+				      GnmValue **val,
+				      GnmExprTop const **texpr,
 				      GOFormat *current_format,
 				      GODateConventions const *date_conv);
 
-#endif /* GNUMERIC_PARSE_UTIL_H */
+GString	*gnm_expr_conv_quote (GnmExprConventions const *conv, char const *str);
+
+#endif /* GNM_PARSE_UTIL_H */
