@@ -3,7 +3,7 @@
  * print-cell.c: Printing of cell regions and cells.
  *
  * Author:
- *    Jody Goldberg 2000-2002	(jody@gnome.org)
+ *    Jody Goldberg 2000-2006	(jody@gnome.org)
  *    Miguel de Icaza 1999 (miguel@kernel.org)
  */
 #include <gnumeric-config.h>
@@ -83,10 +83,13 @@ print_cell (GnmCell const *cell, GnmStyle const *mstyle,
 	RenderedValue *rv, *cell_rv = cell->rendered_value, *cell_rv100 = NULL;
 	GOColor fore_color;
 	gint x, y;
-	ColRowInfo const * const ci = cell->col_info;
-	ColRowInfo const * const ri = cell->row_info;
 	Sheet *sheet = cell->base.sheet;
 
+	/* Get the sizes exclusive of margins and grids */
+	/* Note: +1 because size_pixels includes leading gridline.  */
+	height -= GNM_ROW_MARGIN + GNM_ROW_MARGIN + 1;
+	width  -= GNM_COL_MARGIN + GNM_COL_MARGIN + 1;
+
 	/* Create a rendered value for printing */
 	if (sheet->last_zoom_factor_used != 1) {
 		/*
@@ -94,11 +97,10 @@ print_cell (GnmCell const *cell, GnmStyle const *mstyle,
 		 * Simply create a new RenderedValue at zoom 100% for the
 		 * _screen_ context.
 		 */
-		cell_rv100 =
-			rendered_value_new ((GnmCell *)cell, mstyle,
-					    cell_rv->variable_width,
-					    pango_layout_get_context (cell_rv->layout),
-					    1.0);
+		cell_rv100 = rendered_value_new ((GnmCell *)cell, mstyle,
+			cell_rv->variable_width,
+			pango_layout_get_context (cell_rv->layout),
+			1.0);
 		cell_rv = cell_rv100;
 	}
 
@@ -106,7 +108,7 @@ print_cell (GnmCell const *cell, GnmStyle const *mstyle,
 	 * Since some layout decisions are taken during cell_calc_layout
 	 * we need to make sure that has been called.
 	 */
-	cell_finish_layout (cell, cell_rv, FALSE);
+	cell_finish_layout (cell, cell_rv, width, FALSE);
 
 	/* Now pretend it was made for printing.  */
 	rv = rendered_value_recontext (cell_rv, pcontext);
@@ -123,8 +125,8 @@ print_cell (GnmCell const *cell, GnmStyle const *mstyle,
 			      (int)(height * PANGO_SCALE),
 			      (int)h_center == -1 ? -1 : (int)(h_center * PANGO_SCALE),
 			      &fore_color, &x, &y)) {
-		double x0 = x1 + 1 + ci->margin_a;
-		double y0 = y1 - (1 + ri->margin_a);
+		double x0 = x1 + (1 + GNM_COL_MARGIN);
+		double y0 = y1 - (1 + GNM_ROW_MARGIN);
 		double px = x1 + x / (double)PANGO_SCALE;
 		double py = y1 + y / (double)PANGO_SCALE;
 
@@ -267,17 +269,16 @@ print_merged_range (GnomePrintContext *context, PangoContext *pcontext,
 
 	if (cell != NULL) {
 		ColRowInfo const * const ri = cell->row_info;
-		ColRowInfo const * const ci = cell->col_info;
 
 		if (ri->needs_respan)
-			row_calc_spans ((ColRowInfo *)ri, sheet);
+			row_calc_spans ((ColRowInfo *)ri, cell->pos.row, sheet);
 
-		/* FIXME : get the margins from the far col/row too */
-		print_cell (cell, style, context, pcontext,
-			    l, t,
-			    r - l - (ci->margin_b + ci->margin_a),
-			    t - b - (ri->margin_b + ri->margin_a),
-			    -1.);
+		if (sheet->text_is_rtl)
+			print_cell (cell, style, context, pcontext,
+				r, t, l - r, t - b, -1.);
+		else
+			print_cell (cell, style, context, pcontext,
+				l, t, r - l, t - b, -1.);
 	}
 	style_border_print_diag (style, context, l, t, r, b);
 }
@@ -397,7 +398,7 @@ print_cell_range (GnomePrintContext *context,
 		 * will ever get flagged.
 		 */
 		if (ri->needs_respan)
-			row_calc_spans ((ColRowInfo *)ri, sheet);
+			row_calc_spans ((ColRowInfo *)ri, row, sheet);
 
 		/* look for merges that start on this row, on the first painted row
 		 * also check for merges that start above. */
@@ -513,20 +514,17 @@ print_cell_range (GnomePrintContext *context,
 
 			/* Is this part of a span?
 			 * 1) There are cells allocated in the row
-			 *       (indicated by ri->pos != -1)
+			 *       (indicated by ri->spans != NULL)
 			 * 2) Look in the rows hash table to see if
 			 *    there is a span descriptor.
 			 */
-			if (ri->pos == -1 || NULL == (span = row_span_get (ri, col))) {
+			if (NULL == ri->spans || NULL == (span = row_span_get (ri, col))) {
 
 				/* no need to draw blanks */
 				GnmCell const *cell = sheet_cell_get (sheet, col, row);
 				if (!cell_is_empty (cell))
 					print_cell (cell, style, context, pcontext,
-						    x, y,
-						    ci->size_pts - (ci->margin_b + ci->margin_a),
-						    ri->size_pts - (ri->margin_b + ri->margin_a),
-						    -1.);
+						x, y, ci->size_pts, ri->size_pts, -1.);
 
 			/* Only draw spaning cells after all the backgrounds
 			 * that we are going to draw have been drawn.  No need
@@ -537,10 +535,10 @@ print_cell_range (GnomePrintContext *context,
 				int const start_span_col = span->left;
 				int const end_span_col = span->right;
 				double real_x = x;
-				double center_offset = cell->col_info->size_pts / 2;
-				/* TODO : Use the spanning margins */
-				double tmp_width = ci->size_pts -
-					ci->margin_b - ci->margin_a;
+				ColRowInfo const *cell_col =
+					sheet_col_get_info (sheet, cell->pos.col);
+				double center_offset = cell_col->size_pts / 2;
+				double tmp_width = ci->size_pts;
 
 				if (col != cell->pos.col)
 					style = sheet_style_get (sheet,
@@ -571,10 +569,7 @@ print_cell_range (GnomePrintContext *context,
 				}
 
 				print_cell (cell, style, context, pcontext,
-					    real_x, y,
-					    tmp_width,
-					    ri->size_pts - (ri->margin_b + ri->margin_a),
-					    center_offset);
+					real_x, y, tmp_width, ri->size_pts, center_offset);
 			} else if (col != span->left)
 				sr.vertical [col] = NULL;
 
