@@ -39,6 +39,7 @@
 
 #include <string.h>
 #include <goffice/utils/go-glib-extras.h>
+#include <goffice/utils/go-font.h>
 
 #undef DEBUG_BOUNDING_BOX
 
@@ -66,7 +67,7 @@ calc_indent (PangoContext *context, const GnmStyle *mstyle, double zoom)
 		int n = gnm_style_get_indent (mstyle);
 		if (n) {
 			GnmFont *style_font = gnm_style_get_font (mstyle, context, zoom);
-			indent = PANGO_PIXELS (n * style_font->metrics->avg_digit_width);
+			indent = PANGO_PIXELS (n * style_font->go.metrics->avg_digit_width);
 		}
 	}
 	return MIN (indent, 65535);
@@ -370,16 +371,15 @@ rendered_value_new (GnmCell *cell, GnmStyle const *mstyle,
 				col_width_pixels = sheet_col_get_distance_pixels
 					(sheet,
 					 merged->start.col, merged->end.col + 1);
-			} else
-				col_width_pixels = cell->col_info->size_pixels;
-			/* This probably isn't right for the merged case */
-			col_width_pixels -= (cell->col_info->margin_a +
-					     cell->col_info->margin_b +
-					     1);
+			} else {
+				ColRowInfo const *ci = sheet_col_get_info (sheet, cell->pos.col);
+				col_width_pixels = ci->size_pixels;
+			}
+			col_width_pixels -= (GNM_COL_MARGIN + GNM_COL_MARGIN + 1);
 			col_width = col_width_pixels * PANGO_SCALE;
 		}
 
-		gnm_format_layout (layout, font->metrics, format, cell->value,
+		gnm_format_layout (layout, font->go.metrics, format, cell->value,
 				   &fore, col_width, date_conv, TRUE);
 	}
 
@@ -472,7 +472,7 @@ rendered_value_recontext (RenderedValue *rv, PangoContext *context)
 
 /* Return the value as a single string without format infomation.
  */
-const char *
+char const *
 rendered_value_get_text (RenderedValue const *rv)
 {
 	g_return_val_if_fail (rv != NULL, "ERROR");
