@@ -74,10 +74,12 @@ typedef struct {
 	void (*default_size)	 (SheetObject const *so,
 				  double *width_pts, double *height_pts);
 
+	void (*draw_cairo)		(SheetObject const *so, gpointer data,
+				double width, double height);
+
 	void (*func_pad1)	(void);
 	void (*func_pad2)	(void);
 	void (*func_pad3)	(void);
-	void (*func_pad4)	(void);
 
 	gboolean rubber_band_directly; /* If false, we draw a rectangle where
 					* the object is going to be layed out
@@ -142,7 +144,7 @@ struct _SheetObjectImageableIface {
 
 	GtkTargetList *(*get_target_list) (SheetObject const *so);
 	void           (*write_image)	  (SheetObject const *so, 
-					   const char *format,
+					   char const *format,
 					   double resolution,
 					   GsfOutput *output, 
 					   GError **err);
@@ -155,7 +157,7 @@ struct _SheetObjectExportableIface {
 
 	GtkTargetList *(*get_target_list) (SheetObject const *so);
 	void           (*write_object)	  (SheetObject const *so, 
-					   const char *format,
+					   char const *format,
 					   GsfOutput *output, GError **err);
 };
 
