@@ -48,6 +48,7 @@ typedef struct {
 	gunichar             stringindicator;       /* String indicator */
 	gboolean             indicator_2x_is_single;/* 2 quote chars form a single non-terminating quote */
 	gboolean             duplicates;            /* See two text separators as one? */
+	gboolean             trim_seps;             /* Ignore initial seps.  */
      
 	/* Fixed width related */
 	GArray              *splitpositions;        /* Positions where text will be split vertically */
@@ -68,7 +69,7 @@ typedef struct {
 StfParseOptions_t  *stf_parse_options_new                             (void);
 void                stf_parse_options_free                            (StfParseOptions_t *parseoptions);
 
-StfParseOptions_t  *stf_parse_options_guess                           (const char *data);
+StfParseOptions_t  *stf_parse_options_guess                           (char const *data);
 
 /* MANIPULATION of stf options struct */
 
@@ -77,8 +78,6 @@ void stf_parse_options_set_type                        (StfParseOptions_t *parse
 void stf_parse_options_clear_line_terminator           (StfParseOptions_t *parseoptions);
 void stf_parse_options_add_line_terminator             (StfParseOptions_t *parseoptions,
 							char const *terminator);
-void stf_parse_options_remove_line_terminator          (StfParseOptions_t *parseoptions,
-							char const *terminator);
 void stf_parse_options_set_trim_spaces                 (StfParseOptions_t *parseoptions,
 							StfTrimType_t const trim_spaces);
 void stf_parse_options_csv_set_separators              (StfParseOptions_t *parseoptions,
@@ -89,6 +88,8 @@ void stf_parse_options_csv_set_indicator_2x_is_single  (StfParseOptions_t *parse
 							gboolean const indic_2x);
 void stf_parse_options_csv_set_duplicates              (StfParseOptions_t *parseoptions,
 							gboolean const duplicates);
+void stf_parse_options_csv_set_trim_seps               (StfParseOptions_t *parseoptions,
+							gboolean const trim_seps);
 void stf_parse_options_fixed_splitpositions_clear      (StfParseOptions_t *parseoptions);
 void stf_parse_options_fixed_splitpositions_add        (StfParseOptions_t *parseoptions,
 							int position);
@@ -99,35 +100,37 @@ int stf_parse_options_fixed_splitpositions_nth         (StfParseOptions_t *parse
 
 /* USING the stf structs to actually do some parsing, these are the lower-level functions and utility functions */
 
-GPtrArray          *stf_parse_general                                 (StfParseOptions_t *parseoptions,
-								       GStringChunk *lines_chunk,
-								       char const *data,
-								       char const *data_end);
-void                stf_parse_general_free                            (GPtrArray *lines);
-GPtrArray          *stf_parse_lines                                   (StfParseOptions_t *parseoptions,
-								       GStringChunk *lines_chunk,
-								       const char *data,
-								       int maxlines,
-								       gboolean with_lineno);
+GPtrArray	*stf_parse_general			(StfParseOptions_t *parseoptions,
+							 GStringChunk *lines_chunk,
+							 char const *data,
+							 char const *data_end);
+void		 stf_parse_general_free			(GPtrArray *lines);
+GPtrArray	*stf_parse_lines			(StfParseOptions_t *parseoptions,
+							 GStringChunk *lines_chunk,
+							 char const *data,
+							 int maxlines,
+							 gboolean with_lineno);
 
-void                stf_parse_options_fixed_autodiscover              (StfParseOptions_t *parseoptions,
-								       char const *data,
-								       char const *data_end);
+void		 stf_parse_options_fixed_autodiscover	(StfParseOptions_t *parseoptions,
+							 char const *data,
+							 char const *data_end);
 
-const char         *stf_parse_find_line                               (StfParseOptions_t *parseoptions,
-								       const char *data,
-								       int line);
+char const	*stf_parse_find_line			(StfParseOptions_t *parseoptions,
+							 char const *data,
+							 int line);
 
-char const         *stf_parse_next_token                              (char const *data, 
-								       StfParseOptions_t *parseoptions,
-								       StfTokenType_t *tokentype);
+char const	*stf_parse_next_token			(char const *data, 
+							 StfParseOptions_t *parseoptions,
+							 StfTokenType_t *tokentype);
 
 /* Higher level functions, can be used for directly parsing into an application specific data container */
-gboolean	    stf_parse_sheet                                   (StfParseOptions_t *parseoptions,
-								       char const *data, char const *data_end,
-								       Sheet *sheet,
-								       int start_col, int start_row);
+gboolean	 stf_parse_sheet			(StfParseOptions_t *parseoptions,
+							 char const *data, char const *data_end,
+							 Sheet *sheet,
+							 int start_col, int start_row);
 
-GnmCellRegion         *stf_parse_region                                  (StfParseOptions_t *parseoptions,
-								       char const *data, char const *data_end);
-#endif
+GnmCellRegion	*stf_parse_region			(StfParseOptions_t *parseoptions,
+							 char const *data, char const *data_end,
+							 Workbook const *wb);
+
+#endif /* GNUMERIC_STF_PARSE_H */
