@@ -43,10 +43,10 @@ cond_validate (GnmStyleCond const *cond)
 {
 	g_return_val_if_fail (cond != NULL, FALSE);
 	g_return_val_if_fail (cond->overlay != NULL, FALSE);
-	g_return_val_if_fail (cond->expr[0] != NULL, FALSE);
+	g_return_val_if_fail (cond->texpr[0] != NULL, FALSE);
 	g_return_val_if_fail ((cond->op == GNM_STYLE_COND_BETWEEN ||
 			   cond->op == GNM_STYLE_COND_NOT_BETWEEN) ^
-			  (cond->expr[1] == NULL), FALSE);
+			  (cond->texpr[1] == NULL), FALSE);
 
 	return TRUE;
 }
@@ -57,10 +57,10 @@ cond_unref (GnmStyleCond const *cond)
 	/* Be very delicate, this is called for invalid conditions too */
 	if (cond->overlay)
 		gnm_style_unref (cond->overlay);
-	if (cond->expr[0])
-		gnm_expr_unref (cond->expr[0]);
-	if (cond->expr[1])
-		gnm_expr_unref (cond->expr[1]);
+	if (cond->texpr[0])
+		gnm_expr_top_unref (cond->texpr[0]);
+	if (cond->texpr[1])
+		gnm_expr_top_unref (cond->texpr[1]);
 }
 
 static void
@@ -163,7 +163,7 @@ gnm_style_conditions_overlay (GnmStyleConditions const *sc,
 	res = g_ptr_array_sized_new (sc->conditions->len);
 	for (i = 0 ; i < sc->conditions->len; i++) {
 		overlay = g_array_index (sc->conditions, GnmStyleCond, i).overlay;
-		merge = gnm_style_merge (base, overlay);
+		merge = gnm_style_new_merged (base, overlay);
 		/* We only draw a background colour is the pattern != 0 */
 		if (merge->pattern == 0 &&
 		     elem_is_set (overlay, MSTYLE_COLOR_BACK) &&
@@ -193,7 +193,7 @@ gnm_style_conditions_eval (GnmStyleConditions const *sc, GnmEvalPos const *ep)
 	for (i = 0 ; i < conds->len ; i++) {
 		cond = &g_array_index (conds, GnmStyleCond, i);
 
-		val = gnm_expr_eval (cond->expr[0], ep, GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
+		val = gnm_expr_top_eval (cond->texpr[0], ep, GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
 		if (cond->op == GNM_STYLE_COND_CUSTOM) {
 			use_this = value_get_as_bool (val, NULL);
 #if 0
@@ -202,7 +202,7 @@ gnm_style_conditions_eval (GnmStyleConditions const *sc, GnmEvalPos const *ep)
 			g_print ("'%s' = %s\n", str, use_this ? "true" : "false");
 			g_free (str);
 #endif
-		} else {
+		} else if (cond->op < GNM_STYLE_COND_CONTAINS_STR) {
 			GnmCell const *cell = sheet_cell_get (ep->sheet, ep->eval.col, ep->eval.row);
 			GnmValue const *cv = cell ? cell->value : NULL;
 			GnmValDiff diff = value_compare (cv, val, TRUE);
@@ -215,8 +215,10 @@ gnm_style_conditions_eval (GnmStyleConditions const *sc, GnmEvalPos const *ep)
 				if (diff != IS_LESS)
 					break;
 				value_release (val);
-				val = gnm_expr_eval (cond->expr[1], ep, GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
+				val = gnm_expr_top_eval (cond->texpr[1], ep, GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
 				diff = value_compare (cv, val, TRUE);
+				/* fall through */
+
 			case GNM_STYLE_COND_GT:		use_this = (diff == IS_GREATER); break;
 			case GNM_STYLE_COND_LT:		use_this = (diff == IS_LESS); break;
 			case GNM_STYLE_COND_GTE:	use_this = (diff != IS_LESS); break;
@@ -224,11 +226,36 @@ gnm_style_conditions_eval (GnmStyleConditions const *sc, GnmEvalPos const *ep)
 				if (diff == IS_LESS)
 					break;
 				value_release (val);
-				val = gnm_expr_eval (cond->expr[1], ep, GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
+				val = gnm_expr_top_eval (cond->texpr[1], ep, GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
 				diff = value_compare (cv, val, TRUE);
 				/* fall through */
 			case GNM_STYLE_COND_LTE:	use_this = (diff != IS_GREATER); break;
 			}
+		} else {
+#if 0
+			GnmCell const *cell = sheet_cell_get (ep->sheet, ep->eval.col, ep->eval.row);
+			/* TODO write this */
+			GnmValue const *cv = cell ? cell->value : NULL;
+#endif
+
+			switch (cond->op) {
+			default :
+			case GNM_STYLE_COND_CONTAINS_STR :
+			case GNM_STYLE_COND_NOT_CONTAINS_STR :
+				break;
+			case GNM_STYLE_COND_BEGINS_WITH_STR :
+			case GNM_STYLE_COND_NOT_BEGINS_WITH_STR :
+				break;
+			case GNM_STYLE_COND_ENDS_WITH_STR :
+			case GNM_STYLE_COND_NOT_ENDS_WITH_STR :
+				break;
+			case GNM_STYLE_COND_CONTAINS_ERR :
+			case GNM_STYLE_COND_NOT_CONTAINS_ERR :
+				break;
+			case GNM_STYLE_COND_CONTAINS_BLANKS :
+			case GNM_STYLE_COND_NOT_CONTAINS_BLANKS :
+				break;
+			}
 		}
 
 		value_release (val);
