@@ -7,7 +7,7 @@
  *  (C) 1998-2004 Miguel de Icaza
  */
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include "gnumeric.h"
 #include "style.h"
 #include "style-font.h"
@@ -27,6 +27,7 @@
 #include <gdk/gdkpango.h>
 #include <gtk/gtkmain.h>
 #include <string.h>
+#include <goffice/utils/go-font.h>
 
 #undef DEBUG_REF_COUNT
 #undef DEBUG_FONTS
@@ -51,7 +52,7 @@ get_substitute_font (gchar const *fontname)
 {
 	int i;
 
-	static char const *map[][2] = {
+	static char const * const map[][2] = {
 		{ "Times New Roman", "Times"},
 		{ "Tms Rmn",	     "Times"},
 		{ "Arial",           "Sans"},
@@ -74,44 +75,6 @@ get_substitute_font (gchar const *fontname)
 	return NULL;
 }
 
-static int
-style_font_string_width (PangoLayout *layout, char const *str)
-{
-	int w;
-	pango_layout_set_text (layout, str, -1);
-	pango_layout_get_pixel_size (layout, &w, NULL);
-	return w;
-}
-
-static double
-calc_font_width (PangoLayout *layout, char const *teststr)
-{
-	char const *p1, *p2;
-	int w = 0, w1, w2, dw;
-	char buf[3];
-
-	for (p1 = teststr; *p1; p1++) {
-		buf[0] = *p1;
-		buf[1] = 0;
-		w1 = style_font_string_width (layout, buf);
-		for (p2 = teststr; *p2; p2++) {
-			buf[1] = *p2;
-			buf[2] = 0;
-			w2 = style_font_string_width (layout, buf);
-			dw = w2 - w1;
-			if (dw > w) {
-				w = dw;
-#ifdef DEBUG_FONT_WIDTH
-				fprintf (stderr, "   %s = %d", buf, w);
-#endif
-			}
-		}
-	}
-
-	return w;
-}
-
-
 static GnmFont *
 style_font_new_simple (PangoContext *context,
 		       char const *font_name, double size_pts, double scale,
@@ -138,11 +101,7 @@ style_font_new_simple (PangoContext *context,
 
 	font = (GnmFont *) g_hash_table_lookup (style_font_hash, &key);
 	if (font == NULL) {
-		PangoFontDescription	*desc;
-		PangoFontMetrics	*metrics;
-		PangoLayout		*layout;
-		double pts_scale;
-		int height;
+		PangoFontDescription *desc;
 
 		if (g_hash_table_lookup (style_font_negative_hash, &key))
 			return NULL;
@@ -156,7 +115,8 @@ style_font_new_simple (PangoContext *context,
 		/* One reference for the cache, one for the caller. */
 		font->ref_count = 2;
 
-		desc = pango_context_get_font_description (context);
+		desc = pango_font_description_copy (pango_context_get_font_description (context));
+
 		pango_font_description_set_family (desc, font_name);
 		pango_font_description_set_weight (desc,
 			bold ? PANGO_WEIGHT_BOLD : PANGO_WEIGHT_NORMAL);
@@ -175,42 +135,15 @@ style_font_new_simple (PangoContext *context,
 			}
 
 			if (font->pango.font == NULL) {
-				font->pango.font_descr = NULL;
+				pango_font_description_free (desc);
 				g_hash_table_insert (style_font_negative_hash,
 						     font, font);
 				return NULL;
 			}
 		}
 
-		font->pango.font_descr = pango_font_description_copy (desc);
-
-		metrics = pango_font_get_metrics (font->pango.font,
-				gtk_get_default_language ());
-		height = pango_font_metrics_get_ascent (metrics) +
-			 pango_font_metrics_get_descent (metrics);
-		font->height = PANGO_PIXELS (height);
-		pango_font_metrics_unref (metrics);
-
-		layout = pango_layout_new (context);
-		pango_layout_set_font_description (layout,
-			font->pango.font_descr);
-
-		pts_scale = 72. / (gnm_app_display_dpi_get (TRUE) * scale);
-		/* TODO : ?? Can we pango_font_metrics_get_approximate_digit_width */
-		font->approx_width.pts.digit = pts_scale *
-			(font->approx_width.pixels.digit = calc_font_width (layout, "0123456789"));
-		font->approx_width.pts.decimal = pts_scale *
-			(font->approx_width.pixels.decimal = calc_font_width (layout, ".,"));
-		font->approx_width.pixels.hash = pts_scale *
-			(font->approx_width.pixels.hash = calc_font_width (layout, "#"));
-		font->approx_width.pixels.sign = pts_scale *
-			(font->approx_width.pixels.sign = calc_font_width (layout, "-+"));
-		font->approx_width.pixels.E = pts_scale *
-			(font->approx_width.pixels.E = calc_font_width (layout, "E"));
-		font->approx_width.pixels.e = pts_scale *
-			(font->approx_width.pixels.e = calc_font_width (layout, "e"));
-		g_object_unref (layout);
-
+		font->go.font = go_font_new_by_desc (desc);
+		font->go.metrics = go_font_metrics_new (context, font->go.font);
 		g_hash_table_insert (style_font_hash, font, font);
 	} else
 		font->ref_count++;
@@ -303,9 +236,13 @@ style_font_unref (GnmFont *sf)
 		g_object_unref (G_OBJECT (sf->pango.font));
 		sf->pango.font = NULL;
 	}
-	if (sf->pango.font_descr != NULL) {
-		pango_font_description_free (sf->pango.font_descr);
-		sf->pango.font_descr = NULL;
+	if (sf->go.font) {
+		go_font_unref (sf->go.font);
+		sf->go.font = NULL;
+	}
+	if (sf->go.metrics) {
+		go_font_metrics_free (sf->go.metrics);
+		sf->go.metrics = NULL;
 	}
 	g_hash_table_remove (style_font_hash, sf);
 	g_free (sf->font_name);
@@ -342,6 +279,8 @@ style_font_hash_func (gconstpointer v)
 	return k->size_pts + g_str_hash (k->font_name);
 }
 
+static PangoFontMap *fontmap;
+
 /**
  * gnm_pango_context_get :
  *
@@ -355,13 +294,9 @@ gnm_pango_context_get (void)
 
 	if (screen != NULL) {
 		context = gdk_pango_context_get_for_screen (screen);
-#ifndef GDK_DISABLE_DEPRECATED
-		/* this function is deprecated in newer gtk */
-		gdk_pango_context_set_colormap (context,
-			gdk_screen_get_default_colormap (screen));
-#endif
 	} else {
-		PangoFontMap *fontmap = pango_ft2_font_map_new ();
+		if (!fontmap)
+			fontmap = pango_ft2_font_map_new ();
 		pango_ft2_font_map_set_resolution (PANGO_FT2_FONT_MAP (fontmap), 96, 96);
 		context = pango_ft2_font_map_create_context (PANGO_FT2_FONT_MAP (fontmap));
 	}
@@ -376,6 +311,7 @@ font_init (void)
 {
 	PangoContext *context;
 	GnmFont *gnumeric_default_font = NULL;
+	double pts_scale = 72. / gnm_app_display_dpi_get (TRUE);
 
 	gnumeric_default_font_name = g_strdup (gnm_app_prefs->default_font.name);
 	gnumeric_default_font_size = gnm_app_prefs->default_font.size;
@@ -411,7 +347,8 @@ font_init (void)
 		}
 	}
 
-	gnumeric_default_font_width = gnumeric_default_font->approx_width.pts.digit;
+	gnumeric_default_font_width = pts_scale *
+		PANGO_PIXELS (gnumeric_default_font->go.metrics->avg_digit_width);
 	style_font_unref (gnumeric_default_font);
 	g_object_unref (G_OBJECT (context));
 }
@@ -421,6 +358,17 @@ font_shutdown (void)
 {
 	g_free (gnumeric_default_font_name);
 	gnumeric_default_font_name = NULL;
+
+	if (fontmap) {
+		/*
+		 * Workaround for bug #143542 (PangoFT2Fontmap leak).
+		 * See also bug #148997 (Text layer rendering leaks font file
+		 * descriptor).
+		 */
+		pango_ft2_font_map_substitute_changed (PANGO_FT2_FONT_MAP (fontmap));
+		g_object_unref (fontmap);
+		fontmap = NULL;
+	}
 }
 
 void
@@ -487,22 +435,22 @@ required_updates_for_style (GnmStyle const *style)
 	SpanCalcFlags res = SPANCALC_SIMPLE;
 
 	gboolean const row_height =
-	     gnm_style_is_element_set (style, MSTYLE_FONT_SIZE) ||
-	     gnm_style_is_element_set (style, MSTYLE_WRAP_TEXT) ||
-	     gnm_style_is_element_set (style, MSTYLE_ROTATION);
+		gnm_style_is_element_set (style, MSTYLE_FONT_SIZE) ||
+		gnm_style_is_element_set (style, MSTYLE_WRAP_TEXT) ||
+		gnm_style_is_element_set (style, MSTYLE_ROTATION) ||
+		gnm_style_is_element_set (style, MSTYLE_FONT_SCRIPT);
 	gboolean const size_change = row_height ||
-	     gnm_style_is_element_set (style, MSTYLE_FONT_NAME) ||
-	     gnm_style_is_element_set (style, MSTYLE_FONT_BOLD) ||
-	     gnm_style_is_element_set (style, MSTYLE_FONT_ITALIC);
+		gnm_style_is_element_set (style, MSTYLE_FONT_NAME) ||
+		gnm_style_is_element_set (style, MSTYLE_FONT_BOLD) ||
+		gnm_style_is_element_set (style, MSTYLE_FONT_ITALIC);
 	gboolean const format_change =
-	    (gnm_style_is_element_set (style, MSTYLE_FORMAT) ||
-	     gnm_style_is_element_set (style, MSTYLE_INDENT) ||
-	     gnm_style_is_element_set (style, MSTYLE_ALIGN_H) ||
-	     gnm_style_is_element_set (style, MSTYLE_ALIGN_V) ||
-	     gnm_style_is_element_set (style, MSTYLE_FONT_STRIKETHROUGH) ||
-	     gnm_style_is_element_set (style, MSTYLE_FONT_UNDERLINE) ||
-	     gnm_style_is_element_set (style, MSTYLE_FONT_COLOR) ||
-	     gnm_style_is_element_set (style, MSTYLE_ROTATION));
+		gnm_style_is_element_set (style, MSTYLE_FORMAT) ||
+		gnm_style_is_element_set (style, MSTYLE_INDENT) ||
+		gnm_style_is_element_set (style, MSTYLE_ALIGN_H) ||
+		gnm_style_is_element_set (style, MSTYLE_ALIGN_V) ||
+		gnm_style_is_element_set (style, MSTYLE_FONT_STRIKETHROUGH) ||
+		gnm_style_is_element_set (style, MSTYLE_FONT_UNDERLINE) ||
+		gnm_style_is_element_set (style, MSTYLE_FONT_COLOR);
 
 	if (row_height)
 		res |= SPANCALC_ROW_HEIGHT;
@@ -539,7 +487,6 @@ style_default_halign (GnmStyle const *mstyle, GnmCell const *c)
 		case VALUE_ERROR:
 			return HALIGN_CENTER;
 
-		case VALUE_INTEGER:
 		case VALUE_FLOAT: {
 			double a = gnm_style_get_rotation (mstyle);
 			if (a > 0 && a < 180)
