@@ -10,10 +10,17 @@
 
 #include <gtk/gtknotebook.h>
 
+#ifdef USE_HILDON
+#include <hildon-widgets/hildon-program.h>
+#endif
+
 struct _WorkbookControlGUI {
 	WorkbookControl	wb_control;
 
-	GtkWindow   *toplevel;
+	GtkWidget   *toplevel;
+#ifdef USE_HILDON
+	HildonProgram *hildon_prog;
+#endif
 	GtkNotebook *notebook;
 	GtkWidget   *progress_bar;
 
@@ -69,6 +76,8 @@ struct _WorkbookControlGUI {
 	PangoFontDescription *font_desc;
 
 	GOFileSaver *current_saver;
+
+	gulong sig_view_changed, sig_sheet_order, sig_notify_uri, sig_notify_dirty;
 };
 
 typedef struct {
@@ -100,18 +109,24 @@ typedef struct {
 
 #define WORKBOOK_CONTROL_GUI_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), WORKBOOK_CONTROL_GUI_TYPE, WorkbookControlGUIClass))
 
+#define GNM_RESPONSE_SAVE_ALL -1000
+#define GNM_RESPONSE_DISCARD_ALL -1001
+
 /* Protected functions */
 void	 wbcg_set_toplevel	      (WorkbookControlGUI *wbcg, GtkWidget *w);
 gboolean wbcg_scroll_wheel_support_cb (GtkWidget *ignored,
 				       GdkEventScroll *event,
 				       WorkbookControlGUI *wbcg);
 gboolean wbcg_close_control	      (WorkbookControlGUI *wbcg);
+int 	 wbcg_show_save_dialog	      (WorkbookControlGUI *wbcg, 
+				       Workbook * wb, gboolean exiting);
 int	 wbcg_close_if_user_permits   (WorkbookControlGUI *wbcg,
 				       WorkbookView *wb_view, gboolean close_clean,
 				       gboolean exiting, gboolean ask_user);
 void	 scg_delete_sheet_if_possible (GtkWidget *ignored, SheetControlGUI *scg);
 void	 wbcg_insert_sheet	      (GtkWidget *ignored, WorkbookControlGUI *wbcg);
 void	 wbcg_append_sheet	      (GtkWidget *ignored, WorkbookControlGUI *wbcg);
+void	 wbcg_clone_sheet	      (GtkWidget *ignored, WorkbookControlGUI *wbcg);
 void	 wbcg_set_selection_halign    (WorkbookControlGUI *wbcg, GnmHAlign halign);
 void	 wbcg_set_selection_valign    (WorkbookControlGUI *wbcg, GnmVAlign valign);
 
