@@ -9,10 +9,9 @@
  *
  * (C) 1998, 1999, 2000 Miguel de Icaza
  * (C) 2000-2001 Ximian, Inc.
- * (C) 2002-2005 Jody Goldberg
+ * (C) 2002-2006 Jody Goldberg
  */
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
 #include "gnumeric.h"
 #include "workbook-priv.h"
 
@@ -32,11 +31,12 @@
 #include "history.h"
 #include "commands.h"
 #include "libgnumeric.h"
-#include <goffice/app/file.h>
-#include <goffice/app/io-context.h>
 #include "gutils.h"
 #include "gnm-marshalers.h"
 #include "style-color.h"
+
+#include <goffice/app/file.h>
+#include <goffice/app/io-context.h>
 #include <goffice/utils/go-file.h>
 #include <goffice/utils/go-glib-extras.h>
 
@@ -47,23 +47,24 @@
 #include <gtk/gtkmain.h> /* for gtk_main_quit */
 #endif
 #endif /* WITH_GTK */
+
+#include <gsf/gsf-doc-meta-data.h>
 #include <gsf/gsf-impl-utils.h>
+#include <glib/gi18n-lib.h>
 #include <string.h>
 #include <errno.h>
 
-static GObjectClass *workbook_parent_class;
-
-/* Signals */
 enum {
-	SUMMARY_CHANGED,
-	FILENAME_CHANGED,
+	PROP_0,
+};
+enum {
 	SHEET_ORDER_CHANGED,
 	SHEET_ADDED,
 	SHEET_DELETED,
 	LAST_SIGNAL
 };
-
 static guint signals [LAST_SIGNAL] = { 0 };
+static GObjectClass *workbook_parent_class;
 
 static void
 cb_saver_finalize (Workbook *wb, GOFileSaver *saver)
@@ -104,15 +105,6 @@ workbook_dispose (GObject *wb_object)
 		Sheet *sheet = ptr->data;
 
 		sheet_destroy_contents (sheet);
-		/*
-		 * We need to put this test BEFORE we detach
-		 * the sheet from the workbook.  It is ugly, but should
-		 * be ok for debug code.
-		 */
-		if (gnumeric_debugging > 0) {
-			g_printerr ("Dependencies for %s:\n", sheet->name_unquoted);
-			gnm_dep_container_dump (sheet->deps);
-		}
 	}
 
 	/* Now remove the sheets themselves */
@@ -135,25 +127,17 @@ workbook_dispose (GObject *wb_object)
 			g_warning ("Unexpected left over views");
 	}
 
-	if (wb->uri) {
-		if (wb->file_format_level >= FILE_FL_MANUAL_REMEMBER)
-			gnm_app_history_add (wb->uri);
-	       g_free (wb->uri);
-	       wb->uri = NULL;
-	}
+	if (wb->doc.uri &&
+	    wb->file_format_level >= FILE_FL_MANUAL_REMEMBER)
+		gnm_app_history_add (wb->doc.uri);
 
 	workbook_parent_class->dispose (wb_object);
 }
 
-
-
 static void
-workbook_finalize (GObject *wb_object)
+workbook_finalize (GObject *obj)
 {
-	Workbook *wb = WORKBOOK (wb_object);
-
-	summary_info_free (wb->summary_info);
-	wb->summary_info = NULL;
+	Workbook *wb = WORKBOOK (obj);
 
 	/* Remove ourselves from the list of workbooks.  */
 	gnm_app_workbook_list_remove (wb);
@@ -175,109 +159,7 @@ workbook_finalize (GObject *wb_object)
 		gtk_main_quit ();
 #endif
 #endif
-	workbook_parent_class->finalize (wb_object);
-}
-
-void
-workbook_set_dirty (Workbook *wb, gboolean is_dirty)
-{
-	gboolean changed;
-	int i;
-
-	g_return_if_fail (wb != NULL);
-
-	is_dirty = !!is_dirty;
-	changed = (workbook_is_dirty (wb) != is_dirty);
-	wb->modified = is_dirty;
-
-	if (wb->summary_info != NULL)
-		wb->summary_info->modified = is_dirty;
-
-	for (i = 0; i < (int)wb->sheets->len; i++) {
-		Sheet *sheet = g_ptr_array_index (wb->sheets, i);
-		sheet_set_dirty (sheet, is_dirty);
-	}
-
-	if (changed) {
-		WORKBOOK_FOREACH_CONTROL (wb, view, control,
-			wb_control_update_title (control););
-	}
-}
-
-gboolean
-workbook_is_dirty (Workbook const *wb)
-{
-	int i;
-
-	g_return_val_if_fail (wb != NULL, FALSE);
-	if (wb->summary_info != NULL && wb->summary_info->modified)
-		return TRUE;
-
-	for (i = 0; i < (int)wb->sheets->len; i++) {
-		Sheet *sheet = g_ptr_array_index (wb->sheets, i);
-		if (sheet->modified)
-			return TRUE;
-	}
-
-	return FALSE;
-}
-
-/**
- * workbook_set_placeholder :
- * @wb :
- * @is_placeholder :
- **/
-void
-workbook_set_placeholder (Workbook *wb, gboolean is_placeholder)
-{
-	g_return_if_fail (IS_WORKBOOK (wb));
-	wb->is_placeholder = is_placeholder;
-}
-
-/**
- * workbook_is_placeholder :
- * @wb :
- *
- * Returns TRUE if @wb has no views and was created as a placeholder for data
- * in an external reference.
- **/
-gboolean
-workbook_is_placeholder	(Workbook const *wb)
-{
-	g_return_val_if_fail (IS_WORKBOOK (wb), FALSE);
-	return wb->is_placeholder;
-}
-
-/**
- * workbook_is_pristine:
- * @wb:
- *
- *   This checks to see if the workbook has ever been
- * used ( approximately )
- *
- * Return value: TRUE if we can discard this workbook.
- **/
-gboolean
-workbook_is_pristine (Workbook const *wb)
-{
-	int i;
-
-	g_return_val_if_fail (wb != NULL, FALSE);
-
-	if (workbook_is_dirty (wb))
-		return FALSE;
-
-	if (wb->names ||
-	    (wb->file_format_level > FILE_FL_NEW))
-		return FALSE;
-
-	for (i = 0; i < (int)wb->sheets->len; i++) {
-		Sheet *sheet = g_ptr_array_index (wb->sheets, i);
-		if (!sheet_is_pristine (sheet))
-			return FALSE;
-	}
-
-	return TRUE;
+	workbook_parent_class->finalize (obj);
 }
 
 static void
@@ -286,18 +168,12 @@ workbook_init (GObject *object)
 	Workbook *wb = WORKBOOK (object);
 
 	wb->is_placeholder = FALSE;
-	wb->modified	   = FALSE;
-
 	wb->wb_views = NULL;
 	wb->sheets = g_ptr_array_new ();
 	wb->sheet_hash_private = g_hash_table_new (g_str_hash, g_str_equal);
 	wb->sheet_order_dependents = NULL;
 	wb->sheet_local_functions = NULL;
 	wb->names        = NULL;
-	wb->summary_info = summary_info_new ();
-	summary_info_default (wb->summary_info);
-	wb->summary_info->modified = FALSE;
-	wb->uri = NULL;
 
 	/* Nothing to undo or redo */
 	wb->undo_commands = wb->redo_commands = NULL;
@@ -320,32 +196,46 @@ workbook_init (GObject *object)
 	gnm_app_workbook_list_add (wb);
 }
 
+#if 0
+static void
+workbook_get_property (GObject *object, guint property_id,
+		       GValue *value, GParamSpec *pspec)
+{
+	Workbook *wb = (Workbook *)object;
+
+	switch (property_id) {
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
+	}
+}
+
+static void
+workbook_set_property (GObject *object, guint property_id,
+		       const GValue *value, GParamSpec *pspec)
+{
+	Workbook *wb = (Workbook *)object;
+
+	switch (property_id) {
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+		break;
+	}
+}
+#endif
+
 static void
 workbook_class_init (GObjectClass *object_class)
 {
 	workbook_parent_class = g_type_class_peek_parent (object_class);
 
+#if 0
+	object_class->set_property = workbook_set_property;
+	object_class->get_property = workbook_get_property;
+#endif
 	object_class->finalize = workbook_finalize;
 	object_class->dispose = workbook_dispose;
 
-	signals [SUMMARY_CHANGED] = g_signal_new ("summary_changed",
-		WORKBOOK_TYPE,
-		G_SIGNAL_RUN_LAST,
-		G_STRUCT_OFFSET (WorkbookClass, summary_changed),
-		NULL, NULL,
-		g_cclosure_marshal_VOID__VOID,
-		G_TYPE_NONE,
-		0, G_TYPE_NONE);
-
-	signals [FILENAME_CHANGED] = g_signal_new ("filename_changed",
-		WORKBOOK_TYPE,
-		G_SIGNAL_RUN_LAST,
-		G_STRUCT_OFFSET (WorkbookClass, filename_changed),
-		NULL, NULL,
-		g_cclosure_marshal_VOID__VOID,
-		G_TYPE_NONE,
-		0, G_TYPE_NONE);
-
 	signals [SHEET_ORDER_CHANGED] = g_signal_new ("sheet_order_changed",
 		WORKBOOK_TYPE,
 		G_SIGNAL_RUN_LAST,
@@ -408,7 +298,7 @@ workbook_new (void)
 		}
 		uri = go_filename_to_uri (name);
 
-		is_unique = workbook_set_uri (wb, uri);
+		is_unique = go_doc_set_uri (GO_DOC (wb), uri);
 
 		g_free (uri);
 		g_free (name);
@@ -467,63 +357,12 @@ workbook_new_with_sheets (int sheet_count)
 {
 	Workbook *wb = workbook_new ();
 	while (sheet_count-- > 0)
-		workbook_sheet_add (wb, -1, FALSE);
+		workbook_sheet_add (wb, -1);
+	go_doc_set_dirty (GO_DOC (wb), FALSE);
+	GO_DOC (wb)->pristine = TRUE;
 	return wb;
 }
 
-/**
- * workbook_set_uri:
- * @wb: the workbook to modify
- * @uri: the uri for this worksheet.
- *
- * Sets the internal filename to @name and changes
- * the title bar for the toplevel window to be the name
- * of this file.
- *
- * Returns : TRUE if the name was set succesfully.
- *
- * FIXME : Add a check to ensure the name is unique.
- */
-gboolean
-workbook_set_uri (Workbook *wb, char const *uri)
-{
-	g_return_val_if_fail (wb != NULL, FALSE);
-	g_return_val_if_fail (uri != NULL, FALSE);
-
-	if (uri == wb->uri)
-		return TRUE;
-
-	g_free (wb->uri);
-	wb->uri = g_strdup (uri);
-	WORKBOOK_FOREACH_CONTROL (wb, view, control,
-		wb_control_update_title (control););
-
-	g_signal_emit (G_OBJECT (wb), signals [FILENAME_CHANGED], 0);
-	_gnm_app_flag_windows_changed ();
-	return TRUE;
-}
-
-const gchar *
-workbook_get_uri (Workbook const *wb)
-{
-	g_return_val_if_fail (IS_WORKBOOK (wb), NULL);
-	return wb->uri;
-}
-
-void
-workbook_add_summary_info (Workbook *wb, SummaryItem *sit)
-{
-	if (summary_info_add (wb->summary_info, sit))
-		g_signal_emit (G_OBJECT (wb), signals [SUMMARY_CHANGED], 0);
-}
-
-SummaryInfo *
-workbook_metadata (Workbook const *wb)
-{
-	g_return_val_if_fail (IS_WORKBOOK (wb), NULL);
-	return wb->summary_info;
-}
-
 /**
  * workbook_set_saveinfo:
  * @wb: the workbook to modify
@@ -601,8 +440,8 @@ workbook_foreach_cell_in_range (GnmEvalPos const *pos,
 	g_return_val_if_fail (cell_range != NULL, NULL);
 	g_return_val_if_fail (cell_range->type == VALUE_CELLRANGE, NULL);
 
-	rangeref_normalize (&cell_range->v_range.cell, pos,
-			    &start_sheet, &end_sheet, &r);
+	gnm_rangeref_normalize (&cell_range->v_range.cell, pos,
+		&start_sheet, &end_sheet, &r);
 
 	if (start_sheet != end_sheet) {
 		GnmValue *res;
@@ -641,19 +480,13 @@ workbook_foreach_cell_in_range (GnmEvalPos const *pos,
 GPtrArray *
 workbook_cells (Workbook *wb, gboolean comments)
 {
-	GList *tmp, *sheets;
 	GPtrArray *cells = g_ptr_array_new ();
 
 	g_return_val_if_fail (wb != NULL, cells);
 
-	sheets = workbook_sheets (wb);
-	for (tmp = sheets; tmp; tmp = tmp->next) {
-		Sheet *sheet = tmp->data;
+	WORKBOOK_FOREACH_SHEET (wb, sheet, {
 		int oldlen = cells->len;
-		GPtrArray *scells =
-			sheet_cells (sheet,
-				     0, 0, SHEET_MAX_COLS, SHEET_MAX_ROWS,
-				     comments);
+		GPtrArray *scells = sheet_cells (sheet, comments);
 
 		g_ptr_array_set_size (cells, oldlen + scells->len);
 		memcpy (&g_ptr_array_index (cells, oldlen),
@@ -661,8 +494,7 @@ workbook_cells (Workbook *wb, gboolean comments)
 			scells->len * sizeof (GnmEvalPos *));
 
 		g_ptr_array_free (scells, TRUE);
-	}
-	g_list_free (sheets);
+	});
 
 	return cells;
 }
@@ -728,7 +560,7 @@ workbook_attach_view (Workbook *wb, WorkbookView *wbv)
 {
 	g_return_if_fail (IS_WORKBOOK (wb));
 	g_return_if_fail (IS_WORKBOOK_VIEW (wbv));
-	g_return_if_fail (wb_view_workbook (wbv) == NULL);
+	g_return_if_fail (wb_view_get_workbook (wbv) == NULL);
 
 	if (wb->wb_views == NULL)
 		wb->wb_views = g_ptr_array_new ();
@@ -898,7 +730,7 @@ workbook_focus_other_sheet (Workbook *wb, Sheet *sheet)
 }
 
 /**
- * workbook_sheet_hide_controls :
+ * workbook_sheet_remove_controls :
  * @wb : #Workbook
  * @sheet : #Sheet
  *
@@ -907,7 +739,7 @@ workbook_focus_other_sheet (Workbook *wb, Sheet *sheet)
  * Returns TRUE if there are any remaining sheets visible
  **/
 static gboolean
-workbook_sheet_hide_controls (Workbook *wb, Sheet *sheet)
+workbook_sheet_remove_controls (Workbook *wb, Sheet *sheet)
 {
 	Sheet *focus = NULL;
 
@@ -931,23 +763,6 @@ workbook_sheet_hide_controls (Workbook *wb, Sheet *sheet)
 	return focus != NULL;
 }
 
-static void
-cb_sheet_visibility_change (Sheet *sheet,
-			    G_GNUC_UNUSED GParamSpec *pspec,
-			    G_GNUC_UNUSED gpointer data)
-{
-	g_return_if_fail (IS_SHEET (sheet));
-
-	if (sheet->visibility == GNM_SHEET_VISIBILITY_VISIBLE) {
-		SheetView *sv;
-		WORKBOOK_FOREACH_CONTROL (sheet->workbook, wbv, wbc, {
-			sv = sheet_get_view (sheet, wbv);
-			if (NULL == sv_get_control (sv, wbc))
-				wb_control_sheet_add (wbc, sv);});
-	} else
-		workbook_sheet_hide_controls (sheet->workbook, sheet);
-}
-
 /**
  * workbook_sheet_attach_at_pos :
  * @wb :
@@ -977,13 +792,10 @@ workbook_sheet_attach_at_pos (Workbook *wb, Sheet *new_sheet, int pos)
 	WORKBOOK_FOREACH_VIEW (wb, view,
 		wb_view_sheet_add (view, new_sheet););
 
-	g_signal_connect (G_OBJECT (new_sheet),
-			  "notify::visibility",
-			  G_CALLBACK (cb_sheet_visibility_change),
-			  NULL);
-
 	/* Do not signal until after adding the views [#314208] */
 	post_sheet_index_change (wb);
+
+	go_doc_set_dirty (GO_DOC (wb), TRUE);
 }
 
 /**
@@ -1011,7 +823,7 @@ workbook_sheet_attach (Workbook *wb, Sheet *new_sheet)
  * returned is not ref'd.  (The ref belongs to the workbook.)
  */
 Sheet *
-workbook_sheet_add (Workbook *wb, int pos, gboolean make_dirty)
+workbook_sheet_add (Workbook *wb, int pos)
 {
 	char *name = workbook_sheet_get_free_name (wb, _("Sheet"), TRUE, FALSE);
 	Sheet *new_sheet = sheet_new (wb, name);
@@ -1020,9 +832,8 @@ workbook_sheet_add (Workbook *wb, int pos, gboolean make_dirty)
 	if (pos == -1)
 		pos = wb->sheets->len;
 	workbook_sheet_attach_at_pos (wb, new_sheet, pos);
-	if (make_dirty)
-		sheet_set_dirty (new_sheet, TRUE);
 
+	/* FIXME: Why here?  */
 	g_signal_emit (G_OBJECT (wb), signals [SHEET_ADDED], 0);
 
 	g_object_unref (new_sheet);
@@ -1051,13 +862,11 @@ workbook_sheet_delete (Sheet *sheet)
 	wb = sheet->workbook;
 	sheet_index = sheet->index_in_wb;
 
-	g_signal_handlers_disconnect_by_func (sheet, cb_sheet_visibility_change, NULL);
-
 	if (!wb->during_destruction) {
 		workbook_focus_other_sheet (wb, sheet);
 		/* During destruction this was already done.  */
 		dependents_invalidate_sheet (sheet, FALSE);
-		still_visible_sheets = workbook_sheet_hide_controls (wb, sheet);
+		still_visible_sheets = workbook_sheet_remove_controls (wb, sheet);
 	}
 
 	/* All is fine, remove the sheet */
@@ -1074,6 +883,8 @@ workbook_sheet_delete (Sheet *sheet)
 	g_signal_emit_by_name (G_OBJECT (sheet), "detached_from_workbook", wb);
 	g_object_unref (sheet);
 
+	if (!wb->during_destruction)
+		go_doc_set_dirty (GO_DOC (wb), TRUE);
 	g_signal_emit (G_OBJECT (wb), signals[SHEET_DELETED], 0);
 
 	if (still_visible_sheets)
@@ -1109,11 +920,11 @@ workbook_sheet_move (Sheet *sheet, int direction)
 			Sheet *sheet = g_ptr_array_index (wb->sheets, max_pos);
 			sheet->index_in_wb = max_pos;
 		}
-
-		sheet_set_dirty (sheet, TRUE);
 	}
 
 	post_sheet_index_change (wb);
+
+	go_doc_set_dirty (GO_DOC (wb), TRUE);
 }
 
 /**
@@ -1130,11 +941,11 @@ workbook_sheet_move (Sheet *sheet, int direction)
  **/
 char *
 workbook_sheet_get_free_name (Workbook *wb,
-			      const char *base,
+			      char const *base,
 			      gboolean always_suffix,
 			      gboolean handle_counter)
 {
-	const char *name_format;
+	char const *name_format;
 	char *name, *base_name;
 	unsigned int i = 0;
 	int limit;
@@ -1446,13 +1257,6 @@ workbook_sheet_state_diff (const WorkbookSheetState *wss_a, const WorkbookSheetS
 			/* We do not count reordered sheet.  */
 		}			
 
-		pa = wss_a->sheets[ia].properties;
-		pb = wss_b->sheets[ib].properties;
-		for (; pa && pb; pa = pa->next->next, pb = pb->next->next) {
-			GParamSpec *pspec = pa->data;
-			/* We do not count reordered sheet.  */
-		}			
-
 		pa = wss_a->sheets[ia].properties;
 		pb = wss_b->sheets[ib].properties;
 		for (; pa && pb; pa = pa->next->next, pb = pb->next->next) {
