@@ -9,7 +9,7 @@
  *   Jukka-Pekka Iivonen <jiivonen@hutcs.cs.hut.fi>
  */
 #include <gnumeric-config.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include "gnumeric.h"
 #include "xml-io.h"
 #include "xml-sax.h"
@@ -59,6 +59,7 @@
 #include <goffice/utils/go-glib-extras.h>
 #include <goffice/app/error-info.h>
 #include <gsf/gsf-libxml.h>
+#include <gsf/gsf-doc-meta-data.h>
 #include <gsf/gsf-input.h>
 #include <gsf/gsf-input-gzip.h>
 #include <gsf/gsf-output.h>
@@ -80,9 +81,9 @@
 #define XML_INPUT_BUFFER_SIZE      4096
 #define N_ELEMENTS_BETWEEN_UPDATES 20
 
-#define CC2XML(s) ((const xmlChar *)(s))
+#define CC2XML(s) ((xmlChar const *)(s))
 #define C2XML(s) ((xmlChar *)(s))
-#define CXML2C(s) ((const char *)(s))
+#define CXML2C(s) ((char const *)(s))
 #define XML2C(s) ((char *)(s))
 
 /* ------------------------------------------------------------------------- */
@@ -97,7 +98,6 @@ xml_io_conventions (void)
 	res->ref_parser = rangeref_parse;
 	res->range_sep_colon = TRUE;
 	res->sheet_sep_exclamation = TRUE;
-	res->dots_in_names = TRUE;
 	res->output_sheet_name_sep = "!";
 	res->output_argument_sep = ",";
 	res->output_array_col_sep = ",";
@@ -129,7 +129,7 @@ xml_parse_ctx_new (xmlDocPtr     doc,
 	ctxt->expr_map     = g_hash_table_new (g_direct_hash, g_direct_equal);
 	ctxt->shared_exprs = g_ptr_array_new ();
 	ctxt->wb_view      = wb_view;
-	ctxt->wb	   = (wb_view != NULL) ? wb_view_workbook (wb_view) : NULL;
+	ctxt->wb	   = (wb_view != NULL) ? wb_view_get_workbook (wb_view) : NULL;
 	ctxt->exprconv     = xml_io_conventions ();
 
 	return ctxt;
@@ -299,10 +299,7 @@ xml_read_selection_info (XmlParseContext *ctxt, xmlNodePtr tree)
 	sv_selection_reset (sv);
 	for (sel = selections->xmlChildrenNode; sel; sel = sel->next)
 		if (!xmlIsBlankNode (sel) && xml_node_get_range (sel, &r))
-			sv_selection_add_range (sv,
-						r.start.col, r.start.row,
-						r.start.col, r.start.row,
-						r.end.col, r.end.row);
+			sv_selection_add_range (sv, &r);
 
 	if (xml_node_get_int (selections, "CursorCol", &pos.col) &&
 	    xml_node_get_int (selections, "CursorRow", &pos.row))
@@ -360,12 +357,12 @@ xml_read_names (XmlParseContext *ctxt, xmlNodePtr tree,
 	xmlNode *name = e_xml_get_child_by_name (tree, CC2XML ("Names"));
 	xmlChar *name_str;
 	xmlChar *expr_str;
-	GnmExpr const *expr;
 
 	if (name == NULL)
 		return;
 
 	for (name = name->xmlChildrenNode; name ; name = name->next) {
+		GnmExprTop const *texpr;
 		GnmParseError  perr;
 		GnmParsePos    pp;
 
@@ -398,19 +395,21 @@ xml_read_names (XmlParseContext *ctxt, xmlNodePtr tree,
 		}
 
 		parse_error_init (&perr);
-		expr = gnm_expr_parse_str (CXML2C (expr_str), &pp,
+		texpr = gnm_expr_parse_str (CXML2C (expr_str), &pp,
 					   GNM_EXPR_PARSE_DEFAULT,
 					   ctxt->exprconv, &perr);
 		/* See http://bugzilla.gnome.org/show_bug.cgi?id=317427 */
-		if (!expr)
-			expr = gnm_expr_parse_str (CXML2C (expr_str), &pp,
+		if (!texpr)
+			texpr = gnm_expr_parse_str (CXML2C (expr_str), &pp,
 						   GNM_EXPR_PARSE_DEFAULT,
 						   gnm_expr_conventions_default,
 						   NULL);
 
-		if (expr != NULL) {
+		if (texpr != NULL) {
 			char *err = NULL;
-			expr_name_add (&pp, CXML2C (name_str), expr, &err, TRUE, NULL);
+			expr_name_add (&pp, CXML2C (name_str),
+				       texpr,
+				       &err, TRUE, NULL);
 			if (err != NULL) {
 				gnm_io_warning (ctxt->io_context, err);
 				g_free (err);
@@ -425,52 +424,22 @@ xml_read_names (XmlParseContext *ctxt, xmlNodePtr tree,
 }
 
 static void
-xml_read_summary (XmlParseContext *ctxt, xmlNodePtr tree, SummaryInfo *summary_info)
+dom_read_meta_data (XmlParseContext *ctxt, xmlNodePtr tree, GsfDocMetaData *meta_data)
 {
 	xmlNodePtr child;
+	xmlChar *val_str, *type_str, *name;
 
-	g_return_if_fail (ctxt != NULL);
-	g_return_if_fail (tree != NULL);
-	g_return_if_fail (summary_info != NULL);
+	for (child = tree->xmlChildrenNode; child != NULL ; child = child->next) {
+		if (xmlIsBlankNode (child) || NULL == child->name || strcmp (child->name, "Property") ||
+		    NULL == (name	= xml_node_get_cstr (child, "name")) ||
+		    NULL == (type_str	= xml_node_get_cstr (child, "type")) ||
+		    NULL == (val_str	= xml_node_get_cstr (child, NULL)))
+			continue;
 
-	for (child = tree->xmlChildrenNode; child != NULL ; child = child->next)
-		if (!xmlIsBlankNode (child) && child->name && !strcmp (child->name, "Item")) {
-			xmlNodePtr bits;
-			xmlChar *name = NULL;
-
-			for (bits = child->xmlChildrenNode; bits != NULL ; bits = bits->next) {
-				SummaryItem *sit = NULL;
-
-				if (xmlIsBlankNode (bits))
-					continue;
-
-				if (!strcmp (bits->name, "name")) {
-					name = xml_node_get_cstr (bits, NULL);
-				} else {
-					xmlChar *txt;
-					g_return_if_fail (name);
-
-					txt = xml_node_get_cstr (bits, NULL);
-					if (txt != NULL){
-						if (!strcmp (bits->name, "val-string"))
-							sit = summary_item_new_string (CXML2C (name),
-										       CXML2C (txt),
-										       TRUE);
-						else if (!strcmp (bits->name, "val-int"))
-							sit = summary_item_new_int (CXML2C (name),
-										    atoi (CXML2C (txt)));
-
-						if (sit)
-							summary_info_add (summary_info, sit);
-						xmlFree (txt);
-					}
-				}
-			}
-			if (name) {
-				xmlFree (name);
-				name = NULL;
-			}
-		}
+		xmlFree (val_str);
+		xmlFree (type_str);
+		xmlFree (name);
+	}
 }
 
 static void
@@ -608,7 +577,7 @@ xml_read_print_repeat_range (XmlParseContext *ctxt, xmlNodePtr tree,
 
 		if (s) {
 			GnmRange r;
-			if (parse_range (CXML2C (s), &r)) {
+			if (range_parse (&r, CXML2C (s))) {
 				range->range = r;
 				range->use   = TRUE;
 			}
@@ -807,9 +776,9 @@ xml_read_style (XmlParseContext *ctxt, xmlNodePtr tree)
 		gnm_style_set_wrap_text (style, val);
 
 	if (xml_node_get_int (tree, "Locked", &val))
-		gnm_style_set_content_locked (style, val);
+		gnm_style_set_contents_locked (style, val);
 	if (xml_node_get_int (tree, "Hidden", &val))
-		gnm_style_set_content_hidden (style, val);
+		gnm_style_set_contents_hidden (style, val);
 
 	if (xml_node_get_int (tree, "VAlign", &val))
 		gnm_style_set_align_v (style, val);
@@ -914,7 +883,8 @@ xml_read_style (XmlParseContext *ctxt, xmlNodePtr tree)
 			ValidationOp op = VALIDATION_OP_NONE;
 			xmlChar *title, *msg;
 			gboolean allow_blank, use_dropdown;
-			GnmExpr const *expr0 = NULL, *expr1 = NULL;
+			GnmExprTop const *texpr0 = NULL;
+			GnmExprTop const *texpr1 = NULL;
 
 			if (xml_node_get_int (child, "Style", &val))
 				vstyle = val;
@@ -934,21 +904,29 @@ xml_read_style (XmlParseContext *ctxt, xmlNodePtr tree)
 			parse_pos_init_sheet (&pp, ctxt->sheet);
 			if (NULL != (e_node = e_xml_get_child_by_name (child, CC2XML ("Expression0"))) &&
 			    NULL != (content = xml_node_get_cstr (e_node, NULL))) {
-				expr0 = gnm_expr_parse_str (CXML2C (content), &pp,
-					GNM_EXPR_PARSE_DEFAULT, ctxt->exprconv, NULL);
+				texpr0 = gnm_expr_parse_str
+					(CXML2C (content), &pp,
+					 GNM_EXPR_PARSE_DEFAULT,
+					 ctxt->exprconv, NULL);
 				xmlFree (content);
 			}
 			if (NULL != (e_node = e_xml_get_child_by_name (child, CC2XML ("Expression1"))) &&
 			    NULL != (content = xml_node_get_cstr (e_node, NULL))) {
-				expr1 = gnm_expr_parse_str (CXML2C (content), &pp,
-					GNM_EXPR_PARSE_DEFAULT, ctxt->exprconv, NULL);
+				texpr1 = gnm_expr_parse_str
+					(CXML2C (content), &pp,
+					 GNM_EXPR_PARSE_DEFAULT,
+					 ctxt->exprconv, NULL);
 				xmlFree (content);
 			}
 
-			gnm_style_set_validation (style,
-				validation_new (vstyle, type, op, CXML2C (title),
-					CXML2C (msg), expr0, expr1, allow_blank,
-					use_dropdown));
+			gnm_style_set_validation
+				(style,
+				 validation_new
+				 (vstyle, type, op,
+				  CXML2C (title), CXML2C (msg),
+				  texpr0,
+				  texpr1,
+				  allow_blank, use_dropdown));
 
 			xmlFree (msg);
 			xmlFree (title);
@@ -974,18 +952,22 @@ xml_read_style (XmlParseContext *ctxt, xmlNodePtr tree)
 			parse_pos_init_sheet (&pp, ctxt->sheet);
 			if (NULL != (e_node = e_xml_get_child_by_name (child, CC2XML ("Expression0"))) &&
 			    NULL != (content = xml_node_get_cstr (e_node, NULL))) {
-				cond.expr[0] = gnm_expr_parse_str (CXML2C (content), &pp,
-					GNM_EXPR_PARSE_DEFAULT, ctxt->exprconv, NULL);
+				cond.texpr[0] = gnm_expr_parse_str
+					(CXML2C (content), &pp,
+					 GNM_EXPR_PARSE_DEFAULT,
+					 ctxt->exprconv, NULL);
 				xmlFree (content);
 			} else
-				cond.expr[0] = NULL;
+				cond.texpr[0] = NULL;
 			if (NULL != (e_node = e_xml_get_child_by_name (child, CC2XML ("Expression1"))) &&
 			    NULL != (content = xml_node_get_cstr (e_node, NULL))) {
-				cond.expr[1] = gnm_expr_parse_str (CXML2C (content), &pp,
-					GNM_EXPR_PARSE_DEFAULT, ctxt->exprconv, NULL);
+				cond.texpr[1] = gnm_expr_parse_str
+					(CXML2C (content), &pp,
+					 GNM_EXPR_PARSE_DEFAULT,
+					 ctxt->exprconv, NULL);
 				xmlFree (content);
 			} else
-				cond.expr[1] = NULL;
+				cond.texpr[1] = NULL;
 			if (NULL != (e_node = e_xml_get_child_by_name (child, CC2XML ("Style"))))
 				cond.overlay = xml_read_style (ctxt, e_node);
 			if (NULL == sc)
@@ -1062,17 +1044,17 @@ xml_cell_set_array_expr (XmlParseContext *ctxt,
 			 int const rows, int const cols)
 {
 	GnmParsePos pp;
-	GnmExpr const *expr =
+	GnmExprTop const *texpr =
 		gnm_expr_parse_str (text,
 				    parse_pos_init_cell (&pp, cell),
 				    GNM_EXPR_PARSE_DEFAULT,
 				    ctxt->exprconv, NULL);
 
-	g_return_if_fail (expr != NULL);
+	g_return_if_fail (texpr != NULL);
 	cell_set_array_formula (cell->base.sheet,
 				cell->pos.col, cell->pos.row,
 				cell->pos.col + cols-1, cell->pos.row + rows-1,
-				expr);
+				texpr);
 }
 
 /**
@@ -1153,7 +1135,7 @@ xml_read_cell (XmlParseContext *ctxt, xmlNodePtr tree)
 
 	cell = sheet_cell_get (ctxt->sheet, col, row);
 	if ((is_new_cell = (cell == NULL)))
-		cell = sheet_cell_new (ctxt->sheet, col, row);
+		cell = sheet_cell_create (ctxt->sheet, col, row);
 	if (cell == NULL)
 		return NULL;
 
@@ -1277,17 +1259,17 @@ xml_read_cell (XmlParseContext *ctxt, xmlNodePtr tree)
 				 * until after ValueType was added.
 				 */
 				GnmParsePos pos;
-				GnmExpr const *expr = NULL;
+				GnmExprTop const *texpr = NULL;
 				char const *expr_start = gnm_expr_char_start_p (CXML2C (content));
 				if (NULL != expr_start && *expr_start) {
-					expr = gnm_expr_parse_str (expr_start,
-								   parse_pos_init_cell (&pos, cell),
-								   GNM_EXPR_PARSE_DEFAULT,
-								   ctxt->exprconv, NULL);
+					texpr = gnm_expr_parse_str (expr_start,
+								    parse_pos_init_cell (&pos, cell),
+								    GNM_EXPR_PARSE_DEFAULT,
+								    ctxt->exprconv, NULL);
 				}
-				if (expr != NULL) {
-					cell_set_expr (cell, expr);
-					gnm_expr_unref (expr);
+				if (texpr != NULL) {
+					cell_set_expr (cell, texpr);
+					gnm_expr_top_unref (texpr);
 				} else
 					cell_set_text (cell, CXML2C (content));
 			}
@@ -1298,11 +1280,13 @@ xml_read_cell (XmlParseContext *ctxt, xmlNodePtr tree)
 				if (!cell_has_expr (cell)) {
 					g_warning ("XML-IO: Shared expression with no expession? id = %d\ncontent ='%s'",
 						   shared_expr_index, CXML2C (content));
-					cell_set_expr (cell,
-						gnm_expr_new_constant (value_dup (cell->value)));
+					cell_set_expr
+						(cell,
+						 gnm_expr_top_new_constant
+						 (value_dup (cell->value)));
 				}
 				g_ptr_array_add (ctxt->shared_exprs,
-						 (gpointer) cell->base.expression);
+						 (gpointer)cell->base.texpr);
 			} else {
 				g_warning ("XML-IO: Duplicate or invalid shared expression: %d",
 					   shared_expr_index);
@@ -1311,9 +1295,10 @@ xml_read_cell (XmlParseContext *ctxt, xmlNodePtr tree)
 		xmlFree (content);
 	} else if (shared_expr_index > 0) {
 		if (shared_expr_index <= (int)ctxt->shared_exprs->len + 1) {
-			GnmExpr *expr = g_ptr_array_index (ctxt->shared_exprs,
-							    shared_expr_index - 1);
-			cell_set_expr (cell, expr);
+			GnmExprTop *texpr =
+				g_ptr_array_index (ctxt->shared_exprs,
+						   shared_expr_index - 1);
+			cell_set_expr (cell, texpr);
 		} else {
 			g_warning ("XML-IO: Missing shared expression");
 		}
@@ -1465,7 +1450,7 @@ xml_read_sheet_filters (XmlParseContext *ctxt, xmlNode const *container)
 		area = xml_node_get_cstr (filter_node, "Area");
 		if (area == NULL)
 			continue;
-		if (parse_range (CXML2C (area), &r)) {
+		if (range_parse (&r, CXML2C (area))) {
 			filter = gnm_filter_new (ctxt->sheet, &r);
 			for (field = filter_node->xmlChildrenNode; field != NULL; field = field->next)
 				if (!xmlIsBlankNode (field))
@@ -1654,7 +1639,7 @@ xml_read_sheet_object (XmlParseContext const *ctxt, xmlNodePtr tree)
 
 	/* Class renamed between 1.0.x and 1.2.x */
 	else if (!strcmp (tree->name, "GnmGraph"))
-		so = sheet_object_graph_new (FALSE);
+		so = sheet_object_graph_new (NULL);
 
 	/* Class renamed in 1.2.2 */
 	else if (!strcmp (tree->name, "CellComment"))
@@ -1696,7 +1681,7 @@ xml_read_sheet_object (XmlParseContext const *ctxt, xmlNodePtr tree)
 	tmp = (char *) xmlGetProp (tree, (xmlChar *)"ObjectBound");
 	if (tmp != NULL) {
 		GnmRange r;
-		if (parse_range (tmp, &r))
+		if (range_parse (&r, tmp))
 			so->anchor.cell_bound = r;
 		xmlFree (tmp);
 	}
@@ -1720,9 +1705,9 @@ xml_read_sheet_object (XmlParseContext const *ctxt, xmlNodePtr tree)
 	}
 
 	if (xml_node_get_int (tree, "Direction", &tmp_int))
-		so->anchor.direction = tmp_int;
+		so->anchor.base.direction = tmp_int;
 	else
-		so->anchor.direction = SO_DIR_UNKNOWN;
+		so->anchor.base.direction = GOD_ANCHOR_DIR_UNKNOWN;
 
 	/* Do not assign to a sheet when extracting a cell region */
 	if (NULL != ctxt->sheet) {
@@ -1746,7 +1731,7 @@ xml_read_merged_regions (XmlParseContext const *ctxt, xmlNodePtr sheet)
 			xmlChar *content = xml_node_get_cstr (region, NULL);
 			GnmRange r;
 			if (content != NULL) {
-				if (parse_range (CXML2C (content), &r))
+				if (range_parse (&r, CXML2C (content)))
 					sheet_merge_add (ctxt->sheet, &r, FALSE, NULL);
 				xmlFree (content);
 			}
@@ -1770,91 +1755,77 @@ xml_read_styles (XmlParseContext *ctxt, xmlNodePtr tree)
 	}
 }
 
-/*
- * Create a ColRowInfo equivalent to the XML subtree of doc.
- */
-static int
+static void
 xml_read_colrow_info (XmlParseContext *ctxt, xmlNodePtr tree,
-		      ColRowInfo *info, double *size_pts)
+		      gboolean is_cols)
 {
-	int val, count;
+	ColRowInfo *cri;
+	double size_pts;
+	int pos, val;
+
+	if (!xml_node_get_int (tree, "No", &pos) ||
+	    NULL == (cri = sheet_colrow_fetch (ctxt->sheet, pos, is_cols)))
+		return;
 
-	info->size_pts = -1;
-	xml_node_get_int (tree, "No", &info->pos);
-	xml_node_get_double (tree, "Unit", size_pts);
-	if (xml_node_get_int (tree, "MarginA", &val))
-		info->margin_a = val;
-	if (xml_node_get_int (tree, "MarginB", &val))
-		info->margin_b = val;
 	if (xml_node_get_int (tree, "HardSize", &val))
-		info->hard_size = val;
+		cri->hard_size = val;
+	if (xml_node_get_double (tree, "Unit", &size_pts)) {
+		if (is_cols)
+			sheet_col_set_size_pts (ctxt->sheet,
+				pos, size_pts, cri->hard_size);
+		else
+			sheet_row_set_size_pts (ctxt->sheet,
+				pos, size_pts, cri->hard_size);
+	}
+
 	if (xml_node_get_int (tree, "Hidden", &val) && val)
-		info->visible = FALSE;
+		cri->visible = FALSE;
 	if (xml_node_get_int (tree, "Collapsed", &val) && val)
-		info->is_collapsed = TRUE;
+		cri->is_collapsed = TRUE;
 	if (xml_node_get_int (tree, "OutlineLevel", &val) && val > 0)
-		info->outline_level = val;
-	if (xml_node_get_int (tree, "Count", &count))
-		return count;
-	return 1;
+		cri->outline_level = val;
+
+	if (xml_node_get_int (tree, "Count", &val)) {
+		/* resize flags are already set only need to copy the sizes */
+		for ( ; --val > 0 ; )
+			colrow_copy (sheet_colrow_fetch (ctxt->sheet, ++pos, is_cols), cri);
+	}
 }
 
 static void
 xml_read_cols_info (XmlParseContext *ctxt, xmlNodePtr tree)
 {
 	xmlNodePtr cols, col;
-	double size_pts, tmp;
-	ColRowInfo *info;
-	int count, pos;
-	Sheet *sheet = ctxt->sheet;
+	double tmp;
 
 	cols = e_xml_get_child_by_name (tree, CC2XML ("Cols"));
 	if (cols == NULL)
 		return;
 
 	if (xml_node_get_double (cols, "DefaultSizePts", &tmp))
-		sheet_col_set_default_size_pts (sheet, tmp);
+		sheet_col_set_default_size_pts (ctxt->sheet, tmp);
 
 	for (col = cols->xmlChildrenNode; col; col = col->next)
-		if (!xmlIsBlankNode (col)) {
-			info = sheet_col_new (sheet);
-			count = xml_read_colrow_info (ctxt, col, info, &size_pts);
-			sheet_col_add (sheet, info);
-			sheet_col_set_size_pts (ctxt->sheet, info->pos, size_pts, info->hard_size);
-
-			/* resize flags are already set only need to copy the sizes */
-			for (pos = info->pos ; --count > 0 ; )
-				colrow_copy (sheet_col_fetch (ctxt->sheet, ++pos), info);
-		}
+		if (!xmlIsBlankNode (col))
+			xml_read_colrow_info (ctxt, col, TRUE);
 }
 
 static void
 xml_read_rows_info (XmlParseContext *ctxt, xmlNodePtr tree)
 {
 	xmlNodePtr rows, row;
-	double size_pts, tmp;
-	ColRowInfo *info;
-	int count, pos;
-	Sheet *sheet = ctxt->sheet;
+	double tmp;
 
 	rows = e_xml_get_child_by_name (tree, CC2XML ("Rows"));
 	if (rows == NULL)
 		return;
 
 	if (xml_node_get_double (rows, "DefaultSizePts", &tmp))
-		sheet_row_set_default_size_pts (sheet, tmp);
+		sheet_row_set_default_size_pts (ctxt->sheet, tmp);
 
 	for (row = rows->xmlChildrenNode; row; row = row->next)
-		if (!xmlIsBlankNode (row)) {
-			info = sheet_row_new (sheet);
-			count = xml_read_colrow_info (ctxt, row, info, &size_pts);
-			sheet_row_add (sheet, info);
-			sheet_row_set_size_pts (ctxt->sheet, info->pos, size_pts, info->hard_size);
-
-			/* resize flags are already set only need to copy the sizes */
-			for (pos = info->pos ; --count > 0 ; )
-				colrow_copy (sheet_row_fetch (ctxt->sheet, ++pos), info);
-		}
+		if (!xmlIsBlankNode (row))
+			xml_read_colrow_info (ctxt, row, FALSE);
 }
 
 static void
@@ -2042,11 +2013,12 @@ xml_read_clipboard_cell (XmlParseContext *ctxt, xmlNodePtr tree,
 		if (is_post_52_array) {
 			g_return_if_fail (content[0] == '=');
 
-			cc->expr = gnm_expr_parse_str (CXML2C (content), &pp,
-						   GNM_EXPR_PARSE_DEFAULT,
-						   ctxt->exprconv, NULL);
+			cc->texpr =
+				gnm_expr_parse_str (CXML2C (content), &pp,
+						    GNM_EXPR_PARSE_DEFAULT,
+						    ctxt->exprconv, NULL);
 
-			g_return_if_fail (cc->expr != NULL);
+			g_return_if_fail (cc->texpr != NULL);
 #warning TODO : arrays
 		} else if (is_value)
 			cc->val = value_new_from_string (value_type,
@@ -2055,25 +2027,28 @@ xml_read_clipboard_cell (XmlParseContext *ctxt, xmlNodePtr tree,
 			GODateConventions const *date_conv =
 				ctxt->wb ? workbook_date_conv (ctxt->wb) : NULL;
 			parse_text_value_or_expr (&pp, CXML2C (content),
-				&cc->val, &cc->expr, value_fmt, date_conv);
+				&cc->val, &cc->texpr, value_fmt, date_conv);
 		}
 
 		if (shared_expr_index > 0) {
 			if (shared_expr_index == (int)ctxt->shared_exprs->len + 1) {
-				if (NULL == cc->expr) {
+				if (NULL == cc->texpr) {
 					/* The parse failed, but we know it is
 					 * an expression.  this can happen
 					 * until we get proper interworkbook
 					 * linkages.  Force the content into
 					 * being an expression.
 					 */
-					cc->expr = gnm_expr_new_constant (value_new_string (
-						gnm_expr_char_start_p (CXML2C (content))));
+					cc->texpr =
+						gnm_expr_top_new_constant
+						(value_new_string
+						 (gnm_expr_char_start_p
+						  (CXML2C (content))));
 					value_release (cc->val);
 					cc->val = value_new_empty ();
 				}
 				g_ptr_array_add (ctxt->shared_exprs,
-						 (gpointer) cc->expr);
+						 (gpointer)cc->texpr);
 			} else {
 				g_warning ("XML-IO: Duplicate or invalid shared expression: %d",
 					   shared_expr_index);
@@ -2082,9 +2057,9 @@ xml_read_clipboard_cell (XmlParseContext *ctxt, xmlNodePtr tree,
 		xmlFree (content);
 	} else if (shared_expr_index > 0) {
 		if (shared_expr_index <= (int)ctxt->shared_exprs->len + 1) {
-			cc->expr = g_ptr_array_index (ctxt->shared_exprs,
-				shared_expr_index - 1);
-			gnm_expr_ref (cc->expr);
+			cc->texpr = g_ptr_array_index (ctxt->shared_exprs,
+						       shared_expr_index - 1);
+			gnm_expr_top_ref (cc->texpr);
 		} else {
 			g_warning ("XML-IO: Missing shared expression");
 		}
@@ -2092,7 +2067,7 @@ xml_read_clipboard_cell (XmlParseContext *ctxt, xmlNodePtr tree,
 	}
 	go_format_unref (value_fmt);
 
-	cr->content = g_slist_prepend (cr->content, cc);
+	cr->contents = g_slist_prepend (cr->contents, cc);
 }
 
 /**
@@ -2112,23 +2087,31 @@ xml_cellregion_read (WorkbookControl *wbc, Sheet *sheet, guchar const *buffer, i
 	XmlParseContext *ctxt;
 	xmlNode	   *l, *clipboard;
 	xmlDoc	   *doc;
-	GnmCellRegion *cr;
+	GnmCellRegion *cr = NULL;
 	int dummy;
+	char *old_num_locale, *old_monetary_locale;
 
 	g_return_val_if_fail (buffer != NULL, NULL);
 
+	old_num_locale = g_strdup (go_setlocale (LC_NUMERIC, NULL));
+	go_setlocale (LC_NUMERIC, "C");
+	old_monetary_locale = g_strdup (go_setlocale (LC_MONETARY, NULL));
+	go_setlocale (LC_MONETARY, "C");
+	go_set_untranslated_bools ();
+
 	doc = xmlParseDoc ((guchar *) buffer);
+
 	if (doc == NULL) {
 		go_cmd_context_error_import (GO_CMD_CONTEXT (wbc),
 			_("Unparsable xml in clipboard"));
-		return NULL;
+		goto err;
 	}
 	clipboard = doc->xmlRootNode;
 	if (clipboard == NULL || strcmp (clipboard->name, "ClipboardRange")) {
 		xmlFreeDoc (doc);
 		go_cmd_context_error_import (GO_CMD_CONTEXT (wbc),
 			_("Clipboard is in unknown format"));
-		return NULL;
+		goto err;
 	}
 
 	/* ctxt->sheet must == NULL or copying objects will break */
@@ -2140,7 +2123,7 @@ xml_cellregion_read (WorkbookControl *wbc, Sheet *sheet, guchar const *buffer, i
 	xml_node_get_int (clipboard, "BaseCol", &cr->base.col);
 	xml_node_get_int (clipboard, "BaseRow", &cr->base.row);
 	/* if it exists it is TRUE */
-	cr->not_as_content = xml_node_get_int (clipboard, "NotAsContent", &dummy);
+	cr->not_as_contents = xml_node_get_int (clipboard, "NotAsContent", &dummy);
 
 	l = e_xml_get_child_by_name (clipboard, CC2XML ("Styles"));
 	if (l != NULL)
@@ -2157,7 +2140,7 @@ xml_cellregion_read (WorkbookControl *wbc, Sheet *sheet, guchar const *buffer, i
 			if (!xmlIsBlankNode (l)) {
 				GnmRange r;
 				xmlChar *content = (char *)xmlNodeGetContent (l);
-				if (parse_range (CXML2C (content), &r))
+				if (range_parse (&r, CXML2C (content)))
 					cr->merged = g_slist_prepend (cr->merged,
 								      range_dup (&r));
 				xmlFree (content);
@@ -2179,6 +2162,13 @@ xml_cellregion_read (WorkbookControl *wbc, Sheet *sheet, guchar const *buffer, i
 	xml_parse_ctx_destroy (ctxt);
 	xmlFreeDoc (doc);
 
+ err:
+	/* go_setlocale restores bools to locale translation */
+	go_setlocale (LC_MONETARY, old_monetary_locale);
+	g_free (old_monetary_locale);
+	go_setlocale (LC_NUMERIC, old_num_locale);
+	g_free (old_num_locale);
+
 	return cr;
 }
 
@@ -2319,9 +2309,9 @@ xml_workbook_read (IOContext *context,
 	go_setlocale (LC_MONETARY, "C");
 	go_set_untranslated_bools ();
 
-	child = e_xml_get_child_by_name (tree, CC2XML ("Summary"));
+	child = e_xml_get_child_by_name (tree, CC2XML ("MetaData"));
 	if (child)
-		xml_read_summary (ctxt, child, workbook_metadata (ctxt->wb));
+		dom_read_meta_data (ctxt, child, go_doc_get_meta_data (GO_DOC (ctxt->wb)));
 
 	child = e_xml_get_child_by_name (tree, CC2XML ("DateConvention"));
 	if (child != NULL) {
@@ -2440,7 +2430,7 @@ maybe_convert (GsfInput *input, gboolean quiet)
 	GString *buffer;
 	guint ui;
 	char *converted;
-	const char *encoding;
+	char const *encoding;
 	gboolean ok;
 
 	buf = gsf_input_read (input, strlen (noencheader), NULL);
@@ -2631,7 +2621,7 @@ gnumeric_xml_read_workbook (GOFileOpener const *fo,
 	GnumericXMLVersion    version;
 	guint8 const *buf;
 	gsf_off_t size;
-	size_t len;
+	gsf_off_t len;
 
 	g_return_if_fail (input != NULL);
 
@@ -2694,7 +2684,7 @@ gnumeric_xml_read_workbook (GOFileOpener const *fo,
 	ctxt = xml_parse_ctx_new (res, gmr, wb_view);
 	ctxt->version = version;
 	xml_workbook_read (context, ctxt, res->xmlRootNode);
-	workbook_set_saveinfo (wb_view_workbook (ctxt->wb_view),
+	workbook_set_saveinfo (wb_view_get_workbook (ctxt->wb_view),
 		FILE_FL_AUTO, go_file_saver_for_id ("Gnumeric_xml_sax:xml_sax"));
 
 	xml_parse_ctx_destroy (ctxt);
@@ -2711,11 +2701,12 @@ xml_init (void)
 	xml_sax_prober.error      = (errorSAXFunc) xml_probe_problem;
 	xml_sax_prober.fatalError = (fatalErrorSAXFunc) xml_probe_problem;
 	xml_sax_prober.startElement = (startElementSAXFunc) xml_probe_start_element;
+#warning REMOVE for 2.0
 	go_file_opener_register (go_file_opener_new (
 		"Gnumeric_XmlIO:gnum_xml",
-		_("Gnumeric XML (*.gnumeric)"),
+		_("Gnumeric XML (*.gnumeric) Old slow importer"),
 		suffixes, mimes,
-		xml_probe, gnumeric_xml_read_workbook), 50);
+		xml_probe, gnumeric_xml_read_workbook), 40);
 
 	go_file_opener_register (go_file_opener_new (
 		"Gnumeric_XmlIO:xml_sax",
