@@ -65,7 +65,7 @@
 #include <gsf/gsf-input.h>
 #include <gsf/gsf-input-memory.h>
 #include <gsf/gsf-input-gzip.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <libxml/tree.h>
 #include <libxml/parser.h>
 #include <libxml/parserInternals.h>
@@ -773,36 +773,36 @@ xml_sax_colrow (GsfXMLIn *gsf_state, xmlChar const **attrs)
 
 	ColRowInfo *cri = NULL;
 	double size = -1.;
-	int dummy;
+	int pos, val;
 	int count = 1;
 	gboolean const is_col = gsf_state->node->user_data.v_bool;
 
 	g_return_if_fail (state->sheet != NULL);
 
 	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2) {
-		if (xml_sax_attr_int (attrs, "No", &dummy)) {
+		if (xml_sax_attr_int (attrs, "No", &pos)) {
 			g_return_if_fail (cri == NULL);
 
 			cri = is_col
-				? sheet_col_fetch (state->sheet, dummy)
-				: sheet_row_fetch (state->sheet, dummy);
+				? sheet_col_fetch (state->sheet, pos)
+				: sheet_row_fetch (state->sheet, pos);
 		} else {
 			g_return_if_fail (cri != NULL);
 
 			if (gnm_xml_attr_double (attrs, "Unit", &size)) ;
 			else if (xml_sax_attr_int (attrs, "Count", &count)) ;
-			else if (xml_sax_attr_int (attrs, "MarginA", &dummy))
-				cri->margin_a = dummy;
-			else if (xml_sax_attr_int (attrs, "MarginB", &dummy))
-				cri->margin_b = dummy;
-			else if (xml_sax_attr_int (attrs, "HardSize", &dummy))
-				cri->hard_size = dummy;
-			else if (xml_sax_attr_int (attrs, "Hidden", &dummy))
-				cri->visible = !dummy;
-			else if (xml_sax_attr_int (attrs, "Collapsed", &dummy))
-				cri->is_collapsed = dummy;
-			else if (xml_sax_attr_int (attrs, "OutlineLevel", &dummy))
-				cri->outline_level = dummy;
+			else if (xml_sax_attr_int (attrs, "HardSize", &val))
+				cri->hard_size = val;
+			else if (xml_sax_attr_int (attrs, "Hidden", &val))
+				cri->visible = !val;
+			else if (xml_sax_attr_int (attrs, "Collapsed", &val))
+				cri->is_collapsed = val;
+			else if (xml_sax_attr_int (attrs, "OutlineLevel", &val))
+				cri->outline_level = val;
+			else if (xml_sax_attr_int (attrs, "MarginA", &val))
+				; /* deprecated in 1.7.1 */
+			else if (xml_sax_attr_int (attrs, "MarginB", &val))
+				; /* deprecated in 1.7.1 */
 			else
 				unknown_attr (gsf_state, attrs);
 		}
@@ -811,14 +811,12 @@ xml_sax_colrow (GsfXMLIn *gsf_state, xmlChar const **attrs)
 	g_return_if_fail (cri != NULL && size > -1.);
 
 	if (is_col) {
-		int pos = cri->pos;
 		sheet_col_set_size_pts (state->sheet, pos, size, cri->hard_size);
 		/* resize flags are already set only need to copy the sizes */
 		while (--count > 0)
 			colrow_copy (sheet_col_fetch (state->sheet, ++pos), cri);
 	} else {
-		int pos = cri->pos;
-		sheet_row_set_size_pts (state->sheet, cri->pos, size, cri->hard_size);
+		sheet_row_set_size_pts (state->sheet, pos, size, cri->hard_size);
 		/* resize flags are already set only need to copy the sizes */
 		while (--count > 0)
 			colrow_copy (sheet_row_fetch (state->sheet, ++pos), cri);
@@ -943,8 +941,8 @@ xml_sax_styleregion_font (GsfXMLIn *gsf_state, xmlChar const **attrs)
 	}
 }
 
-static const char *
-font_component (const char *fontname, int idx)
+static char const *
+font_component (char const *fontname, int idx)
 {
 	int i = 0;
 	char const *p = fontname;
@@ -970,7 +968,7 @@ font_component (const char *fontname, int idx)
  * Returns: A valid style font.
  */
 static void
-style_font_read_from_x11 (GnmStyle *mstyle, const char *fontname)
+style_font_read_from_x11 (GnmStyle *mstyle, char const *fontname)
 {
 	char const *c;
 
@@ -1376,7 +1374,7 @@ xml_sax_cell_content (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 
 	cell = sheet_cell_get (state->sheet, col, row);
 	if ((is_new_cell = (cell == NULL)))
-		cell = sheet_cell_new (state->sheet, col, row);
+		cell = sheet_cell_create (state->sheet, col, row);
 
 	if (cell == NULL)
 		return;
@@ -1446,7 +1444,7 @@ xml_sax_merge (GsfXMLIn *gsf_state, G_GNUC_UNUSED GsfXMLBlob *blob)
 	GnmRange r;
 	g_return_if_fail (gsf_state->content->len > 0);
 
-	if (parse_range (gsf_state->content->str, &r))
+	if (range_parse (&r, gsf_state->content->str))
 		sheet_merge_add (state->sheet, &r, FALSE,
 			GO_CMD_CONTEXT (state->context));
 }
@@ -1506,11 +1504,11 @@ xml_sax_object_start (GsfXMLIn *gsf_state, xmlChar const **attrs)
 	}
 	state->so = so;
 
-	so->anchor.direction = SO_DIR_UNKNOWN;
+	so->anchor.base.direction = GOD_ANCHOR_DIR_UNKNOWN;
 	for (i = 0; attrs != NULL && attrs[i] && attrs[i+1] ; i += 2) {
 		if (!strcmp (attrs[i], "ObjectBound")) {
 			GnmRange r;
-			if (parse_range (attrs[i+1], &r))
+			if (range_parse (&r, attrs[i+1]))
 				so->anchor.cell_bound = r;
 		} else if (!strcmp (attrs[i], "ObjectOffset")) {
 			sscanf (attrs[i+1], "%g %g %g %g",
@@ -1523,7 +1521,7 @@ xml_sax_object_start (GsfXMLIn *gsf_state, xmlChar const **attrs)
 			for (count = 4; count-- > 0 ; )
 				so->anchor.type[count] = n[count];
 		} else if (xml_sax_attr_int (attrs+i, "Direction", &tmp_int))
-			so->anchor.direction = tmp_int;
+			so->anchor.base.direction = tmp_int;
 #if 0 /* There may be extra attributes that are handled by the objects */
 		else
 			unknown_attr (gsf_state, attrs+i);
@@ -1860,7 +1858,7 @@ maybe_convert (GsfInput *input, gboolean quiet)
 	GString *buffer;
 	guint ui;
 	char *converted;
-	const char *encoding;
+	char const *encoding;
 	gboolean ok;
 
 	buf = gsf_input_read (input, strlen (noencheader), NULL);
